@@ -1,11 +1,11 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #include <glib.h>
@@ -45,120 +44,109 @@
 #include <pan/task-save.h>
 #include <pan/util.h>
 
+
+/***
+****  Finding a match in pan.sent
+***/
+
+typedef struct
+{
+	GMimeMessage * message;
+	char * body;
+	GMimeMessage * match;
+}
+FindMatchingStruct;
+
+static void
+find_matching_sent_foreach (GMimeMessage * compare, gpointer user_data)
+{
+	char * body;
+	gboolean is_html;
+	FindMatchingStruct * data = (FindMatchingStruct*) user_data;
+
+	/* do we already have a match? */
+	if (data->match != NULL)
+		return;
+
+	/* comparison #1: subject must match */
+	if (pan_strcmp (g_mime_message_get_subject(compare), g_mime_message_get_subject(data->message)))
+		return;
+
+	/* comparison #2: author address must match */
+	if (pan_strcmp (g_mime_message_get_sender(compare), g_mime_message_get_sender(data->message)))
+		return;
+
+	/* comparison #3: body must match, if we have one */
+	body = g_mime_message_get_body (compare, TRUE, &is_html);
+	if (body!=NULL && data->body!=NULL) {
+		g_strstrip (body);
+		if (!strcmp (body, data->body)) {
+			g_object_ref (compare);
+			data->match = compare;
+			g_free (body);
+		}
+	}
+}
+
 /**
  * Find a matching article without comparing by message-id.
  * This is because many news servers alter the message-id of
  * an article being posted, so we can't rely on searching by
  * the message-id that Pan generated.
- *
- * If a match is found, the refcount of Pan.sent is incremented
- * by one and must be decremented by the caller.
  */
-static Article*
-find_matching_sent_article (const Article * article)
+static GMimeMessage*
+find_matching_sent (GMimeMessage* message)
 {
-	Group * sent;
-	Article * retval = NULL;
-	GPtrArray * articles;
-	guint i;
+	gboolean is_html;
+	FindMatchingStruct data;
 	debug_enter ("find_matching_sent_article");
 
-       	sent = serverlist_get_named_folder (PAN_SENT);
-
 	/* sanity clause */
-	g_return_val_if_fail (article_is_valid(article), NULL);
-	g_return_val_if_fail (sent!=NULL, NULL);
+	g_return_val_if_fail (GMIME_IS_MESSAGE(message), NULL);
 
-	group_ref_articles (sent, NULL);
-	articles = group_get_article_array (sent);
-
-	for (i=0; retval==NULL && i!=articles->len; ++i)
-	{
-		gchar * body_1;
-		gchar * body_2;
-		Article * compare = ARTICLE(g_ptr_array_index(articles,i));
-
-		/* comparison #1: subject must match */
-		if (pan_strcmp (article_get_subject(compare), article_get_subject(article)))
-			continue;
-
-		/* comparison #2: author address must match */
-		if (pan_strcmp (compare->author_addr, article->author_addr))
-			continue;
-
-		/* comparison #3: body address must match */
-		body_1 = article_get_body (article);
-		body_2 = article_get_body (compare);
-		if (body_1!=NULL && body_2!=NULL)
-		{
-			g_strstrip (body_1);
-			g_strstrip (body_2);
-			if (!strcmp (body_1, body_2))
-				retval = compare;
-		}
-		else
-		if (body_1==NULL)
-		{
-			/* article's body is NULL, meaning it hasn't been
-			   read yet. Risk it and cancel this one */
-			retval = compare;
-		}
-		g_free (body_1);
-		g_free (body_2);
-	}
-
-	if (retval == NULL)
-		group_unref_articles (sent, NULL);
-
-	debug_exit ("find_matching_sent_article");
-	g_ptr_array_free (articles, TRUE);
-	return retval;
+	/* find the match */
+	data.message = message;
+	data.body = g_strstrip (g_mime_message_get_body (data.message, TRUE, &is_html));
+	data.match = NULL;
+	acache_path_foreach (PAN_SENT, find_matching_sent_foreach, &data);
+	g_free (data.body);
+	return data.match;
 }
 
-static gint
+/***
+****
+***/
+
+static int
 article_cancel_run (TaskFuncPtr* task, gpointer user_data)
 {
-	Article * article = ARTICLE(user_data);
-        int       status = nntp_cancel (STATUS_ITEM(task),
-			                article,
-	                                TASK(task)->sock);
+	GMimeMessage * message = GMIME_MESSAGE (user_data);
+	const char * message_id = g_mime_message_get_message_id (message);
+        int status = nntp_cancel (STATUS_ITEM(task), message_id, TASK(task)->sock);
 
 	if (status == TASK_SUCCESS)
 	{
-		gchar        * subject = g_strdup (article_get_subject (article));
-		gchar        * message_id = g_strdup (article_get_message_id (article));
+		const char * subject = g_mime_message_get_subject (message);
+		const char * newsgroups = g_mime_message_get_header (message, HEADER_NEWSGROUPS);
+		char ** groups = g_strsplit (newsgroups, ",", 0);
+		guint i;
 
-#if 0
-		/* FIXME: this doesn't seem to work: group_get_article_by_message_id */
-		/* doesn't return the article. Not sure why.                         */
-
-		const gchar  * newsgroups = article_get_extra_header (article, HEADER_NEWSGROUPS);
-		gchar       ** groups = g_strsplit (newsgroups, ",", 0);
-		guint          i;
-
-		for (i=0; groups[i]; i++) {
-
-			Group     * group = server_get_named_group (TASK(task)->server, groups[i]);
-		      
+		for (i=0; groups[i]; ++i)
+		{
+			Group * group = server_get_named_group (TASK(task)->server, groups[i]);
 			group_ref_articles (group, STATUS_ITEM(task));
-
-			article = (Article *) group_get_article_by_message_id (group, message_id);
-			if (article) {
-				group_remove_article (group, article);
+			{
+				Article * article = ARTICLE(group_get_article_by_message_id (group, message_id));
+				if (article)
+					group_remove_article (group, article);
 			}
-
 			group_unref_articles (group, STATUS_ITEM(task));
 		}
 
 		g_strfreev (groups);
 
-#endif
-		log_add_va (LOG_INFO|LOG_URGENT, _("Article %s (%s) cancelled"),
+		log_add_va (LOG_INFO|LOG_URGENT, _("Article %s (%s) canceled"),
 			subject, message_id);
-
-		g_free (subject);
-		g_free (message_id);
-
 	}
 
 	return 0;
@@ -166,29 +154,23 @@ article_cancel_run (TaskFuncPtr* task, gpointer user_data)
 static gchar*
 article_cancel_describe (const StatusItem * status)
 {
-	return g_strdup (_("Cancelling article"));
+	return g_strdup (_("Canceling article"));
 }
 static void
-article_cancel_dtor (gpointer data)
+article_cancel_dtor (gpointer message)
 {
 	debug_enter ("article_cancel_dtor");
 
-	group_unref_articles (serverlist_get_named_folder(PAN_SENT), NULL);
+	g_object_unref (message);
 
 	debug_exit ("article_cancel_dtor");
 }
 void
-article_cancel (Article * article)
+article_cancel (Server * server, GMimeMessage * message)
 {
-	Server * server;
-	Article * ours;
-
-	/* sanity clause */
-	g_return_if_fail (article != NULL);
-	server = article->group==NULL ? NULL : article->group->server;
-	g_return_if_fail (server != NULL);
-
-	ours = find_matching_sent_article (article);
+	GMimeMessage * ours;
+	
+	ours = find_matching_sent (message);
 	if (ours == NULL)
 	{
 		log_add_va (LOG_ERROR|LOG_URGENT,
@@ -197,10 +179,10 @@ article_cancel (Article * article)
 	else
 	{
 		queue_add (TASK(task_func_ptr_new (server,
-			article_cancel_describe,
-			article_cancel_run,
-			article_cancel_dtor, 
-			ours, FALSE)));
+		                article_cancel_describe,
+		                article_cancel_run,
+		                article_cancel_dtor, 
+		                ours, FALSE)));
 	}
 }
 
@@ -209,20 +191,16 @@ article_cancel (Article * article)
 ****/
 
 void
-article_supersede (const Article * article)
+article_supersede (GMimeMessage * message)
 {
-	const Article * ours;
-	Group * sendlater;
+	GMimeMessage * ours;
 	debug_enter ("article_supersede");
 
-       	sendlater = serverlist_get_named_folder (PAN_SENDLATER);
-
 	/* sanity clause */
-	g_return_if_fail (article_is_valid(article));
-	g_return_if_fail (sendlater != NULL);
+	g_return_if_fail (GMIME_IS_MESSAGE (message));
 
-	/* find the original copy so that the headers will match */
-	ours = find_matching_sent_article (article);
+	/* find the original copy so that the headers will match. */
+	ours = find_matching_sent (message);
 	if (ours == NULL)
 	{
 		log_add_va (LOG_ERROR|LOG_URGENT,
@@ -230,18 +208,26 @@ article_supersede (const Article * article)
 	}
 	else
 	{
-		gchar * message_id = gnksa_generate_message_id_from_email_addr (ours->author_addr);
-		Article * super = article_dup (sendlater, ours);
-		article_set_header (super, HEADER_MESSAGE_ID, message_id, DO_CHUNK);
-		article_set_header (super, HEADER_SUPERSEDES, article_get_message_id(article), DO_CHUNK);
+		const char * old_message_id;
+		const char * domain;
+		char * new_message_id;
 
-		/* edit the copy */
-		message_edit_window (super);
+		/* get the new message-id */
+		old_message_id = g_mime_message_get_message_id (ours);
+		domain = strchr (old_message_id, '@');
+		if (domain)
+			++domain;
+		new_message_id = gnksa_generate_message_id (domain);
+
+		/* update the headers for superseding */
+		g_mime_message_set_header (ours, HEADER_SUPERSEDES, old_message_id);
+		g_mime_message_set_header (ours, HEADER_MESSAGE_ID, new_message_id);
+
+		/* edit the message */
+		message_edit_window (ours);
 
 		/* cleanup */
-		g_free (message_id);
-		group_unref_articles (ours->group, NULL);
-		/* FIXME: super dangles */
+		g_free (new_message_id);
 	}
 
 	debug_exit ("article_supersede");
@@ -254,66 +240,44 @@ article_supersede (const Article * article)
 static void
 article_copy_articles_to_folder_cb (gpointer obj, gpointer arg, gpointer data)
 {
-	gint status;
-	TaskBodies * task;
-	GPtrArray * articles;
+	int status;
+	Task * task;
 	Group * folder;
 	debug_enter ("article_copy_articles_to_folder_cb");
 
 	/* sanity clause */
 	g_return_if_fail (obj!=NULL);
 	g_return_if_fail (data!=NULL);
-	g_return_if_fail (group_is_folder(GROUP(data)));
+	folder = (Group*) data;
+	g_return_if_fail (group_is_valid(folder));
+	g_return_if_fail (group_is_folder(folder));
 
+	/* pump argument list */
 	status = GPOINTER_TO_INT(arg);
-	task = TASK_BODIES(obj);
-	articles = task->articles;
-	folder = GROUP(data);
+	task = TASK(obj);
 
+	/* copy them over if the task succeeded ... */
 	if (status == TASK_SUCCESS)
 	{
-		gint i;
-		GPtrArray * addme;
-
-		/* init */
-		group_ref_articles (folder, NULL);
-
-		/* create an array of articles to add. */
-		addme = g_ptr_array_new ();
-		pan_g_ptr_array_reserve (addme, articles->len);
-		for (i=0; i!=articles->len; ++i)
+		guint i;
+		for (i=0; i<task->identifiers->len; ++i)
 		{
-			const Article * a_old;
-			Article * a_new;
-			gchar * body;
-
-			a_old = ARTICLE(g_ptr_array_index(articles,i));
-			body = article_get_body (a_old);
-			a_new = article_dup (folder, a_old);
-			article_set_header (a_new, PAN_BODY, body, DO_CHUNK);
-			g_ptr_array_add (addme, a_new);
-
-			g_free (body);
+			MessageIdentifier * mid = MESSAGE_IDENTIFIER (g_ptr_array_index (task->identifiers, i));
+			GMimeMessage * message = acache_get_message (&mid, 1);
+			folder_add_message (folder, message);
+			g_object_unref (message);
 		}
 
-		/* smoke 'em if you've got 'em */
-		if (addme->len != 0)
-			group_add_articles (folder, addme, NULL, NULL, NULL);
-
 		/* clean up */
-		group_unref_articles (folder, NULL);
-		g_ptr_array_free (addme, TRUE);
 	}
 
 	debug_exit ("articlelist_article_to_folder_cb");
 }
 
 gboolean
-article_copy_articles_to_folder (Group * folder, Article ** articles, gint qty)
+article_copy_articles_to_folder (Group * folder, const Article ** articles, int qty)
 {
-	gint i;
 	Task * task;
-	GPtrArray * a;
 	debug_enter ("article_copy_articles_to_folder");
 
 	/* sanity clause */
@@ -321,14 +285,10 @@ article_copy_articles_to_folder (Group * folder, Article ** articles, gint qty)
 	g_return_val_if_fail (group_is_folder(folder), FALSE);
 	g_return_val_if_fail (articles!=NULL, FALSE);
 	g_return_val_if_fail (qty>0, FALSE);
-	for (i=0; i<qty; ++i)
-		g_return_val_if_fail (article_is_valid(articles[i]), FALSE);
+	g_return_val_if_fail (articles_are_valid(articles, qty), FALSE);
 
-	/* make a task to download the bodies if we don't already have them */
-	a = g_ptr_array_new ();
-	pan_g_ptr_array_assign (a, (gpointer*)articles, qty);
-	task = TASK(task_bodies_new (articles[0]->group, a));
-	g_ptr_array_free (a, TRUE);
+	/* make a task to download the bodies in case we don't already have them */
+	task = TASK(task_bodies_new_from_articles (articles, qty));
 	if (task != NULL) {
 		pan_callback_add (task->task_ran_callback, article_copy_articles_to_folder_cb, folder);
 		queue_add (task);
@@ -347,120 +307,60 @@ article_copy_articles_to_folder (Group * folder, Article ** articles, gint qty)
 ***/
 
 static void
-open_article (const Article * a)
+save_attachments_impl (Article ** articles, int len)
 {
-	gchar * file;
-	const gchar * files;
+	int i;
+	GSList * tasks = NULL;
+	debug_enter ("save_attachments_impl");
 
-	g_return_if_fail (article_is_valid(a));
+	g_return_if_fail (articles_are_valid ((const Article**)articles, len));
 
-	files = article_get_extra_header (a, PAN_HEADER_FILENAME);
-	while ((file = get_next_token_str (files, '\n', &files))) {
-		open_outside_file (file);
-		g_free (file);
-	}
+	for (i=0; i<len; ++i)
+	{
+		GSList * l;
+		Article * a = articles[i];
+		PanObject * task;
+		GPtrArray * tmp = g_ptr_array_sized_new (64);
 
-}
+		/* build an array of this article and all its children. */
+		g_ptr_array_add (tmp, a);
+		for (l=a->threads; l!=NULL; l=l->next)
+			g_ptr_array_add (tmp, l->data);
 
-static void
-open_decoded_cb (gpointer call_obj, gpointer call_arg, gpointer user_data)
-{
-	gint status = GPOINTER_TO_INT(call_arg);
-
-	if (status == TASK_SUCCESS) {
-		TaskSave * task = TASK_SAVE(call_obj);
-		if (task!=NULL && task->articles!=NULL) {
-			const Article * a = ARTICLE(task->articles->data);
-			if (article_is_valid(a))
-				open_article (a);
-		}
-	}
-}
-
-static void
-decode_impl (Article * a,
-             gboolean save_bodies,
-             gboolean save_attachments,
-             gboolean open_attachments)
-{
-	Task * task;
-	GSList * articles;
-	const gchar * files;
-	debug_enter ("decode_impl");
-
-	g_return_if_fail (article_is_valid(a));
-	g_return_if_fail (save_bodies || save_attachments || open_attachments);
-
-	/* got to save before you can open */
-	if (open_attachments)
-		save_attachments = TRUE;
-
-	/* if user is opening the files & we already have them... */
-	files = article_get_extra_header (a, PAN_HEADER_FILENAME);
-	if (open_attachments && is_nonempty_string(files)) {
-		open_article (a);
-		debug_exit ("decode_impl");
-		return;
-	}
-
-	/* build an array of this article and all its children. */
-	articles = g_slist_copy (a->threads);
-	articles = g_slist_prepend (articles, a);
-
-	/* queue for decode */
-	task = TASK(task_save_new (a->group, article_get_subject(a), articles));
-	if (save_bodies)
-		task_save_set_bodies (TASK_SAVE(task), NULL, NULL);
-	if (save_attachments)
+		/* queue for decode */
+		task_save_weed_duplicates (tmp);
+		task = task_save_new_from_articles (article_get_subject(a), (const Article**)tmp->pdata, tmp->len);
 		task_save_set_attachments (TASK_SAVE(task), NULL, NULL);
-	if (open_attachments)
-		pan_callback_add (task->task_ran_callback, open_decoded_cb, NULL);
-	task_save_validate_and_queue_or_self_destruct (TASK_SAVE(task), -1);
+		tasks = g_slist_prepend (tasks, task);
+
+		/* cleanup */
+		g_ptr_array_free (tmp, TRUE);
+	}
+
+	if (tasks != NULL)
+	{
+		tasks = g_slist_reverse (tasks);
+		queue_insert_tasks (tasks, -1);
+		tasks = NULL;
+	}
 
 	/* cleanup this loop */
-	g_slist_free (articles);
-	debug_exit ("decode_impl");
-}
-
-static void
-seleted_decode_impl (gboolean save_bodies,
-                     gboolean save_attachments,
-                     gboolean open_attachments)
-{
-	guint i;
-	GPtrArray * sel;
-	debug_enter ("seleted_decode_impl");
-
-	/* decode each selected article */
-	sel = articlelist_get_selected_articles_nolock ();
-	for (i=0; i<sel->len; ++i)
-		decode_impl (ARTICLE(g_ptr_array_index(sel,i)),
-		             save_bodies,
-		             save_attachments,
-		             open_attachments);
-	g_ptr_array_free (sel, TRUE);
-
-	debug_exit ("seleted_decode_impl");
-}
-
-void
-article_action_selected_save (void)
-{
-	seleted_decode_impl (TRUE, TRUE, FALSE);
+	debug_exit ("save_attachments_impl");
 }
 
 void
 article_action_selected_save_attachments (void)
-{		
-	seleted_decode_impl (FALSE, TRUE, FALSE);
-}
-
-void
-article_action_selected_open_attachments (void)
 {
-	seleted_decode_impl (FALSE, TRUE, TRUE);
-}
+	GPtrArray * sel;
+	debug_enter ("article_action_selected_save_attachments");
 
+	/* decode each selected article */
+	sel = articlelist_get_selected_articles_nolock ();
+	save_attachments_impl ((Article**)sel->pdata, sel->len);
+	g_ptr_array_free (sel, TRUE);
+
+	debug_exit ("article_action_selected_save_attachments");
+}
 
 void
 article_action_selected_save_as (void)
@@ -477,11 +377,12 @@ article_action_selected_save_as (void)
 	sel = articlelist_get_selected_articles_nolock ();
 	for (i=0; i!=sel->len; ++i)
 	{
-		GSList * l;
+		GSList * l = NULL;
 		Article * article = ARTICLE(g_ptr_array_index(sel,i));
 
 		/* build an array of this node and all its children. */
-		l = g_slist_copy (article->threads);
+		if (article->parts > 1)
+			l = g_slist_copy (article->threads);
 		l = g_slist_prepend (l, article);
 
 		/* add this to the gslist of articles */
@@ -507,7 +408,7 @@ article_action_selected_save_as (void)
 static void
 remove_duplicate_articles (GPtrArray * articles)
 {
-	gint i;
+	int i;
 	GHashTable * hash;
 
 	/* sanity claus */
@@ -550,9 +451,10 @@ articlelist_remove_xrefs_ghfunc (gpointer key,
  * GArrays with the article numbers of this crosspost.
  */
 static void
-articlelist_remove_xrefs_xreffunc (Group   * group,
-                                   gulong    number,
-                                   gpointer  data)
+articlelist_remove_xrefs_xreffunc (Server   * server,
+                                   Group    * group,
+                                   gulong     number,
+                                   gpointer   data)
 {
 	GHashTable * hash = (GHashTable*) data;
 
@@ -614,30 +516,14 @@ delete_articles (GPtrArray * articles)
 		group_remove_articles (group, articles);
 
 		/* log */
-		log_add_va (LOG_INFO, _("Deleted %u articles from `%s'"),
+		log_add_va (LOG_INFO, _("Deleted %u articles from \"%s\""),
 		            articles->len,
-		            group_get_readable_name(group));
+		            group_get_name(group));
 	}
 
 	debug_exit ("article_action_delete_articles");
 }
 
-void
-article_action_delete_selected_threads (void)
-{
-	GPtrArray * sel;
-	GPtrArray * all;
-	debug_enter ("article_action_delete_selected_threads");
-
-       	sel = articlelist_get_selected_articles_nolock ();
-       	all = article_get_unique_threads (sel, GET_WHOLE_THREAD);
-	delete_articles (all);
-	g_ptr_array_free (all, TRUE);
-	g_ptr_array_free (sel, TRUE);
-
-	debug_exit ("article_action_delete_selected_threads");
-}
-
 void
 article_action_delete_selected_articles (void)
 {
@@ -668,12 +554,19 @@ article_action_delete_selected_articles (void)
 void
 article_action_edit_selected (void)
 {
-        Article * a;
+        Article * a = NULL;
+	MessageIdentifier * mid = NULL;
+	GMimeMessage * message = NULL;
 	debug_enter ("article_action_edit_selected");
 
 	a = articlelist_get_selected_article_nolock ();
 	if (a != NULL)
-		message_edit_window (a);
+		mid = message_identifier_new_from_article (a);
+	if (mid != NULL)
+		message = acache_get_message (&mid, 1);
+	if (message != NULL)
+		message_edit_window (message);
 
+	g_object_unref (mid);
 	debug_exit ("article_action_edit_selected");
 }
