@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,13 +15,13 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #include <config.h>
 #include <string.h>
 
-#include <libgnomeui/libgnomeui.h>
+#include <glib.h>
+#include <gtk/gtk.h>
 
 #include <pan/base/article.h>
 #include <pan/base/fnmatch.h>
@@ -36,6 +34,12 @@
 #include <pan/gui.h>
 #include <pan/util.h>
 
+enum
+{
+	PAN_RESPONSE_FIND_NEXT = 1,
+	PAN_RESPONSE_FIND_PREV = 2
+};
+
 static GtkWidget *find_dialog;
 
 static void
@@ -53,9 +57,9 @@ typedef struct
 	gboolean unread_only;
 	gboolean case_sensitive;
 
-	const gchar *author;
-	const gchar *subject;
-	const gchar *message_id;
+	const char *author;
+	const char *subject;
+	const char *message_id;
 } 
 header_compare_data;
 
@@ -94,16 +98,16 @@ compare_article (const Article* article,
 }
 
 static void
-find_article (const gchar* author,
-              const gchar* subject,
-              const gchar* message_id,
+find_article (const char* author,
+              const char* subject,
+              const char* message_id,
               gboolean case_sensitive,
               gboolean unread_only,
               gboolean reverse)
 {
-	gchar * author_tmp;
-	gchar * subject_tmp;
-	gchar * message_id_tmp;
+	char * author_tmp;
+	char * subject_tmp;
+	char * message_id_tmp;
 	GtkCTreeNode * sel;
 	GtkCTreeNode * tmp;
 	GtkCTree* tree = GTK_CTREE(Pan.article_ctree);
@@ -189,38 +193,31 @@ find_article (const gchar* author,
 	g_free (subject_tmp);
 }
 
-static gchar * find_article__author = NULL;
-static gchar * find_article__subject = NULL;
-static gchar * find_article__message_id = NULL;
+static char * find_article__author = NULL;
+static char * find_article__subject = NULL;
+static char * find_article__message_id = NULL;
 static gboolean find_article__case_sensitive = FALSE;
 static gboolean find_article__unread_only = FALSE;
 static gboolean find_article__reverse = FALSE;
 
 
 static void
-find_dialog_clicked_cb (GnomeDialog    * dialog,
-                        int              button,
-                        GtkWidget      * table)
+find_dialog_response_cb (GtkDialog     * dialog,
+                         int             response,
+                         GtkWidget     * table)
 {
- 	g_return_if_fail (button==0 || button==1 || button==2);
 	g_return_if_fail (table!=NULL);
 
-	if (button==2) /* "close" button */
-	{
-		pan_lock();
-		gtk_widget_unmap (GTK_WIDGET(find_dialog));
-		pan_unlock();
-	}
-	else if (button==0 || button==1) /* "find next" button */
+	if (response==PAN_RESPONSE_FIND_PREV || response==PAN_RESPONSE_FIND_NEXT)
 	{
 		GtkWidget * author_entry;
 		GtkWidget * subject_entry;
 		GtkWidget * message_id_entry;
 		GtkWidget * case_sensitive_checkbox;
 		GtkWidget * unread_only_checkbox;
-		gchar * author;
-		gchar * subject;
-		gchar * message_id;
+		char * author;
+		char * subject;
+		char * message_id;
 		gboolean case_sensitive;
 		gboolean unread_only;
 
@@ -240,9 +237,9 @@ find_dialog_clicked_cb (GnomeDialog    * dialog,
 		if (!*subject)
 			replace_gstr (&subject, NULL);
 
-		/* get message-id to search for, or NULL if none... */
+		/* get message-id to search for, or NULL if none...*/
 		message_id_entry = (GtkWidget*) gtk_object_get_data ( GTK_OBJECT(find_dialog), "message-id" );
-		g_return_if_fail ( message_id_entry!=NULL );
+		g_return_if_fail ( message_id_entry != NULL );
 		message_id = g_strdup ( gtk_entry_get_text ( GTK_ENTRY(message_id_entry) ) );
 		g_strstrip (message_id);
 		if (!*message_id)
@@ -265,9 +262,13 @@ find_dialog_clicked_cb (GnomeDialog    * dialog,
 		find_article__case_sensitive = case_sensitive;
 		find_article__unread_only = unread_only;
 
-		find_article__reverse = button==0;
+		find_article__reverse = response == PAN_RESPONSE_FIND_PREV;
 		articlelist_find_next_cb ( );
 	}
+	else
+	{
+		gtk_widget_unmap (GTK_WIDGET(find_dialog));
+	}
 }
 
 void
@@ -283,7 +284,7 @@ articlelist_find_next_cb (void)
 		return;
 	}
 
-	/* got some criteria .. let's go find 'em */
+	/* got a subject and/or an author.. let's go find 'em */
 	find_article (find_article__author,
 	              find_article__subject,
 	              find_article__message_id,
@@ -298,12 +299,10 @@ articlelist_find_next_cb (void)
 void
 articlelist_find_text_cb (void)
 {
-	GtkWidget * table;
-	GtkWidget * author_label;
+	GtkWidget * t;
+	GtkWidget * w;
 	GtkWidget * author_entry;
-	GtkWidget * subject_label;
 	GtkWidget * subject_entry;
-	GtkWidget * message_id_label;
 	GtkWidget * message_id_entry;
 	GtkWidget * case_sensitive_checkbutton;
 	GtkWidget * unread_only_checkbutton;
@@ -312,120 +311,79 @@ articlelist_find_text_cb (void)
 	/* There can be only one! (find window) */
 	if (find_dialog != NULL)
 	{
-		gnome_dialog_set_parent (GNOME_DIALOG(find_dialog),
-		                         GTK_WINDOW(Pan.window));
 		gtk_widget_show_all (find_dialog);
-		gtk_widget_map (GTK_WIDGET(find_dialog));
+		gtk_widget_map (find_dialog);
 		return;
 	}
-	
-	find_dialog = gnome_dialog_new (_("Pan: Find Message"),
-					GNOME_STOCK_PIXMAP_BACK,
-					GNOME_STOCK_PIXMAP_FORWARD,
-					GNOME_STOCK_PIXMAP_CLOSE, NULL);
-	gnome_dialog_set_default ( GNOME_DIALOG(find_dialog), 0 );
 
-	table = gtk_table_new (5, 2, FALSE);
+	find_dialog = gtk_dialog_new_with_buttons (_("Pan: Find Message"),
+	                                           GTK_WINDOW(Pan.window),
+	                                           GTK_DIALOG_DESTROY_WITH_PARENT,
+	                                           GTK_STOCK_GO_BACK, PAN_RESPONSE_FIND_PREV,
+	                                           GTK_STOCK_GO_FORWARD, PAN_RESPONSE_FIND_NEXT,
+	                                           GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
+	                                           NULL);
 
 	row = 0;
-	subject_label = gtk_label_new ( _("Subject:") );
-	gtk_table_attach ( GTK_TABLE(table),
-			   subject_label,
-			   0, 1,
-			   row, row+1,
-			   0,
-			   0,
-			   GNOME_PAD_SMALL, GNOME_PAD_SMALL );
+	t = gtk_table_new (6, 2, FALSE);
+	gtk_container_set_border_width (GTK_CONTAINER (t), 18);
+	gtk_table_set_row_spacings (GTK_TABLE(t), 6);
+
+	w = gtk_label_new_with_mnemonic (_("_Subject:"));
+	gtk_misc_set_alignment (GTK_MISC(w), 0.0f, 0.5f);
+	gtk_label_set_justify (GTK_LABEL(w), GTK_JUSTIFY_LEFT);
+	gtk_table_attach_defaults (GTK_TABLE(t), w, 0, 1, row, row+1);
 
 	subject_entry = gtk_entry_new ( );
+	gtk_label_set_mnemonic_widget (GTK_LABEL(w), subject_entry);
 	gtk_object_set_data (GTK_OBJECT(find_dialog), "subject", subject_entry);
-	gtk_table_attach (GTK_TABLE(table),
-			  subject_entry,
-			  1, 2,
-			  row, row+1,
-			  GTK_EXPAND|GTK_FILL|GTK_SHRINK,
-			  0,
-			  GNOME_PAD_SMALL, GNOME_PAD_SMALL );
-
+	gtk_table_attach_defaults (GTK_TABLE(t), subject_entry, 1, 2, row, row+1);
 	++row;
-	author_label = gtk_label_new (_("Author:"));
-	gtk_table_attach (GTK_TABLE(table),
-			  author_label,
-			  0, 1,
-			  row, row+1,
-			  0,
-			  0,
-			  GNOME_PAD_SMALL, GNOME_PAD_SMALL );
+
+	w = gtk_label_new_with_mnemonic (_("_Author:"));
+	gtk_misc_set_alignment (GTK_MISC(w), 0.0f, 0.5f);
+	gtk_label_set_justify (GTK_LABEL(w), GTK_JUSTIFY_LEFT);
+	gtk_table_attach_defaults (GTK_TABLE(t), w, 0, 1, row, row+1);
 
 	author_entry = gtk_entry_new ( );
-	gtk_object_set_data ( GTK_OBJECT(find_dialog), "author", author_entry );
-	gtk_table_attach (GTK_TABLE(table),
-			  author_entry,
-			  1, 2,
-			  row, row+1,
-			  GTK_EXPAND|GTK_FILL|GTK_SHRINK,
-			  0,
-			  GNOME_PAD_SMALL, GNOME_PAD_SMALL );
-
+	gtk_label_set_mnemonic_widget (GTK_LABEL(w), author_entry);
+	gtk_object_set_data (GTK_OBJECT(find_dialog), "author", author_entry);
+	gtk_table_attach_defaults (GTK_TABLE(t), author_entry, 1, 2, row, row+1);
 	++row;
-	message_id_label = gtk_label_new (_("Message-ID:"));
-	gtk_table_attach (GTK_TABLE(table),
-			  message_id_label,
-			  0, 1,
-			  row, row+1,
-			  0,
-			  0,
-			  GNOME_PAD_SMALL, GNOME_PAD_SMALL );
 
-	message_id_entry = gtk_entry_new ( );
+	w = gtk_label_new_with_mnemonic (_("_Message-ID:"));
+	gtk_misc_set_alignment (GTK_MISC(w), 0.0f, 0.5f);
+	gtk_label_set_justify (GTK_LABEL(w), GTK_JUSTIFY_LEFT);
+	gtk_table_attach_defaults (GTK_TABLE(t), w, 0, 1, row, row+1);
+
+	message_id_entry = gtk_entry_new ();
+	gtk_label_set_mnemonic_widget (GTK_LABEL(w), message_id_entry);
 	gtk_object_set_data ( GTK_OBJECT(find_dialog), "message-id", message_id_entry );
-	gtk_table_attach (GTK_TABLE(table),
-			  message_id_entry,
-			  1, 2,
-			  row, row+1,
-			  GTK_EXPAND|GTK_FILL|GTK_SHRINK,
-			  0,
-			  GNOME_PAD_SMALL, GNOME_PAD_SMALL );
-
+	gtk_table_attach_defaults (GTK_TABLE(t), message_id_entry, 1, 2, row, row+1);
 	++row;
-	unread_only_checkbutton =
-		gtk_check_button_new_with_label (_("Unread Only"));
-	gtk_toggle_button_set_active (
-		GTK_TOGGLE_BUTTON(unread_only_checkbutton), FALSE);
-	gtk_object_set_data (GTK_OBJECT(find_dialog),
-		"unread", unread_only_checkbutton );
-	gtk_table_attach ( GTK_TABLE(table),
-			   unread_only_checkbutton,
-			   0, 2,
-			   row, row+1,
-			   GTK_EXPAND | GTK_FILL | GTK_SHRINK,
-			   GTK_EXPAND | GTK_FILL,
-			   GNOME_PAD_SMALL, GNOME_PAD_SMALL );
 
+	w = gtk_alignment_new (0.0f, 0.0f, 0.0f, 0.0f);
+	gtk_widget_set_usize (w, 0u, 12u);
+	gtk_table_attach_defaults (GTK_TABLE(t), w, 0, 2, row, row+1);
 	++row;
-	case_sensitive_checkbutton = gtk_check_button_new_with_label ( _("Case Sensitive") );
-	gtk_toggle_button_set_active ( GTK_TOGGLE_BUTTON(case_sensitive_checkbutton), FALSE );
-	gtk_object_set_data ( GTK_OBJECT(find_dialog), "case", case_sensitive_checkbutton );
-	gtk_table_attach ( GTK_TABLE(table),
-			   case_sensitive_checkbutton,
-			   0, 2,
-			   row, row+1,
-			   GTK_EXPAND | GTK_FILL | GTK_SHRINK,
-			   GTK_EXPAND | GTK_FILL,
-			   GNOME_PAD_SMALL, GNOME_PAD_SMALL );
 
+	case_sensitive_checkbutton = gtk_check_button_new_with_mnemonic (_("Case Sensiti_ve"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(case_sensitive_checkbutton), FALSE);
+	gtk_object_set_data (GTK_OBJECT(find_dialog), "case", case_sensitive_checkbutton);
+	gtk_table_attach_defaults (GTK_TABLE(t), case_sensitive_checkbutton, 0, 2, row, row+1);
+	++row;
+
+	unread_only_checkbutton = gtk_check_button_new_with_mnemonic (_("_Unread Only"));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(unread_only_checkbutton), FALSE);
+	gtk_object_set_data (GTK_OBJECT(find_dialog), "unread", unread_only_checkbutton );
+	gtk_table_attach_defaults (GTK_TABLE(t), unread_only_checkbutton, 0, 2, row, row+1);
+	++row;
 	
-	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (find_dialog)->vbox), table, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (GTK_DIALOG(find_dialog)->vbox), t, FALSE, FALSE, 0);
 	
-	gtk_signal_connect (GTK_OBJECT(find_dialog), "clicked",
-			    GTK_SIGNAL_FUNC(find_dialog_clicked_cb),
-			    table);
-	gtk_signal_connect (GTK_OBJECT(find_dialog), "destroy",
-			    GTK_SIGNAL_FUNC(find_dialog_destroy_cb), NULL);
+	g_signal_connect (find_dialog, "response", G_CALLBACK(find_dialog_response_cb), t);
+	g_signal_connect (find_dialog, "destroy", G_CALLBACK(find_dialog_destroy_cb), NULL);
 
 	gtk_widget_grab_focus (subject_entry);
-
-	gnome_dialog_set_parent (GNOME_DIALOG(find_dialog),
-	                         GTK_WINDOW(Pan.window));
 	gtk_widget_show_all (find_dialog);
 }
