@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #include <config.h>
@@ -25,9 +22,11 @@
 #include <glib.h>
 
 #include <ctype.h>
+#include <limits.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include <pan/base/acache.h>
 #include <pan/base/article.h>
 #include <pan/base/article-thread.h>
 #include <pan/base/base-prefs.h>
@@ -66,36 +65,53 @@ Norm;
  * When we're threading articles, it's a big speedup to normalize the
  * subjects at the outset instead of normalizing them in each comparison.
  */
-static gchar*
-normalize_subject (gchar * buf, const Article * a)
+static size_t
+normalize_subject (char * buf, const Article * a)
 {
+	static gboolean _inited = FALSE;
+	static gboolean _keep_chars[UCHAR_MAX+1];
+	register const guchar * in = (guchar*) skip_reply_leader (a->subject);
+	register char * out = buf;
 	const gboolean multipart = a->parts != 0;
-	const gchar * in = skip_reply_leader (a->subject);
-	gchar * out = buf;
+
+	/* populate the _keep_chars array */
+	if (!_inited) {
+		int i;
+		for (i=0; i<=UCHAR_MAX; ++i) {
+			guchar uch = (guchar)i;
+			_keep_chars[i] = isalnum(uch) || isdigit(uch) || isspace(uch);
+		}
+		_inited = TRUE;
+	}
 
 	/* skip the leading noise */
-	while (*in && !isalnum((guchar)*in) && !isdigit((guchar)*in))
+	while (*in && !isalnum(*in))
 		++in;
 
 	while (*in)
 	{
 		/* strip multipart information */
-		if (multipart && (*in=='('||*in=='[') && isdigit((guchar)in[1])) {
-			const char ch = *in=='(' ? ')' : ']';
-			while (*in && *in!=ch)
+		if (multipart && (*in=='('||*in=='[') && isdigit(in[1])) {
+			const guchar * start = in;
+			while (*in && *in!=']' && *in!=')') {
+				if (isalpha(*in)) { /* oops, not multipart information */
+					in = ++start;
+					break;
+				} 
 				++in;
+			}
 			continue;
 		}
 
-		/* strip out junk that breaks sorting  */
-		if (isalnum((guchar)*in) || isdigit((guchar)*in) || isspace((guchar)*in))
-			*out++ = ('A'<=*in && *in<='Z') ? tolower(*in) : *in;
+		/* strip out junk that breaks sorting */
+		if (_keep_chars[*in])
+			*out++ = tolower(*in);
 
 		++in;
 	}
 
 	*out = '\0';
-	return buf;
+	return out - buf;
 }
 
 
@@ -137,8 +153,8 @@ normalize_articles (Article    ** articles,
 		Article * a = articles[i];
 		norm_buf[i].a = a;
 		norm_buf[i].is_reply = skip_reply_leader (a->subject) != a->subject ? 1 : 0;
-		norm_buf[i].subject = normalize_subject (str_buf+str_buf_idx, a);
-		str_buf_idx += strlen(norm_buf[i].subject) + 1;
+		norm_buf[i].subject = str_buf + str_buf_idx;
+		str_buf_idx += normalize_subject (norm_buf[i].subject, a) + 1;
 	}
 }
 
@@ -214,18 +230,20 @@ compare_pN_to_pN_by_subject (const void * va, const void * vb)
 static int
 compare_ppA_to_ppA_by_linecount (const void* va, const void* vb)
 {
-        const Article * a = *(const Article**)va;
-        const Article * b = *(const Article**)vb;
+        const register Article * a = *(const Article**)va;
+        const register Article * b = *(const Article**)vb;
 	return article_get_combined_linecount(a) - article_get_combined_linecount(b);
 }
 
 static int
 compare_ppA_to_ppA_by_action (const void * va, const void * vb)
 {
-	const Article * a = *(const Article **)va;
-	const Article * b = *(const Article **)vb;
-	gint ia, ib;
+	const register Article * a = *(const Article **)va;
+	const register Article * b = *(const Article **)vb;
+	int ia, ib;
 
+#if 0
+	ia = article
 	ia = article_flag_on (a, STATE_SAVE_QUEUED) ? 1 : 0;
 	ib = article_flag_on (b, STATE_SAVE_QUEUED) ? 1 : 0;
 	if (ia != ib)
@@ -235,14 +253,17 @@ compare_ppA_to_ppA_by_action (const void * va, const void * vb)
 	ib = article_flag_on (b, STATE_DOWNLOAD_FLAGGED) ? 1 : 0;
 	if (ia != ib)
 		return ib - ia;
+#else
+#warning fixme -- how to sort on queue?
+#endif
 
 	ia = article_flag_on (a, STATE_DECODED) ? 1 : 0;
 	ib = article_flag_on (b, STATE_DECODED) ? 1 : 0;
 	if (ia != ib)
 		return ib - ia;
 
-	ia = article_has_body (a) ? 1 : 0;
-	ib = article_has_body (b) ? 1 : 0;
+	ia = acache_has_message (a->message_id) ? 1 : 0;
+	ib = acache_has_message (b->message_id) ? 1 : 0;
 	if (ia != ib)
 		return ib - ia;
 
@@ -252,8 +273,8 @@ compare_ppA_to_ppA_by_action (const void * va, const void * vb)
 static int
 compare_ppA_to_ppA_by_read (const void * va, const void * vb)
 {
-	const Article * a = *(const Article **)va;
-	const Article * b = *(const Article **)vb;
+	const register Article * a = *(const Article **)va;
+	const register Article * b = *(const Article **)vb;
 	const gboolean a_is_read = article_is_read (a);
 	const gboolean b_is_read = article_is_read (b);
 	const gboolean a_is_new = article_is_new (a);
@@ -315,7 +336,7 @@ compare_ppA_to_ppA_by_message_id (const void* a, const void* b)
 
 typedef struct
 {
-	gchar * data;
+	char name[128];
 	Article * article;
 }
 ArticleStruct;
@@ -325,7 +346,7 @@ compare_pAS_to_pAS_by_data (const void * va, const void * vb)
 {
 	const ArticleStruct * a = (const ArticleStruct*)va;
 	const ArticleStruct * b = (const ArticleStruct*)vb;
-	return strcmp (a->data, b->data);
+	return strcmp (a->name, b->name);
 }
 
 void
@@ -336,6 +357,9 @@ sort_articles (Article      ** buf,
 {
 	g_return_if_fail (articles_are_valid ((const Article **)buf, article_qty));
 
+	if (!article_qty)
+		return;
+
 	switch (sort_type)
 	{
 		case ARTICLE_SORT_AUTHOR:
@@ -344,20 +368,16 @@ sort_articles (Article      ** buf,
 			ArticleStruct * as = g_new (ArticleStruct, article_qty);
 			for (i=0; i<article_qty; ++i)
 			{
-				gchar author[256];
-				article_get_short_author_str (buf[i], author, sizeof(author));
-				as[i].data = g_strdup (author);
 				as[i].article = buf[i];
-				g_strdown (as[i].data);
+				article_get_short_author_str (buf[i], as[i].name, sizeof(as[i].name));
+				g_strdown (as[i].name);
 			}
 			msort (as,
 			       article_qty,
 			       sizeof(ArticleStruct),
 			       compare_pAS_to_pAS_by_data);
-			for (i=0; i<article_qty; ++i) {
+			for (i=0; i<article_qty; ++i)
 				buf[i] = as[i].article;
-				g_free (as[i].data);
-			}
 			g_free (as);
 			break;
 		}
@@ -391,7 +411,7 @@ sort_articles (Article      ** buf,
 		{
 			gint i;
 			Norm * norm_buf = NULL;
-			gchar * str_buf = NULL;
+			char * str_buf = NULL;
 			normalize_articles (buf, article_qty, &norm_buf, &str_buf);
 			msort (norm_buf, article_qty, sizeof(Norm), compare_pN_to_pN_by_subject);
 			for (i=0; i<article_qty; ++i)
@@ -436,10 +456,23 @@ article_get_part_state (Article * a)
 {
 	guint retval;
 
+	/* not a multipart */
 	if (a->parts<1)
 		retval = 0;
-	else if (a->part>1)
+
+	/* someone's posted a "00/124" nfo message */
+	else if (a->part==0)
+		retval = 0;
+
+	/* incomplete multipart */
+	else if (a->part>1 && a->parent==NULL)
 		retval = STATE_MULTIPART_SOME;
+
+	/* someone's posted a followup to a multipart */
+	else if (a->linecount<250 && !g_strncasecmp(a->subject,"Re:", 3))
+		retval = 0;
+
+	/* a multipart */
 	else {
 		GSList * l;
 		gint part = a->part + 1;
@@ -478,14 +511,13 @@ thread_articles (GPtrArray    * articles)
 {
 	guint i;
 	guint qty = articles->len;
-	Article ** refs;
 	Article search_a;
-	Article * p_search_a=&search_a;
 	gchar * norm_str_buf;
 	Norm * norm;
 	Norm * sorted_norm;
 	GArray * buf = NULL;
 	GString * tmp_references;
+	GHashTable * ref_hash;
 
 	/* sanity clause */
 	g_return_if_fail (articles!=NULL);
@@ -503,8 +535,11 @@ thread_articles (GPtrArray    * articles)
 	search_a.references = NULL;
 
 	/* make a message-id-sorted array of the articles */
-	refs = g_memdup (articles->pdata, sizeof(gpointer)*qty);
-	qsort (refs, qty, sizeof(Article*), compare_ppA_to_ppA_by_message_id);
+	ref_hash = g_hash_table_new (g_str_hash, g_str_equal);
+	for (i=0; i<qty; ++i) {
+		Article * a = ARTICLE (g_ptr_array_index (articles, i));
+		g_hash_table_insert (ref_hash, (gpointer)a->message_id, a);
+	}
 
 	/* normalize the articles */
 	norm = NULL;
@@ -546,33 +581,24 @@ thread_articles (GPtrArray    * articles)
 
 			while (parent==NULL && is_nonempty_string(message_id))
 			{
-				gboolean exact = FALSE;
-
-				search_a.message_id = message_id;
-
-				index = lower_bound (&p_search_a,
-						     refs,
-						     qty,
-						     sizeof(Article*),
-						     compare_ppA_to_ppA_by_message_id,
-						     &exact);
+				Article * match = (Article*) g_hash_table_lookup (ref_hash, message_id);
 
 				/* if we found the ancestor & it's worthy, thread it */
-				if (exact && !is_child_of(refs[index],a))
+				if (match!=NULL && !is_child_of(match,a))
 				{
 					gboolean subject_changed = FALSE;
 
 					if (break_thread_when_subject_changes)
 					{
-						const gchar * new_subject = norm[i].subject;
-						const gchar * old_subject = refs[index]->subject;
+						const char * new_subject = norm[i].subject;
+						const char * old_subject = article_get_subject (match);
 						g_array_set_size (buf, strlen(old_subject)+1);
-						normalize_subject (buf->data, refs[index]);
+						normalize_subject (buf->data, match);
 						subject_changed = strcmp (buf->data, new_subject);
 					}
 
 					if (!subject_changed)
-						parent = refs[index];
+						parent = match;
 				}
 
 				/* if we couldn't find the ancestor, march up the References string */
@@ -679,13 +705,13 @@ thread_articles (GPtrArray    * articles)
 	/* check multipart count */
 	for (i=0; i!=articles->len; ++i) {
 		Article * a = ARTICLE(g_ptr_array_index(articles,i));
-		a->state &= ~(STATE_MULTIPART_ALL&STATE_MULTIPART_SOME);
+		a->state &= ~(STATE_MULTIPART_ALL|STATE_MULTIPART_SOME);
 		if (a->parent == NULL)
 			set_children_part_state (a, article_get_part_state(a));
 	}
 
 	/* cleanup */
-	g_free (refs);
+	g_hash_table_destroy (ref_hash);
 	g_free (norm);
 	g_free (norm_str_buf);
 	g_free (sorted_norm);
