@@ -1,11 +1,11 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #include <config.h>
@@ -33,15 +32,15 @@
 #include <pan/base/group.h>
 #include <pan/base/pan-glib-extensions.h>
 #include <pan/base/pan-i18n.h>
-#include <pan/base/run.h>
+/*#include <pan/base/run.h>*/
 #include <pan/base/server.h>
 #include <pan/base/group.h>
 #include <pan/base/util-mime.h>
 
-const gchar * default_incoming_name_addr;
-const gchar * default_incoming_name_real;
+const char * default_incoming_name_addr;
+const char * default_incoming_name_real;
 
-static void fire_articles_changed (Group*, Article**, gint article_qty, ArticleChangeType);
+static void fire_articles_changed (Group*, Article**, int article_qty, ArticleChangeType);
 
 /***
 ****
@@ -55,10 +54,6 @@ article_destructor (Article * a)
         g_return_if_fail (a != NULL);
 
 	/* clean up article parts */
-	if (a->headers != NULL) {
-		g_hash_table_destroy (a->headers);
-		a->headers = NULL;
-	}
 	if (a->threads != NULL) {
 		g_slist_free (a->threads);
 		a->threads = NULL;
@@ -89,7 +84,6 @@ article_constructor (Article * article, Group * group)
 	article->part = (gint16)0;
 	article->parts = (gint16)0;
 	article->score = (gint16)0;
-	article->crosspost_qty = (gint8)0;
 	article->linecount = (guint16)0;
 	article->state = (guint16)0;
 	article->unread_children = (guint16)0;
@@ -104,7 +98,6 @@ article_constructor (Article * article, Group * group)
 	article->references = NULL;
 	article->parent = NULL;
 	article->threads = NULL;
-	article->headers = NULL;
 }
 
 Article*
@@ -120,12 +113,6 @@ article_new (Group * group)
 }
 
 
-static void
-article_dup_ghfunc (gpointer key, gpointer val, gpointer user_data)
-{
-	Article * article = ARTICLE(user_data);
-	article_init_header (article, key, val, DO_CHUNK_SHARE);
-}
 Article*
 article_dup (Group * group, const Article * article)
 {
@@ -146,7 +133,6 @@ article_dup (Group * group, const Article * article)
 	retval->parts               = article->parts;
 	retval->part                = article->part;
 	retval->linecount           = article->linecount;
-	retval->crosspost_qty       = article->crosspost_qty;
 	retval->state               = article->state;
 	retval->date                = article->date;
 	retval->number              = article->number;
@@ -158,10 +144,6 @@ article_dup (Group * group, const Article * article)
 	retval->references          = chunk_if_exists (article->references, TRUE);
 	retval->parent              = NULL;
 	retval->threads             = NULL;
-	retval->headers             = NULL;
-
-	if (article->headers != NULL)
-		g_hash_table_foreach (article->headers, article_dup_ghfunc, retval);
 
 	return retval;
 }
@@ -171,7 +153,7 @@ article_dup (Group * group, const Article * article)
 ***/
 
 static void
-articles_set_dirty (Article ** articles, gint qty)
+articles_set_dirty (Article ** articles, int qty)
 {
 	Group * group;
 	debug_enter ("articles_set_dirty");
@@ -262,8 +244,8 @@ article_is_read (const Article * a)
 static void
 articles_set_read_numbers_ghfunc (gpointer key, gpointer value, gpointer user_data)
 {
-	gint i;
-	gint changed_qty;
+	int i;
+	int changed_qty;
 	Group * g = GROUP(key);
 	GArray * numbers = (GArray*) value;
 	gboolean read = user_data!=NULL;
@@ -293,30 +275,32 @@ articles_set_read_numbers_ghfunc (gpointer key, gpointer value, gpointer user_da
 	g_array_free (numbers, TRUE);
 }
 
-static void
-articles_set_read_articles_ghfunc (gpointer key, gpointer value, gpointer user_data)
+void
+articles_set_read_simple (Article    ** articles,
+                          int           article_qty,
+                          gboolean      read)
 {
 	guint i;
-	Group * g = GROUP(key);
-	GArray * articles = (GArray*) value;
-	gboolean read = user_data!=NULL;
-	Newsrc * newsrc = group_get_newsrc (g);
-	GPtrArray * changed = g_ptr_array_new ();
+	Group * g;
+	Newsrc * newsrc;
+	GPtrArray * changed;
+
+	/* sanity clause */
+	g_return_if_fail (article_qty>0);
+	g_return_if_fail (articles_are_valid ((const Article **)articles, article_qty));
+
+	/* prep local variables */
+	g = articles[0]->group;
+	newsrc = group_get_newsrc (g);
+       	changed = g_ptr_array_sized_new (article_qty);
+
+	/* find the articles that will be changed by this action */
+	for (i=0; i<article_qty; ++i)
+		if (!!read != !!newsrc_mark_article (newsrc, articles[i]->number, read))
+			g_ptr_array_add (changed, articles[i]);
 
 	/**
-	***  Walk through the articles & make a list of those to be changed
-	**/
-
-	for (i=0; i!=articles->len; ++i)
-	{
-		Article * a = ARTICLE(g_array_index(articles,Article*,i));
-
-		if (!!read != !!newsrc_mark_article (newsrc, a->number, read))
-			g_ptr_array_add (changed, a);
-	}
-
-	/**
-	***  If anything to be changed:
+	***  If anything is to be changed:
 	***  (1) update parent counts
 	***  (2) update group's counts
 	***  (3) fire an event
@@ -340,9 +324,14 @@ articles_set_read_articles_ghfunc (gpointer key, gpointer value, gpointer user_d
 		/* fire an event */
 		fire_articles_changed (g, (Article**)changed->pdata, changed->len, ARTICLE_CHANGED_READ);
 	}
+}
 
-
-	/* cleanup */
+static void
+articles_set_read_articles_ghfunc (gpointer key, gpointer value, gpointer user_data)
+{
+	GArray * articles = (GArray*) value;
+	const gboolean read = user_data != NULL;
+	articles_set_read_simple ((Article**)articles->data, articles->len, read);
 	g_array_free (articles, TRUE);
 }
 
@@ -351,7 +340,7 @@ articles_set_read_articles_ghfunc (gpointer key, gpointer value, gpointer user_d
  * If no matching GArray* exists, it's created and added to the hash before returning.
  */
 static GArray*
-get_group_array_from_hash (GHashTable * hash, Group * group, gint size)
+get_group_array_from_hash (GHashTable * hash, Group * group, int size)
 {
 	GArray * arr = (GArray*) g_hash_table_lookup (hash, group);
 	if (arr == NULL)
@@ -363,7 +352,7 @@ get_group_array_from_hash (GHashTable * hash, Group * group, gint size)
  * Builds a GHashTable of Group* -> GArray*, where the GArray is filled with message numbers.
  */
 static void
-mark_read_xreffunc (Group * g, gulong number, gpointer user_data)
+mark_read_xreffunc (Server * server, Group * g, gulong number, gpointer user_data)
 {
 	GHashTable ** group_to_numbers = (GHashTable**) user_data;
 
@@ -455,11 +444,11 @@ articles_set_read (Article ** articles, int article_qty, gboolean read)
 
 void
 articles_add_flag (Article  ** articles,
-                   gint        qty,
+                   int         qty,
                    gushort     flag)
 {
-	gint i;
-	gint changed_qty;
+	int i;
+	int changed_qty;
 	Article ** changed;
 
 	/* sanity clause */
@@ -484,11 +473,11 @@ articles_add_flag (Article  ** articles,
 
 void
 articles_remove_flag (Article         ** articles,
-                      gint               qty,
+                      int                qty,
                       gushort            flag)
 {
-	gint i;
-	gint changed_qty;
+	int i;
+	int changed_qty;
 	Article ** changed;
 
 	/* sanity clause */
@@ -518,11 +507,11 @@ articles_remove_flag (Article         ** articles,
 ****
 ***/
 
-gchar*
+char*
 article_get_thread_message_id (const Article* article)
 {
 	char * pch;
-	const gchar * refs;
+	const char * refs;
 
 	/* sanity clause */
 	g_return_val_if_fail (article_is_valid(article), NULL);
@@ -539,7 +528,7 @@ article_get_thread_message_id (const Article* article)
 	}
 	else /* top of the thread */
 	{
-		const gchar * msg_id = article_get_message_id (article);
+		const char * msg_id = article_get_message_id (article);
 		g_return_val_if_fail (is_nonempty_string(msg_id), NULL);
 		pch = g_strdup (msg_id);
 	}
@@ -614,13 +603,11 @@ article_get_unique_threads (const GPtrArray    * articles,
 	debug_enter ("articlelist_get_unique_threads");
 
 	/* sanity clause */
-	retval = g_ptr_array_new ();
+	retval = g_ptr_array_sized_new (128);
 	g_return_val_if_fail (articles_are_valid ((const Article **)articles->pdata, articles->len), retval);
 
 	/* get the thread for each article passed in */
-	thread = g_ptr_array_new ();
-	pan_g_ptr_array_reserve (retval, 128);
-	pan_g_ptr_array_reserve (thread, 128);
+	thread = g_ptr_array_sized_new (128);
 	all = g_hash_table_new (g_str_hash, g_str_equal);
 	for (i=0; i<articles->len; ++i)
 	{
@@ -642,7 +629,7 @@ article_get_unique_threads (const GPtrArray    * articles,
 		for (j=0; j<thread->len; ++j)
 		{
 			Article * b = ARTICLE(g_ptr_array_index(thread,j));
-			const gchar * message_id = article_get_message_id (b);
+			const char * message_id = article_get_message_id (b);
 
 			if (g_hash_table_lookup (all, message_id) == NULL)
 			{
@@ -660,14 +647,53 @@ article_get_unique_threads (const GPtrArray    * articles,
 }
 
 
+const char **
+article_get_message_ids  (const Article ** articles,
+                          int              article_qty)
+{
+	int i;
+	const char ** message_ids;
+
+	g_return_val_if_fail (articles!=NULL, NULL);
+	g_return_val_if_fail (article_qty>=0, NULL);
+	g_return_val_if_fail (articles_are_valid (articles, article_qty), NULL);
+
+	message_ids = g_new (const char *, article_qty);
+	for (i=0; i<article_qty; ++i)
+		message_ids[i] = article_get_message_id (ARTICLE(articles[i]));
+
+	return message_ids;
+}
+
+
 /***
 ****
 ****  OTHER HEADERS
 ****
 ***/
 
+int
+article_get_crosspost_qty (const Article  * article)
+{
+	int retval;
+
+	g_return_val_if_fail (article_is_valid (article), 0);
+
+	if (!is_nonempty_string (article->xref))
+		retval = 1;
+	else {
+		const char * pch;
+		retval = 0;
+		for (pch=article->xref+6; *pch; ++pch)
+			if (*pch==':')
+				++retval;
+	}
+
+	return retval;
+}
+
 void
-article_remove_header (Article * a, const gchar * key)
+article_remove_header (Article * a, const char * key)
 {
 	g_return_if_fail (a!=NULL);
 	g_return_if_fail (is_nonempty_string(key));
@@ -685,40 +711,17 @@ article_remove_header (Article * a, const gchar * key)
 		a->author_addr = NULL;
 		a->author_real = NULL;
 	}
-	else if (a->headers!=NULL)
-	{
-		g_hash_table_remove (a->headers, key);
-		if (!g_hash_table_size(a->headers))
-		{
-			g_hash_table_destroy (a->headers);
-			a->headers = NULL;
-		}
-	}
-}
-
-const gchar*
-article_get_extra_header (const Article * a, const gchar * key)
-{
-	const gchar * retval = NULL;
-
-	/* sanity clause */
-	g_return_val_if_fail (article_is_valid(a), NULL);
-	g_return_val_if_fail (is_nonempty_string(key), NULL);
-
-	if (a->headers!=NULL)
-		retval = (const gchar*) g_hash_table_lookup (a->headers, key);
-
-	return retval;
+	else g_error (_("Don't know what to do with header \"%s\""), key);
 }
 
 gboolean
-article_header_is_internal (const gchar * key)
+article_header_is_internal (const char * key)
 {
-	return is_nonempty_string(key) && !g_strncasecmp(key,"Pan-",4);
+	return is_nonempty_string(key) && !g_strncasecmp(key,"X-Pan-Internal-",15);
 }
 
 gboolean
-article_header_is_extra (const gchar * key)
+article_header_is_extra (const char * key)
 {
 	/* sanity check */
 	if (!is_nonempty_string(key)) return FALSE;
@@ -732,15 +735,49 @@ article_header_is_extra (const gchar * key)
 	if (!strcmp(key,HEADER_ORGANIZATION)) return FALSE;
 	if (!strcmp(key,HEADER_REPLY_TO)) return FALSE;
 	if (!strcmp(key,HEADER_DATE)) return FALSE;
+	if (!strcmp(key,HEADER_XREF)) return FALSE;
+	if (!strcmp(key,HEADER_FROM)) return FALSE;
 
 	return TRUE;
 }
 
-static void
-ensure_extra_headers_exists (Article * a)
+void
+xref_foreach (const char         * xref_string,
+              Server             * server,
+              ServerGroupsType     set,
+              Group              * skip,
+              ArticleXRefFunc      func,
+              gpointer             user_data)
 {
-	if (a->headers == NULL)
-		a->headers = g_hash_table_new (g_str_hash, g_str_equal);
+	/* sanity clause */
+	g_return_if_fail (server_is_valid(server));
+
+	/* get the xref header */
+	if (is_nonempty_string(xref_string))
+	{
+		const char * march, * run_str = NULL;
+		int run_len;
+
+		/* skip the servername; we've got the server already */
+		skip_next_token (xref_string, ' ', &march);
+
+		/* walk through the xrefs, of format "group1:number group2:number" */
+		while (get_next_token_run (march, ' ', &march, &run_str, &run_len))
+		{
+			const char * delimit = g_strstr_len (run_str, run_len, ":");
+			if (delimit != NULL)
+			{
+				const gulong number = strtoul (delimit+1, NULL, 10);
+				Group * group = NULL;
+				char * group_name;
+
+				pan_strndup_alloca (group_name, run_str, delimit-run_str);
+				group = server_get_named_group_in_type (server, group_name, set);
+				if (group!=NULL && group!=skip)
+					(*func)(server, group, number, user_data);
+			}
+		}
+	}
 }
 
 void
@@ -750,47 +787,17 @@ article_xref_foreach (const Article       * a,
 		      ServerGroupsType      set,
                       gboolean              skip_group_a)
 {
-	/* sanity clause */
 	g_return_if_fail (article_is_valid(a));
 
-	/* get the xref header */
-	if (is_nonempty_string(a->xref))
-	{
-		Run run;
-		const gchar * march;
-		Server * const server = a->group->server;
-
-		/* skip the servername; we've got the server already */
-		skip_next_token (a->xref, ' ', &march);
-
-		/* walk through the xrefs, of format "group1:number group2:number" */
-		while (get_next_token_run (march, ' ', &march, &run.str, &run.len))
-		{
-			const gchar * delimit = run_strchr (&run, ':');
-			if (delimit != NULL)
-			{
-				const gulong number = strtoul (delimit+1, NULL, 10);
-				Group * g = NULL;
-				gchar * group_name;
-
-				pan_strndup_alloca (group_name, run.str, delimit-run.str);
-
-				if (server_has_group_in_type (server, group_name, set))
-					g = server_get_named_group (server, group_name);
-
-				if (g!=NULL && (a->group!=g || !skip_group_a))
-					(*func)(g, number, user_data);
-			}
-		}
-	}
+	xref_foreach (a->xref, a->group->server, set, skip_group_a?a->group:NULL, func, user_data);
 }
 
 static void
-article_set_header_impl (Article         * a,
-                         const gchar     * key,
-                         const gchar     * val,
-                         HeaderAction      action,
-                         gboolean          init)
+article_set_header_impl (Article        * a,
+                         const char     * key,
+                         const char     * val,
+                         HeaderAction     action,
+                         gboolean         init)
 {
 	g_return_if_fail (a!=NULL);
 	g_return_if_fail (is_nonempty_string(key));
@@ -817,23 +824,14 @@ article_set_header_impl (Article         * a,
 			else
 				article_set_author_from_header (a, val);
 		}
-		else
-		{
-			ensure_extra_headers_exists (a);
-			key = group_chunk_string (a->group, key, TRUE);
-			g_hash_table_insert (a->headers,
-			                     (gpointer)key,
-			                     (gpointer)val);
-			if (!init)
-				articles_set_dirty (&a, 1);
-		}
+		else pan_warn_if_reached ();
 	}
 }
 
 void
 article_init_header (Article         * a,
-                     const gchar     * key,
-                     const gchar     * val,
+                     const char      * key,
+                     const char      * val,
                      HeaderAction      action)
 {
 	article_set_header_impl (a, key, val, action, TRUE);
@@ -841,342 +839,94 @@ article_init_header (Article         * a,
 
 void
 article_set_header (Article         * a,
-                    const gchar     * key,
-                    const gchar     * val,
+                    const char      * key,
+                    const char      * val,
                     HeaderAction      action)
 {
 	article_set_header_impl (a, key, val, action, FALSE);
 }
 
-static void
-article_get_all_headers_ghfunc (gpointer key, gpointer value, gpointer user_data)
-{
-	GPtrArray * a = (GPtrArray*)user_data;
-	g_ptr_array_add (a, key);
-	g_ptr_array_add (a, value);
-}
-
-GPtrArray*
-article_get_all_headers (const Article * a)
-{
-	GPtrArray * retval = g_ptr_array_new ();
-
-	/* sanity clause */
-	g_return_val_if_fail (a!=NULL, retval);
-
-	/* add the headers */
-	if (a->headers != NULL)
-	{
-		pan_g_ptr_array_reserve (retval, g_hash_table_size(a->headers)*2);
-		g_hash_table_foreach (a->headers, article_get_all_headers_ghfunc, retval);
-	}
-
-	return retval;
-}
-
-gboolean
-article_has_attachment (const Article *a)
-{
-	g_return_val_if_fail (article_is_valid(a), FALSE);
-
-	return article_get_extra_header(a,PAN_ATTACH_FILE) != NULL;
-}
-
 /***
 ****
 ****  BODY
 ****
 ***/
 
-gboolean
-article_has_body (const Article * a)
+char*
+article_format_author_str (const char * author_addr,
+                           const char * author_real,
+                           char       * buf,
+                           int          bufsize)
 {
-	gboolean retval = FALSE;
-
-	g_return_val_if_fail (article_is_valid(a), FALSE);
-
-	/* try to get it from acache... */
-	if (!retval && is_nonempty_string(a->message_id))
-		retval = acache_has_message (a->message_id);
-
-	/* maybe the article's in a folder so the body
-	 * is hidden as a header... */
-	if (!retval)
-		if (article_get_extra_header (a, PAN_BODY) != NULL)
-			retval = TRUE;
-
-	return retval;
-}
-
-gchar*
-article_get_author_str (const Article * a, gchar * buf, gint bufsize)
-{
-	const gboolean have_addr = a!=NULL && is_nonempty_string (a->author_addr);
-	const gboolean have_name = a!=NULL && is_nonempty_string (a->author_real);
+	const gboolean have_addr = is_nonempty_string (author_addr);
+	const gboolean have_name = is_nonempty_string (author_real);
 
 	*buf = '\0';
 
-	if (have_addr && have_name) {
-		g_snprintf (buf, bufsize, "\"%s\" <%s>", a->author_real, a->author_addr);
-	}
-	else if (have_addr) {
-		strncpy (buf, a->author_addr, bufsize);
-		buf[bufsize-1] = '\0';
-	}
-	else if (have_name) {
-		strncpy (buf, a->author_real, bufsize);
-		buf[bufsize-1] = '\0';
-	}
+	if (have_addr && have_name)
+		g_snprintf (buf, bufsize, "\"%s\" <%s>", author_real, author_addr);
+	else if (have_addr)
+		g_strlcpy (buf, author_addr, bufsize);
+	else if (have_name)
+		g_strlcpy (buf, author_real, bufsize);
 
 	g_strstrip (buf);
 	return buf;
 }
 
-void
-article_get_short_author_str (const Article * a, gchar * buf, gint len)
+char*
+article_get_author_str (const Article * a, char * buf, int bufsize)
 {
-	g_return_if_fail (article_is_valid(a));
-	g_return_if_fail (buf!=NULL);
-	g_return_if_fail (len>0);
+	*buf = '\0';
 
-	if (is_nonempty_string (a->author_real) && strcmp(a->author_real,default_incoming_name_real))
+	/* sanity clause */
+	g_return_val_if_fail (buf!=NULL, buf);
+	g_return_val_if_fail (a!=NULL, buf);
+	g_return_val_if_fail (bufsize>0, buf);
+
+	return article_format_author_str (a->author_addr, a->author_real, buf, bufsize);
+}
+
+int
+article_format_short_author_str (const char * author_addr,
+                                 const char * author_real,
+                                 char       * buf,
+                                 int          buflen)
+{
+	int retval;
+
+	/* sanity clause */
+	g_return_val_if_fail (buf!=NULL, 0);
+	g_return_val_if_fail (buflen>0, 0);
+
+	if (is_nonempty_string (author_real) && strcmp(author_real, default_incoming_name_real))
 	{
-		strncpy (buf, a->author_real, len);
-		buf[len-1] = '\0';
+		retval = g_strlcpy (buf, author_real, buflen);
 	}
-	else if (is_nonempty_string (a->author_addr) && strcmp(a->author_addr,default_incoming_name_addr))
+	else if (is_nonempty_string (author_addr) && strcmp(author_addr,default_incoming_name_addr))
 	{
-		const gchar * author = a->author_addr;
-		const gchar * pch = strchr (author, '@');
-		if (pch == NULL)
-			pch = author + strlen(author);
-		strncpy (buf, author, MIN(len,pch-author));
-		buf[len-1] = '\0';
+		const char * author = author_addr;
+		const char * p = strchr (author, '@');
+		if (p == NULL)
+			p = author + strlen(author);
+		retval = g_strlcpy (buf, author, MIN(buflen,p-author));
 	}
 	else
 	{
-		strncpy (buf, default_incoming_name_real, len);
-		buf[len-1] = '\0';
-	}
-}
-
-
-gchar*
-article_get_message (const Article * a)
-{
-	gchar * retval = NULL;
-
-	/* sanity clause */
-	g_return_val_if_fail (article_is_valid(a), g_strdup(""));
-
-	if (retval==NULL)
-	{
-		const gchar * message_id = article_get_message_id (a);
-		if (acache_has_message (message_id))
-			retval = acache_get_message (message_id);
-	}
-
-	if (retval==NULL)
-	{
-		GString * s = g_string_new (NULL);
-		gchar * pch;
-
-		pch = article_get_headers (a);
-		g_string_append (s, pch);
-		g_free (pch);
-
-		g_string_append (s, "\n");
-
-		pch = article_get_body (a);
-		g_string_append (s, pch);
-		g_free (pch);
-
-		retval = s->str;
-		g_string_free (s, FALSE);
+		retval = g_strlcpy (buf, default_incoming_name_real, buflen);
 	}
 
 	return retval;
 }
 
-gchar*
-article_get_headers (const Article * a)
+int
+article_get_short_author_str (const Article * a, char * buf, int len)
 {
-	gchar * retval = NULL;
-
-	g_return_val_if_fail (article_is_valid(a), g_strdup(""));
-
-	/* do we have it in acache? */
-	if (retval==NULL)
-	{
-		const gchar * message_id = article_get_message_id (a);
-		if (acache_has_message (message_id))
-		{
-			gchar * s = acache_get_message (message_id);
-			gchar * pch = pan_strstr (s, "\n\n");
-			if (pch != NULL) {
-				*pch = '\0';
-				retval = s;
-			}
-		}
-	}
-
-	/* can we build it from a? */
-	if (retval==NULL)
-	{
-		gchar * tmp;
-		const gchar * c_tmp;
-		GString * s = g_string_new (NULL);
-
-		/* subject */
-		c_tmp = article_get_subject (a);
-		if (is_nonempty_string (c_tmp))
-			g_string_sprintfa (s, "%s: %s\n", HEADER_SUBJECT, c_tmp);
-
-		/* author */
-		if (1) {
-			gchar author[512] = { '\0' };
-	       		article_get_author_str (a, author, sizeof(author));
-			if (is_nonempty_string (author))
-				g_string_sprintfa (s, "%s: %s\n", HEADER_FROM, author);
-		}
-
-		/* date */
-		if (a->date != 0) {
-			tmp = rfc822_date_generate (a->date);
-			g_string_sprintfa (s, "%s: %s\n", HEADER_DATE, tmp);
-			g_free (tmp);
-		}
-
-		/* message-id */
-		c_tmp = article_get_message_id (a);
-		if (is_nonempty_string (c_tmp))
-			g_string_sprintfa (s, "%s: %s\n", HEADER_MESSAGE_ID, c_tmp);
-
-		/* references */
-		c_tmp = a->references;
-		if (is_nonempty_string (c_tmp))
-			g_string_sprintfa (s, "%s: %s\n", HEADER_REFERENCES, c_tmp);
-
-		/* extra headers */
-		if (1) {
-			gint i;
-			GPtrArray * z = article_get_all_headers (a);
-			for (i=0; i<z->len; i+=2) {
-				const char* key = (const char*) g_ptr_array_index (z, i);
-				const char* val = (const char*) g_ptr_array_index (z, i+1);
-				if (strncmp (key, PAN_BODY, strlen(PAN_BODY)) != 0)
-					g_string_sprintfa (s, "%s: %s\n",  key, val);
-			}
-			g_ptr_array_free (z, TRUE);
-		}
-
-		retval = s->str;
-		g_string_free (s, FALSE);
-	}
-
-	return retval;
+	return article_format_short_author_str (a->author_addr,
+	                                        a->author_real,
+	                                        buf, len);
 }
 
-char*
-article_get_body (const Article * a)
-{
-	char * retval = NULL;
-
-	g_return_val_if_fail (article_is_valid(a), g_strdup(""));
-
-	/* see if we've got the body hidden in a header... */
-	if (retval==NULL)
-	{
-		const char * body = article_get_extra_header (a, PAN_BODY);
-		if (body != NULL)
-			retval = g_strdup (body);
-	}
-
-	/* see if we've got it in acache... */
-	if (retval==NULL)
-	{
-		const char * message_id = article_get_message_id (a);
-		if (acache_has_message (message_id))
-		{
-			char * s = acache_get_message (message_id);
-			gboolean is_html = FALSE;
-			GMimeMessage * m = pan_g_mime_parser_construct_message ((const char **)&s, 1);
-			retval = g_mime_message_get_body (m, TRUE, &is_html);
-			g_mime_object_unref (GMIME_OBJECT(m));
-			g_free (s);
-		}
-	}
-
-	return retval;
-}
-
-gboolean
-article_get_header_run_hash_from_text (const char     * article_text,
-                                       GHashTable    ** makeme_hash,
-                                       gpointer       * allocme_buf)
-{
-	guint line_qty;
-	guint line_len;
-	const char * line; 
-	const char * march;
-	GHashTable * hash;
-	Run * buf;
-	guint buf_qty;
-	guint buf_size;
-
-	/* sanity clause */
-	g_return_val_if_fail (article_text, FALSE);
-	g_return_val_if_fail (makeme_hash!=NULL, FALSE);
-	g_return_val_if_fail (allocme_buf!=NULL, FALSE);
-
-	/* count number of lines */
-	line = NULL;
-	line_qty = 0;
-	for (march=article_text; get_next_token_run(march,'\n',&march,&line,&line_len) && line_len!=0; )
-		++line_qty;
-
-	/* alloc a buffer of enough Run structs */
-	buf_size = line_qty * 2u;
-	buf_qty = 0u;
-	buf = g_malloc (sizeof(Run) * buf_size);
-
-	/* alloc the hashtable */
-	hash = g_hash_table_new (run_hash, run_equal);
-
-	/* populate the hashtable */
-	march = article_text;
-	while (get_next_token_run (march, '\n', &march, &line, &line_len))
-	{
-		guint key_len = 0;
-		const char * key = NULL;
-
-		if (line_len == 0) /* \n\n -- end of headers */
-			break;
-
-		if (get_next_token_run (line, ':', NULL, &key, &key_len))
-		{
-			gint val_len = 0;
-			const char * val = NULL;
-			Run * key_run = buf + buf_qty++;
-			Run * val_run = buf + buf_qty++;
-
-			val = key + key_len + 1; /* skip past ':' */
-			val_len = 0;
-			while (val[val_len]!='\n' && val[val_len]!='\0')
-				++val_len;
-
-			g_hash_table_insert (hash, 
-			                     run_strstrip (run_init (key_run, key, key_len)),
-			                     run_strstrip (run_init (val_run, val, val_len)));
-		}
-	}
-
-	*makeme_hash = hash;
-	*allocme_buf = buf;
-	return TRUE;
-}
-
-
 static void
 article_set_author_from_header_impl (Article       * a,
                                      const char    * header_from,
@@ -1187,26 +937,26 @@ article_set_author_from_header_impl (Article       * a,
 	char * addr = addr_buf;
 	char * real = real_buf;
 	char * pch;
+	const char * charset;
 
+	/* sanity clause */
 	g_return_if_fail (a!=NULL);
 	g_return_if_fail (a->group!=NULL);
 	g_return_if_fail (is_nonempty_string(header_from));
 
-	/* note that strict is FALSE here because we can't control
-	 * how other people have posted to usenet */
-	gnksa_do_check_from (header_from,
+	/* get the author address & real name */
+	charset = group_get_default_charset (a->group);
+	pch = pan_header_to_utf8 (header_from, -1, charset);
+	gnksa_do_check_from (pch,
 	                     addr_buf, sizeof(addr_buf),
 	                     real_buf, sizeof(real_buf),
-	                     FALSE);
-	if (pan_header_is_8bit_encoded (addr_buf))
-		addr = g_mime_utils_8bit_header_decode ((const guchar *)addr_buf);
-	if (pan_header_is_8bit_encoded (real_buf))
-		real = g_mime_utils_8bit_header_decode ((const guchar *)real_buf);
+	                     FALSE); /* not strict */
+	g_free (pch);
 
 	/* use the real mail address, or fill in a default */
 	pch = addr;
 	if (!is_nonempty_string (pch))
-		pch = (gchar*)default_incoming_name_addr;
+		pch = (char*)default_incoming_name_addr;
 	a->author_addr = group_chunk_string (a->group, pch, TRUE);
 
 	/* use the real name, or fill in a default. */
@@ -1215,93 +965,65 @@ article_set_author_from_header_impl (Article       * a,
 		a->author_real = group_chunk_string (a->group, real, TRUE);
 	}
 	else if (addr!=NULL && ((pch=strchr(addr,'@'))!=NULL)) {
-		gchar * tmp;
+		char * tmp;
 		pan_strndup_alloca (tmp, addr, pch-addr);
 		a->author_real = group_chunk_string (a->group, tmp, TRUE);
 	}
 	else {
-		const gchar * cpch = (gchar*)default_incoming_name_real;
+		const char * cpch = (const char*)default_incoming_name_real;
 		a->author_real = group_chunk_string (a->group, cpch, TRUE);
 	}
 
 	if (!init)
 		articles_set_dirty (&a, 1);
-
-	/* cleanup */
-	if (addr!=addr_buf)
-		g_free (addr);
-	if (real!=real_buf)
-		g_free (real);
 }
 
 void
 article_init_author_from_header (Article         * a,
-                                 const gchar     * header_from)
+                                 const char      * header_from)
 {
 	article_set_author_from_header_impl (a, header_from, TRUE);
 }
 void
 article_set_author_from_header (Article         * a,
-                                const gchar     * header_from)
+                                const char      * header_from)
 {
 	article_set_author_from_header_impl (a, header_from, FALSE);
 }
 
-
-static void
-set_from_raw_init_header_gmhfunc (const char * name,
-                                  const char * value,
-                                  gpointer data)
-{
-	Article * a = ARTICLE(data);
-
-	g_return_if_fail (a!=NULL);
-	g_return_if_fail (is_nonempty_string(name));
-	g_return_if_fail (is_nonempty_string(value));
-
-	article_init_header (a, name, value, DO_CHUNK);
-}
-
 void
-article_set_from_raw_message   (Article         * a,
-                                const char      * text)
+article_set_from_g_mime_message   (Article         * a,
+                                   GMimeMessage    * msg)
 {
-	guint16 lines;
-	GMimeMessage * msg;
-	char * body;
-	const char * pch;
-	gboolean foo;
+	const char * cpch;
 	debug_enter ("article_set_from_raw_message");
 
 	/* get the message */
 	g_return_if_fail (a != NULL);
-	g_return_if_fail (is_nonempty_string(text));
-	msg = pan_g_mime_parser_construct_message ((const char **)&text, 1);
-	g_return_if_fail (msg != NULL);
+	g_return_if_fail (GMIME_IS_MESSAGE(msg));
 
-	/* body */
-	body = g_mime_message_get_body (msg, TRUE, &foo);
-	article_init_header (a, PAN_BODY, body, DO_CHUNK);
-	for (lines=0, pch=body; is_nonempty_string(pch); ++pch)
-		if (*pch=='\n')
-			++lines;
-	a->linecount = lines;
-	g_free (body);
+	g_mime_message_get_date (msg, &a->date, NULL);
 
-	/* headers */
-	if (msg->header != NULL) {
-		a->date = msg->header->date;
-		if (msg->header->from != NULL)
-			article_init_header (a, HEADER_FROM, msg->header->from, DO_CHUNK_SHARE);
-		if (msg->header->reply_to != NULL)
-			article_init_header (a, HEADER_REPLY_TO, msg->header->reply_to, DO_CHUNK_SHARE);
-		if (msg->header->subject != NULL)
-			article_init_header (a, HEADER_SUBJECT, msg->header->subject, DO_CHUNK_SHARE);
-		if (msg->header->message_id != NULL)
-			article_init_header (a, HEADER_MESSAGE_ID, msg->header->message_id, DO_CHUNK);
-		g_mime_header_foreach (msg->header->headers, set_from_raw_init_header_gmhfunc, a);
-	}
+	cpch = g_mime_message_get_sender (msg);
+	if (cpch != NULL)
+		article_init_header (a, HEADER_FROM, cpch, DO_CHUNK_SHARE);
 
+	cpch = g_mime_message_get_reply_to (msg);
+	if (cpch != NULL)
+		article_init_header (a, HEADER_REPLY_TO, cpch, DO_CHUNK_SHARE);
+
+	cpch = g_mime_message_get_subject (msg);
+	if (cpch != NULL)
+		article_init_header (a, HEADER_SUBJECT, cpch, DO_CHUNK_SHARE);
+
+	cpch = g_mime_message_get_message_id (msg);
+	if (cpch != NULL)
+		article_init_header (a, HEADER_MESSAGE_ID, cpch, DO_CHUNK);
+
+	cpch = g_mime_message_get_header (msg, HEADER_REFERENCES);
+	if (cpch != NULL)
+		article_init_header (a, HEADER_REFERENCES, cpch, DO_CHUNK);
+	
 	/* fallback if no message-id */
 	if (!is_nonempty_string(a->message_id)) {
 		char * id;
@@ -1312,46 +1034,28 @@ article_set_from_raw_message   (Article         * a,
 		article_init_header (a, HEADER_SUBJECT, _("Unparseable Subject"), DO_CHUNK_SHARE);
 	}
 
-	/* state */
-	pch = article_get_extra_header (a, "Status");
-	if (pch != NULL) {
-		const gboolean read = strchr (pch, 'R') != NULL;
-		articles_set_read (&a, 1, read);
-		article_remove_header (a, "Status");
-	}
-
-	/* references */
-	pch = article_get_extra_header (a, HEADER_REFERENCES);
-	if (pch != NULL)
-		a->references = pch;
 
 	/* cleanup */
-	g_mime_object_unref (GMIME_OBJECT(msg));
 	debug_exit ("article_set_from_raw_message");
 }
 
-
-static gboolean
-breakpoint (const Article * a)
-{
-	if (a!=NULL && a->group!=NULL)
-		g_message ("bad article's group: [%s]", a->group->name);
-	return FALSE;
-}
+/****
+*****  SANITY CHECKS
+****/
 
 gboolean
 article_is_valid (const Article * a)
 {
-	g_return_val_if_fail (a!=NULL, breakpoint(a));
-	g_return_val_if_fail (a->group!=NULL, breakpoint(a));
-	g_return_val_if_fail (group_is_valid(a->group), breakpoint(a));
-	g_return_val_if_fail (a->group->_articles_refcount>=0, breakpoint(a));
-	g_return_val_if_fail (group_is_folder(a->group) || a->group->_articles_refcount>0, breakpoint(a));
-	g_return_val_if_fail (a->message_id!=NULL, breakpoint(a));
-	g_return_val_if_fail (*a->message_id=='<', breakpoint(a));
-	g_return_val_if_fail (is_nonempty_string(a->subject), breakpoint(a));
-	g_return_val_if_fail (a->number!=0, breakpoint(a));
-	g_return_val_if_fail (!a->references || *a->references=='<', breakpoint(a));
+	g_return_val_if_fail (a!=NULL, FALSE);
+	g_return_val_if_fail (a->group!=NULL, FALSE);
+	g_return_val_if_fail (group_is_valid(a->group), FALSE);
+	g_return_val_if_fail (a->group->_articles_refcount>=0, FALSE);
+	g_return_val_if_fail (group_is_folder(a->group) || a->group->_articles_refcount>0, FALSE);
+	g_return_val_if_fail (a->message_id!=NULL, FALSE);
+	g_return_val_if_fail (*a->message_id=='<', FALSE);
+	g_return_val_if_fail (is_nonempty_string(a->subject), FALSE);
+	g_return_val_if_fail (a->number!=0, FALSE);
+	g_return_val_if_fail (!a->references || *a->references=='<', FALSE);
 
 	return TRUE;
 }
@@ -1359,7 +1063,7 @@ article_is_valid (const Article * a)
 gboolean
 articles_are_valid (const Article ** a, int qty)
 {
-	gint i;
+	int i;
 
 	/* sanity clause */
 	g_return_val_if_fail (qty>=0, FALSE);
@@ -1372,14 +1076,14 @@ articles_are_valid (const Article ** a, int qty)
 	return TRUE;
 }
 
-const gchar*
+const char*
 article_get_message_id (const Article * a)
 {
 	g_return_val_if_fail (article_is_valid(a), "");
 	return a->message_id;
 }
  
-const gchar*
+const char*
 article_get_subject (const Article * a)
 {
 	g_return_val_if_fail (article_is_valid(a), "");
@@ -1397,7 +1101,7 @@ article_get_combined_linecount (const Article * a)
 	retval = a->linecount;
 	if (a->part!=0 && a->parts!=0 && a->threads!=NULL) {
 		GSList * l;
-		gint part = a->part + 1;
+		int part = a->part + 1;
 		for (l=a->threads; l; l=l->next) {
 			Article * child = ARTICLE(l->data);
 			if (child->part == part) {
@@ -1426,7 +1130,7 @@ article_get_articles_changed_callback (void)
 static void
 fire_articles_changed (Group              * group,
                        Article           ** articles,
-                       gint                 article_qty,
+                       int                  article_qty,
                        ArticleChangeType    type)
 {
 	ArticleChangeEvent e;
