@@ -1,12 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
- * Copyright (C) 2001  Pan Development Team <pan@rebelbase.com>
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #include <config.h>
@@ -26,8 +24,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/time.h>
-#include <unistd.h>
 
 #include <glib.h>
 
@@ -55,17 +51,16 @@ file_grouplist_097_style_exists (const Server * server)
 	ServerGroupsType retval = 0;
 
 	/* sanity clause */
-	g_return_val_if_fail (server_is_valid(server), FALSE);
-	g_return_val_if_fail (is_nonempty_string(data_dir), FALSE);
+	g_return_val_if_fail (server_is_valid (server), FALSE);
 
 	/* check for sub */
-	g_snprintf (fname, sizeof(fname), "%s%c%s_sub.idx", data_dir, G_DIR_SEPARATOR, server_get_name (server));
-	if (file_exists (fname))
+	g_snprintf (fname, sizeof(fname), "%s%c%s_sub.idx", get_data_dir(), G_DIR_SEPARATOR, server_get_name (server));
+	if (pan_file_exists  (fname))
 		retval |= SERVER_GROUPS_SUBSCRIBED;
 
 	/* check for unsub */
-	g_snprintf (fname, sizeof(fname), "%s%c%s_unsub.idx", data_dir, G_DIR_SEPARATOR, server_get_name (server));
-	if (file_exists (fname))
+	g_snprintf (fname, sizeof(fname), "%s%c%s_unsub.idx", get_data_dir(), G_DIR_SEPARATOR, server_get_name (server));
+	if (pan_file_exists  (fname))
 		retval |= SERVER_GROUPS_UNSUBSCRIBED;
 
 	/* return results */
@@ -78,11 +73,10 @@ file_grouplist_096_style_exists (const Server * server)
 	gchar fname [PATH_MAX];
 	gboolean retval;
 
-	g_return_val_if_fail (server_is_valid(server), FALSE);
-	g_return_val_if_fail (is_nonempty_string(data_dir), FALSE);
+	g_return_val_if_fail (server_is_valid (server), FALSE);
        
-	g_snprintf (fname, sizeof(fname), "%s%c%s.idx", data_dir, G_DIR_SEPARATOR, server_get_name (server));
-       	retval = file_exists (fname);
+	g_snprintf (fname, sizeof(fname), "%s%c%s.idx", get_data_dir(), G_DIR_SEPARATOR, server_get_name (server));
+       	retval = pan_file_exists  (fname);
 
 	return retval;
 }
@@ -93,7 +87,7 @@ file_grouplist_exists (const Server* server)
 	ServerGroupsType retval = 0;
 
 	/* sanity clause */
-	g_return_val_if_fail (server_is_valid(server), 0);
+	g_return_val_if_fail (server_is_valid (server), 0);
 
 	/* first check the new style... */
 	retval = file_grouplist_097_style_exists (server);
@@ -115,9 +109,10 @@ file_grouplist_exists (const Server* server)
 static gint
 read_group_data (Server        * server,
                  StatusItem    * status,
-                 const gchar   * dat_str,
+                 const char    * dat_str,
                  guint           dat_len,
-                 const gchar   * idx)
+                 const char    * idx,
+                 int           * discarded_qty)
 {
 	gint version;
 	GPtrArray * groups;
@@ -126,18 +121,20 @@ read_group_data (Server        * server,
 	debug_enter ("read_group_data");
 
 	/* sanity clause */
-	g_return_val_if_fail (server_is_valid(server), 0);
+	g_return_val_if_fail (server_is_valid (server), 0);
 	g_return_val_if_fail (dat_str!=NULL, 0);
 	g_return_val_if_fail (idx!=NULL, 0);
+	g_return_val_if_fail (discarded_qty!=NULL, 0);
 
+	*discarded_qty = 0;
 	version = get_next_token_int (march_idx, '\n', &march_idx);
 	groups = g_ptr_array_new ();
 	if (version==1 || version==2 || version==3 || version==4 || 
 	    version==5 || version==6 || version==7 || version==8 ||
-	    version==9 || version==10)
+	    version==9 || version==10 || version==11 || version==12)
 	{
 		long i;
-		glong qty = get_next_token_long (march_idx, '\n', &march_idx);
+		long qty = get_next_token_long (march_idx, '\n', &march_idx);
 
 		pan_g_ptr_array_reserve (groups, qty);
 
@@ -155,10 +152,9 @@ read_group_data (Server        * server,
 			l = get_next_token_long (march_idx, '\n', &march_idx);
 			g = group_new (server, dat_str+l);
 
-			/* readable name */
-			l = get_next_token_long (march_idx, '\n', &march_idx);
-			if (0<=l && l<dat_len)
-				g->readable_name = g_strdup (dat_str+l);
+			/* readable name removed in version 12 */
+			if (version<12)
+				skip_next_token (march_idx, '\n', &march_idx);
 
 			/* description */
 			l = get_next_token_long (march_idx, '\n', &march_idx);
@@ -170,6 +166,13 @@ read_group_data (Server        * server,
 			if (0<=l && l<dat_len)
 				g->download_dir = g_strdup (dat_str+l);
 
+			/* default charset added in version 11 */
+			if (version>=11) {
+				l = get_next_token_long (march_idx, '\n', &march_idx);
+				if (0<=l && l<dat_len)
+					g->default_charset = g_strdup (dat_str+l);
+			}
+
 			/* newsrc string */
 			newsrc = NULL;
 			l = get_next_token_long (march_idx, '\n', &march_idx);
@@ -261,7 +264,10 @@ read_group_data (Server        * server,
 			if (is_nonempty_string(purged))
 				newsrc_init (group_get_purged(g), purged, g->article_low, g->article_high);
 
-			g_ptr_array_add (groups, g);
+			if (group_is_valid (g))
+				g_ptr_array_add (groups, g);
+			else
+				++(*discarded_qty);
 
 			if (status != NULL) {
 				status_item_emit_next_step (status);
@@ -273,7 +279,7 @@ read_group_data (Server        * server,
 	}
 	else /* unsupported version */
 	{
-		log_add_va (LOG_ERROR, _("Unsupported data version %d for `%s' data file.\nAre you running an old version of Pan by accident?"),
+		log_add_va (LOG_ERROR, _("Unsupported data version %d for \"%s\" data file.\nAre you running an old version of Pan by accident?"),
 			version,
 			server_get_name (server));
 	}
@@ -295,40 +301,40 @@ read_group_data (Server        * server,
 static gint
 file_grouplist_load_file (Server * server, StatusItem * status, const gchar * filemarker)
 {
-	GArray * dat = NULL;
-	GArray * idx = NULL;
-	gint group_qty = 0;
+	char * dat = NULL;
+	char * idx = NULL;
+	gsize dat_len = 0;
+	gsize idx_len = 0;
+	int group_qty = 0;
+	char idx_path [PATH_MAX];
+	char dat_path [PATH_MAX];
 	debug_enter ("file_grouplist_load_file");
 
 	/* sanity clause */
 	g_return_val_if_fail (server_is_valid (server), 0);
 	g_return_val_if_fail (filemarker!=NULL, 0);
-	g_return_val_if_fail (is_nonempty_string(data_dir), 0);
 
 	/* open the index file */
-	if (1) {
-		gchar path [PATH_MAX];
-		g_snprintf (path, sizeof(path), "%s%c%s%s.idx", data_dir, G_DIR_SEPARATOR, server_get_name (server), filemarker);
-		pan_normalize_filename_inplace (path);
-		idx = read_file (path);
-	}
+	g_snprintf (idx_path, sizeof(idx_path), "%s%c%s%s.idx", get_data_dir(), G_DIR_SEPARATOR, server_get_name (server), filemarker);
+	pan_file_normalize_inplace (idx_path);
+	g_file_get_contents (idx_path, &idx, &idx_len, NULL);
 
 	/* open the data file */
-	if (1) {
-		gchar path [PATH_MAX];
-		g_snprintf (path, sizeof(path), "%s%c%s%s.dat", data_dir, G_DIR_SEPARATOR, server_get_name (server), filemarker);
-		pan_normalize_filename_inplace (path);
-		dat = read_file (path);
-	}
+	g_snprintf (dat_path, sizeof(dat_path), "%s%c%s%s.dat", get_data_dir(), G_DIR_SEPARATOR, server_get_name (server), filemarker);
+	pan_file_normalize_inplace (dat_path);
+	g_file_get_contents (dat_path, &dat, &dat_len, NULL);
 
 	/* read the groups */
-	if (dat!=NULL && dat->len!=0 && idx!=NULL && idx->len!=0)
-		group_qty = read_group_data (server, status, dat->data, dat->len, idx->data);
-
+	if (dat!=NULL && dat_len!=0 && idx!=NULL && idx_len!=0) {
+		int discarded_qty = 0;
+		group_qty = read_group_data (server, status, dat, dat_len, idx, &discarded_qty);
+		if (discarded_qty > 0)
+			log_add_va (LOG_URGENT|LOG_ERROR, _("Skipped %d groups due to possible corruption in \"%s\" and \"%s\".  You may want to delete these files and get a new grouplist from your news server."), discarded_qty, idx_path, dat_path);
+	}
 
 	/* cleanup */
-	if (dat!=NULL) g_array_free (dat, TRUE);
-	if (idx!=NULL) g_array_free (idx, TRUE);
+	g_free (dat);
+	g_free (idx);
 
 	debug_exit ("file_grouplist_load_file");
 	return group_qty;
@@ -337,14 +343,17 @@ file_grouplist_load_file (Server * server, StatusItem * status, const gchar * fi
 void
 file_grouplist_load (Server * server, ServerGroupsType group_type, StatusItem * status)
 {
-	struct timeval start;
+	GTimeVal start;
 	int group_qty = 0;
 	debug_enter ("file_grouplist_load");
 
-	g_return_if_fail (server_is_valid(server));
+	g_return_if_fail (server_is_valid (server));
 	g_return_if_fail (group_type!=0);
 
-	gettimeofday (&start, NULL);
+	if (!pan_strcmp (server->name, INTERNAL_SERVER_NAME))
+		return;
+
+	g_get_current_time (&start);
 
 	if (file_grouplist_097_style_exists (server))
 	{
@@ -364,13 +373,13 @@ file_grouplist_load (Server * server, ServerGroupsType group_type, StatusItem *
 	/* stats */	
 	if (group_qty > 0)
 	{
-		struct timeval finish;
+		GTimeVal finish;
 		double diff;
 
-		gettimeofday (&finish, NULL);
+		g_get_current_time (&finish);
 		diff = finish.tv_sec - start.tv_sec;
-		diff += (finish.tv_usec - start.tv_usec)/1000000.0;
-		log_add_va (LOG_INFO, _("Loaded %d groups for server `%s' in %.1f seconds (%.0f groups/sec)"),
+		diff += (finish.tv_usec - start.tv_usec)/(double)G_USEC_PER_SEC;
+		log_add_va (LOG_INFO, _("Loaded %d groups for server \"%s\" in %.1f seconds (%.0f groups/sec)"),
 			group_qty,
 			server_get_name (server),
 			diff,
@@ -403,7 +412,7 @@ write_groups (FILE * idx_fp, FILE * dat_fp, GPtrArray * groups, StatusItem * sta
 
 	/* Write the group information... */
 	pos = 0;
-	fprintf (idx_fp, "10\n"); /* file format version number */
+	fprintf (idx_fp, "12\n"); /* file format version number */
 	fprintf (idx_fp, "%u\n", groups->len); /* number of groups */
 	for (i=0; i!=groups->len; ++i)
 	{
@@ -419,9 +428,9 @@ write_groups (FILE * idx_fp, FILE * dat_fp, GPtrArray * groups, StatusItem * sta
 
 		/* write the group name */
 		const long name_idx = get_string_offset (NULL, NULL, group->name, dat_fp, &pos);
-		const long read_idx = get_string_offset (NULL, NULL, group->readable_name, dat_fp, &pos);
 		const long desc_idx = get_string_offset (NULL, NULL, group->description, dat_fp, &pos);
 		const long path_idx = get_string_offset (NULL, NULL, group->download_dir, dat_fp, &pos);
+		const long char_idx = get_string_offset (NULL, NULL, group->default_charset, dat_fp, &pos);
 		const long news_idx = get_string_offset (NULL, NULL, pch?pch:"", dat_fp, &pos);
 		const long dead_idx = get_string_offset (NULL, NULL, purged?purged:"", dat_fp, &pos);
 		const long filt_idx = get_string_offset (NULL, NULL, group->filter_name, dat_fp, &pos);
@@ -431,7 +440,7 @@ write_groups (FILE * idx_fp, FILE * dat_fp, GPtrArray * groups, StatusItem * sta
 		fprintf (idx_fp,
 		         "%ld\n" "%ld\n" "%ld\n" "%ld\n" "%ld\n" "%ld\n" "%ld\n" "%ld\n"
 		         "%u\n" "%lu\n" "%lu\n" "%d\n" "%d\n" "%d\n" "%lu\n" "%lu\n" "%lu\n" "%d\n" "%d\n" "%c\n",
-		         name_idx, read_idx, desc_idx, path_idx, news_idx, dead_idx, filt_idx, iden_idx,
+		         name_idx, desc_idx, path_idx, char_idx, news_idx, dead_idx, filt_idx, iden_idx,
 		         (unsigned)(group->flags & ~GROUP_NEW),
 		         group->filter_show,
 		         (gulong)group->filter_bits,
@@ -478,18 +487,18 @@ save_groups_to_file (Server * server, GPtrArray * groups, gchar * filemarker, St
 	if (groups->len == 0)
 	{
 		gchar path [PATH_MAX];
-		g_snprintf (path, PATH_MAX, "%s%c%s_%s.idx", data_dir, G_DIR_SEPARATOR, server_get_name (server), filemarker);
-		unlink (path);
-		g_snprintf (path, PATH_MAX, "%s%c%s_%s.dat", data_dir, G_DIR_SEPARATOR, server_get_name (server), filemarker);
-		unlink (path);
+		g_snprintf (path, PATH_MAX, "%s%c%s_%s.idx", get_data_dir(), G_DIR_SEPARATOR, server_get_name (server), filemarker);
+		remove (path);
+		g_snprintf (path, PATH_MAX, "%s%c%s_%s.dat", get_data_dir(), G_DIR_SEPARATOR, server_get_name (server), filemarker);
+		remove (path);
 	}
 	else
 	{
-		gchar path [PATH_MAX];
 		FILE * idx_fp = NULL;
 		FILE * dat_fp = NULL;
-		gchar * idx_path = pan_make_temp (&idx_fp);
-		gchar * dat_path = pan_make_temp (&dat_fp);
+		char * idx_path = pan_file_make_temp (&idx_fp);
+		char * dat_path = pan_file_make_temp (&dat_fp);
+		gboolean ok;
 
 		/* make sure that we were able to create the temp files */
 		if (idx_fp==NULL || dat_fp==NULL)
@@ -503,39 +512,56 @@ save_groups_to_file (Server * server, GPtrArray * groups, gchar * filemarker, St
 
 		/* save the groups */
 		write_groups (idx_fp, dat_fp, groups, status);
+		ok = !ferror(idx_fp) && !ferror(dat_fp);
 		fclose (idx_fp);
 		fclose (dat_fp);
 
-		/* move the tmp files to the real thing */
-		g_snprintf (path, PATH_MAX, "%s%c%s_%s.idx", data_dir, G_DIR_SEPARATOR, server_get_name (server), filemarker);
-		pan_normalize_filename_inplace (path);
-		pan_rename (idx_path, path);
+		/* rename the temp files (on success) or delete them (on error) */
+		if (!ok)
+		{
+			log_add_va (LOG_ERROR, _("Error saving list of groups to \"%s\" - is the disk full?"), get_data_dir());
+			unlink (idx_path);
+			unlink (dat_path);
+		}
+		else
+		{
+			char path [PATH_MAX];
+
+			g_snprintf (path, PATH_MAX, "%s%c%s_%s.idx", get_data_dir(), G_DIR_SEPARATOR, server_get_name (server), filemarker);
+			pan_file_normalize_inplace (path);
+			pan_file_rename (idx_path, path);
+
+			g_snprintf (path, PATH_MAX, "%s%c%s_%s.dat", get_data_dir(), G_DIR_SEPARATOR, server_get_name (server), filemarker);
+			pan_file_normalize_inplace (path);
+			pan_file_rename (dat_path, path);
+		}
+
+		/* cleanup */
 		g_free (idx_path);
-		g_snprintf (path, PATH_MAX, "%s%c%s_%s.dat", data_dir, G_DIR_SEPARATOR, server_get_name (server), filemarker);
-		pan_normalize_filename_inplace (path);
-		pan_rename (dat_path, path);
 		g_free (dat_path);
-		debug_exit ("save_groups_to_file");
 	}
+
+	debug_exit ("save_groups_to_file");
 }
 
 void
 file_grouplist_save (Server * server, ServerGroupsType group_type, StatusItem * status)
 {
 	gint save_qty = 0;
-	struct timeval start;
+	GTimeVal start;
 	debug_enter ("file_grouplist_save");
 
 	/* sanity clause */
-	g_return_if_fail (server_is_valid(server));
-	g_return_if_fail (is_nonempty_string(data_dir));
+	g_return_if_fail (server_is_valid (server));
 	g_return_if_fail ((group_type|SERVER_GROUPS_SUBSCRIBED)||(group_type|SERVER_GROUPS_UNSUBSCRIBED));
 
+	if (!pan_strcmp (server->name, INTERNAL_SERVER_NAME))
+		return;
 	/**
 	***  Save the Groups
 	**/
 
-	gettimeofday (&start, NULL);
+	g_get_current_time (&start);
 
 	if (group_type & SERVER_GROUPS_SUBSCRIBED)
 	{
@@ -555,12 +581,12 @@ file_grouplist_save (Server * server, ServerGroupsType group_type, StatusItem *
 	/* finish timing stats */
 	if (save_qty > 0)
 	{
-		struct timeval finish;
+		GTimeVal finish;
 		double diff;
 
-		gettimeofday (&finish, NULL);
+		g_get_current_time (&finish);
 		diff = finish.tv_sec - start.tv_sec;
-		diff += (finish.tv_usec - start.tv_usec)/1000000.0;
+		diff += (finish.tv_usec - start.tv_usec)/(double)G_USEC_PER_SEC;
 		log_add_va (LOG_INFO, _("Saved %d groups in \"%s\" in %.1f seconds (%.0f groups/sec)"),
 			save_qty,
 			server_get_name (server),
