@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #include <config.h>
@@ -26,12 +23,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <sys/time.h>
 
 #include <glib.h>
 
 #include <gmime/memchunk.h>
 
+#include <pan/base/acache.h>
 #include <pan/base/article.h>
 #include <pan/base/base-prefs.h>
 #include <pan/base/debug.h>
@@ -49,29 +46,83 @@
 ****
 ***/
 
+typedef struct
+{
+	Group * folder;
+	GPtrArray * articles;
+}
+FolderForeachStruct;
+
+static void
+file_headers_load_folder_foreach (GMimeMessage * message, gpointer user_data)
+{
+	Article * article;
+	FolderForeachStruct * ffs = (FolderForeachStruct*) user_data;
+
+	article = article_new (ffs->folder);
+	article_set_from_g_mime_message (article, message);
+	article->number = ffs->articles->len + 1;
+	g_ptr_array_add (ffs->articles, article);
+}
+
+static gboolean
+file_headers_load_folder (Group * folder, StatusItem * status)
+{
+	FolderForeachStruct ffs;
+	debug_enter ("file_headers_load_folder");
+
+	/* sanity clause */
+	g_return_val_if_fail (group_is_valid(folder), FALSE);
+	g_return_val_if_fail (group_is_folder(folder), FALSE);
+
+	/* prep for foreach */
+	ffs.folder = folder;
+	ffs.articles = g_ptr_array_new ();
+	acache_path_foreach (folder->name, file_headers_load_folder_foreach, &ffs);
+
+	/* if we've got articles then add them; otherwise; clean up */
+	if (ffs.articles->len != 0)
+		group_init_articles (folder, ffs.articles, status);
+	else if (folder->_articles) {
+		g_hash_table_destroy (folder->_articles);
+		folder->_articles = NULL;
+	}
+	folder->articles_dirty = FALSE;
+
+	g_ptr_array_free (ffs.articles, TRUE);
+	debug_exit ("file_headers_load_folder");
+	return folder->_articles!=NULL && g_hash_table_size(folder->_articles)!=0;
+}
+
 static gboolean
 file_headers_load_group (Group * group, StatusItem * status)
 {
 	gboolean success = FALSE;
-        gchar * path;
-	GArray *dat, *idx;
+	char * dat = NULL;
+	char * idx = NULL;
+	size_t dat_len = 0;
+	size_t idx_len = 0;
+	char path[PATH_MAX];
 	debug_enter ("file_headers_load_group");
 
 	g_return_val_if_fail (group_is_valid (group), FALSE);
-	g_return_val_if_fail (is_nonempty_string(data_dir), FALSE);
 
 	if (status!=NULL)
-		status_item_emit_status_va (status, _("Loading %s"), group_get_readable_name(group));
+		status_item_emit_status_va (status, _("Loading group \"%s\""), group_get_name(group));
 
 	/* open the index file */
-	path = g_strdup_printf ("%s%c%s%c%s.idx", data_dir, G_DIR_SEPARATOR, server_get_name (group->server), G_DIR_SEPARATOR, group->name);
-	idx = read_file (path);
-	g_free (path);
+	g_snprintf (path, sizeof(path), "%s%c%s%c%s.idx",
+	            get_data_dir(), G_DIR_SEPARATOR,
+	            server_get_name (group->server), G_DIR_SEPARATOR,
+	            group->name);
+	g_file_get_contents (path, &idx, &idx_len, NULL);
 
 	/* open the data file */
-	path = g_strdup_printf ("%s%c%s%c%s.dat", data_dir, G_DIR_SEPARATOR, server_get_name (group->server), G_DIR_SEPARATOR, group->name);
-	dat = read_file (path);
-	g_free (path);
+	g_snprintf (path, sizeof(path), "%s%c%s%c%s.dat",
+	            get_data_dir(), G_DIR_SEPARATOR,
+	            server_get_name (group->server), G_DIR_SEPARATOR,
+	            group->name);
+	g_file_get_contents (path, &dat, &dat_len, NULL);
 
 	/* allocate the articles array */
 	pan_warn_if_fail (group->_articles==NULL);
@@ -79,53 +130,54 @@ file_headers_load_group (Group * group, StatusItem * status)
 
 	if (dat!=NULL && idx!=NULL)
 	{
-		const gchar * march = idx->data;
+		const char * march = idx;
 		const glong version = get_next_token_int (march, '\n', &march);
 
-		if (version==1 || version==2 || version==3 || version==4 || version==5)
+		if (version==1 || version==2 || version==3 || version==4 || version==5 || version==6 || version==7 || version==8)
 		{
 			int i;
+			int purged_article_count = 0;
 			long l;
 			const long qty = get_next_token_long (march, '\n', &march);
-			GPtrArray * addme = g_ptr_array_new ();
-			gint purged_article_count = 0;
+			GPtrArray * addme;
 
+			/* set the "one big chunk" */
 			pan_warn_if_fail (group->_one_big_chunk == NULL);
-			group->_one_big_chunk = dat->data;
+			group->_one_big_chunk = dat;
 
+			/* load the articles */
+		       	addme = g_ptr_array_sized_new (qty);
 			for (i=0; i!=qty; ++i)
 			{
 				Article * a = article_new (group);
-				int extra_header_qty;
-				int j;
 
 				/* message id */
 				l = get_next_token_long (march, '\n', &march);
-				if (0<=l && l<dat->len)
-					a->message_id = dat->data + l;
+				if (0<=l && l<dat_len)
+					a->message_id = dat + l;
 
 				/* author */
 				if (version<2) /* version 2 split author into 2 fields */
 				{
 					l = get_next_token_long (march, '\n', &march);
-					if (0<=l && l<dat->len)
-						article_init_author_from_header (a, dat->data+l);
+					if (0<=l && l<dat_len)
+						article_init_author_from_header (a, dat+l);
 				}
 				else
 				{
 					l = get_next_token_long (march, '\n', &march);
-					if (0<=l && l<dat->len)
-						a->author_addr = dat->data + l;
+					if (0<=l && l<dat_len)
+						a->author_addr = dat + l;
 
 					l = get_next_token_long (march, '\n', &march);
-					if (0<=l && l<dat->len)
-						a->author_real = dat->data + l;
+					if (0<=l && l<dat_len)
+						a->author_real = dat + l;
 				}
 
 				/* subject */
 				l = get_next_token_long (march, '\n', &march);
-				if (0<=l && l<dat->len)
-					a->subject = dat->data + l;
+				if (0<=l && l<dat_len)
+					a->subject = dat + l;
 
 				/* date string - removed in version 3 */
 				if (version<3)
@@ -133,22 +185,29 @@ file_headers_load_group (Group * group, StatusItem * status)
 
 				/* references */
 				l = get_next_token_long (march, '\n', &march);
-				if (0<=l && l<dat->len)
-					a->references = dat->data + l;
+				if (0<=l && l<dat_len)
+					a->references = dat + l;
 
 				/* xrefs added in version 4 */
 				if (version>=4) {
 					l = get_next_token_long (march, '\n', &march);
-					if (0<=l && l<dat->len)
-						a->xref = dat->data + l;
+					if (0<=l && l<dat_len)
+						a->xref = dat + l;
 				}
 
 				/* numeric fields */
 				a->part           = (gint16) get_next_token_int (march, '\n', &march);
 				a->parts          = (gint16) get_next_token_int (march, '\n', &march);
 				a->linecount      = (guint16) get_next_token_int (march, '\n', &march);
-				a->crosspost_qty  = (gint8) get_next_token_int (march, '\n', &march);
-				a->state          = (guint16) get_next_token_int (march, '\n', &march);
+
+				/* crosspost_qty - removed in version 6 */
+				if (version<6)
+					skip_next_token (march, '\n', &march);
+
+				/* state - removed in version 6, back in 7 */
+				if (version!=6)
+					a->state  = (guint16) get_next_token_int (march, '\n', &march);
+
 				a->date           = (time_t) get_next_token_ulong (march, '\n', &march);
 				a->number         = (gulong) get_next_token_ulong (march, '\n', &march);
 
@@ -162,17 +221,9 @@ file_headers_load_group (Group * group, StatusItem * status)
 					a->is_new = b;
 				}
 
-				/* extra headers */
-				extra_header_qty  = get_next_token_int (march, '\n', &march);
-				for (j=0; j<extra_header_qty; ++j)
-				{
-					const glong key_idx = get_next_token_long (march, '\n', &march);
-					const glong val_idx = get_next_token_long (march, '\n', &march);
-					const gchar * key = 0<=key_idx && key_idx<dat->len ? dat->data+key_idx : NULL;
-					const gchar * val = 0<=val_idx && val_idx<dat->len ? dat->data+val_idx : NULL;
-					if (key!=NULL && val!=NULL)
-						article_init_header (a, key, val, 0);
-				}
+				/* extra headers removed in version 8 */
+				if (version<8)
+					skip_next_token (march, '\n', &march);
 
 				/* let the user know what we're doing */
 				if (status != NULL) {
@@ -193,9 +244,11 @@ file_headers_load_group (Group * group, StatusItem * status)
 			if (purged_article_count != 0)
 			{
 				log_add_va (LOG_ERROR,
-					_("Pan skipped %d corrupt headers from the local cache for group %s.\nYou may want to empty this group and download fresh headers."),
+					_("Pan skipped %d corrupt headers from the local cache for group \"%s\"."),
 					purged_article_count,
-					group_get_readable_name(group));
+					group_get_name(group));
+				log_add (LOG_ERROR,
+					_("You may want to empty this group and download fresh headers."));
 			}
 
 			group_init_articles (group, addme, status);
@@ -211,127 +264,32 @@ file_headers_load_group (Group * group, StatusItem * status)
 		}
 	}
 
-	if (dat != NULL) g_array_free (dat, !success);
-	if (idx != NULL) g_array_free (idx, TRUE);
+	/* cleanup */
+	g_free (idx);
+	if (!success)
+		g_free (dat);
 
 	debug_exit ("file_headers_load_group");
 	return success;
 }
 
-static gchar*
-get_mbox_filename (const Group * folder)
-{
-	gchar * retval;
-	debug_enter ("get_mbox_filename");
-
-	g_return_val_if_fail (folder!=NULL, NULL);
-	g_return_val_if_fail (server_is_valid (folder->server), NULL);
-	g_return_val_if_fail (group_is_folder(folder), NULL);
-	g_return_val_if_fail (is_nonempty_string(data_dir), NULL);
-
-	retval = g_strdup_printf ("%s%c%s%c%s.mbox", 
-			data_dir,
-	                G_DIR_SEPARATOR,
-	                server_get_name (folder->server),
-	                G_DIR_SEPARATOR,
-	                folder->name);
-
-	debug_exit ("get_mbox_filename");
-	return retval;
-}
-
-static gboolean
-file_headers_load_folder (Group * folder, StatusItem * status)
-{
-	gint i;
-	gchar * text;
-	const gchar * march;
-	GPtrArray * articles;
-	GString * msg;
-	debug_enter ("file_headers_load_folder");
-
-	g_return_val_if_fail (folder!=NULL, FALSE);
-	g_return_val_if_fail (group_is_folder(folder), FALSE);
-
-	/* get the mbox file */
-	{
-		GArray * array;
-		gchar * fname = get_mbox_filename (folder);
-		array = read_file (fname);
-		g_free (fname);
-		if (array == NULL)
-			return FALSE;
-		text = array->data;
-		g_array_free (array, FALSE);
-	}
-
-	/* allocate the articles array */
-	pan_warn_if_fail (folder->_articles == NULL);
-	group_get_articles (folder);
-
-	/* walk through the mbox */
-	i = 0;
-	march = text;
-	articles = g_ptr_array_new ();
-	msg = g_string_sized_new (2048);
-	while (march != NULL)
-	{
-		gchar * from = NULL;
-
-		/* parse the next mbox message */
-		march = mbox_get_next_message (march, &from, msg);
-		if (msg->len != 0)
-		{
-			Article * a = article_new (folder);
-			a->number = ++i;
-			article_set_from_raw_message (a, msg->str);
-
-			/* keep the good ones */
-			if (article_is_valid (a))
-				g_ptr_array_add (articles, a);
-		}
-
-		g_free (from);
-	}
-	g_string_free (msg, TRUE);
-
-	/* if we've got articles then add them; otherwise, clean up */
-	if (articles->len != 0)
-		group_init_articles (folder, articles, status);
-	else {
-		g_hash_table_destroy (folder->_articles);
-		folder->_articles = NULL;
-	}
-	folder->articles_dirty = FALSE;
-
-	/* cleanup */
-	g_free (text);
-
-	debug_exit ("file_headers_load_folder");
-	return folder->_articles!=NULL && g_hash_table_size(folder->_articles)!=0;
-}
 
 void
 file_headers_load (Group * group, StatusItem * status)
 {
-	gint size;
-	struct timeval start;
-	struct timeval finish;
-	double diff;
 	gboolean success = FALSE;
+	int size;
+	double diff;
+	GTimeVal start;
+	GTimeVal finish;
 	debug_enter ("file_headers_load");
 
 	/* start the stopwatch */
-	gettimeofday (&start, NULL);
+	g_get_current_time (&start);
 
-	if (group_is_folder (group))
-	{
-		success = file_headers_load_folder (group, status);
-
-
-	}
-	if (!success)
-		success = file_headers_load_group (group, status);
+	success = group_is_folder(group)
+		? file_headers_load_folder (group, status)
+		: file_headers_load_group (group, status);
 
 	/* expire the old articles, if any */
 	if (success)
@@ -344,14 +302,14 @@ file_headers_load (Group * group, StatusItem * status)
 	}
 
 	/* timing stats */
-	gettimeofday (&finish, NULL);
+	g_get_current_time (&finish);
 	diff = finish.tv_sec - start.tv_sec;
-	diff += (finish.tv_usec - start.tv_usec)/1000000.0;
+	diff += (finish.tv_usec - start.tv_usec)/(double)G_USEC_PER_SEC;
 	size = g_hash_table_size(group_get_articles(group));
 	if (size != 0)
-		log_add_va (LOG_INFO, _("Loaded %d articles for group `%s' in %.1f seconds (%.0f art/sec)"),
+		log_add_va (LOG_INFO, _("Loaded %d articles for group \"%s\" in %.1f seconds (%.0f art/sec)"),
 			size,
-			group_get_readable_name(group),
+			group_get_name(group),
 			diff,
 			size/(fabs(diff)<0.001?0.001:diff));
 
@@ -366,7 +324,7 @@ file_headers_load (Group * group, StatusItem * status)
 long
 get_string_offset (GHashTable   * hash,
                    MemChunk     * long_chunk,
-                   const gchar  * str,
+                   const char   * str,
                    FILE         * fp,
                    long         * pos)
 {
@@ -401,127 +359,23 @@ get_string_offset (GHashTable   * hash,
 }
 
 
-static gboolean
-file_headers_save_folder (Group * folder, StatusItem * status)
-{
-	FILE * fp = NULL;
-	gchar * fname = NULL;
-	gchar * fname_tmp = NULL;
-	gboolean ok = TRUE;
-	debug_enter ("file_headers_save_folder");
-
-	/* sanity clause */
-	g_return_val_if_fail (folder!=NULL, FALSE);
-	g_return_val_if_fail (group_is_folder(folder), FALSE);
-
-	/* make sure the directory exists */
-	if (ok) {
-		gchar * pch = g_strdup_printf ("%s%c%s", data_dir, G_DIR_SEPARATOR, server_get_name (folder->server));
-		if (!directory_check (pch)) {
-			log_add_va (LOG_ERROR, _("Can't create directory \"%s\""), pch);
-			ok = FALSE;
-		}
-		g_free (pch);
-	}
-
-	/* get the filenames & start writing to the outfile */
-	if (ok) {
-		fname = get_mbox_filename (folder);
-		fname_tmp = g_strdup_printf ("%s.tmp", fname);
-		fp = fopen (fname_tmp, "w+");
-		if (fp == NULL) {
-			log_add_va (LOG_ERROR, _("The group will not be saved -- can't create file \"%s\""), fname_tmp);
-			ok = FALSE;
-		}
-	}
-
-	/* save the articles to the mbox */
-	if (ok) {
-		guint i = 0;
-		GPtrArray * array = group_get_article_array (folder);
-		GString * s = g_string_new (NULL);
-
-		for (i=0; i!=array->len; ++i) {
-			Article * a = ARTICLE(g_ptr_array_index(array,i));
-			gchar buf[32], *pch, *mbox;
-
-			/* add headers */
-			pch = article_get_headers (a);
-			g_string_assign (s, pch);
-			g_free (pch);
-
-			/* add Status: header */
-			*(pch=buf)='\0';
-			if (article_is_read(a)) *pch++ = 'R';
-			if (!article_is_new(a)) *pch++ = 'O';
-			*pch = '\0';
-			if (*buf != '\0')
-				g_string_sprintfa (s, "Status: %s\n", buf);
-  
-			/* add header/body separator */ 
-			g_string_append (s, "\n");
-  
-			/* add body */ 
-			pch = article_get_body (a);
-			if (is_nonempty_string(pch))
-				g_string_append (s, pch);
-			g_free (pch);
-
-			/* write the message in mbox format */
-			mbox = mbox_format_message (s->str, a->author_addr, a->date);
-			if (mbox != NULL) {
-				if (fputs (mbox, fp) == EOF) {
-					ok = FALSE;
-					break;
-				}
-				g_free (mbox);
-			}
-
-			pan_warn_if_fail (article_is_valid(a)); /* FIXME: just testing, remove later */
-		}
-		fclose (fp);
-
-		/* cleanup */
-		g_ptr_array_free (array, TRUE);
-		g_string_free (s, TRUE);
-	}
-
-	/* wrote to tmp file okay... now make it permanent */
-	if (ok) {
-		if (!pan_rename (fname_tmp, fname)) {
-			log_add_va (LOG_ERROR, _("The group will not be saved -- can't move \"%s\" to \"%s\""),
-			fname_tmp, fname);
-			ok = FALSE;
-		}
-	}
-
-	g_free (fname);
-	g_free (fname_tmp);
-	debug_exit ("file_headers_save_folder");
-	return ok;
-}
-
 static gboolean
 file_headers_save_group (Group * group, StatusItem * status)
 {
 	guint i;
 	FILE * idx_fp;
 	FILE * dat_fp;
-	gchar * pch;
-	gchar * idx_path;
-	gchar * dat_path;
+	char * pch;
+	char * idx_path;
+	char * dat_path;
 	long pos = 0;
-	GHashTable * hash_name = NULL;
-	GHashTable * hash_mail = NULL;
-	GHashTable * hash_refs = NULL;
-	GHashTable * hash_misc = NULL;
+	GHashTable * hash = NULL;
 	MemChunk * long_chunk = NULL;
 	GPtrArray * articles;
 	gboolean ok = TRUE;
 	debug_enter ("file_headers_save_group");
 
-	g_return_val_if_fail (group_is_valid(group), FALSE);
-	g_return_val_if_fail (is_nonempty_string(data_dir), FALSE);
+	g_return_val_if_fail (group_is_valid (group), FALSE);
 
 	/* if nothing to save */
 	articles = group_get_article_array (group);
@@ -536,8 +390,8 @@ file_headers_save_group (Group * group, StatusItem * status)
 	**/
 
 	if (1) {
-		gchar * pch = g_strdup_printf ("%s%c%s", data_dir, G_DIR_SEPARATOR, server_get_name (group->server));
-		if (!directory_check (pch))
+		char * pch = g_strdup_printf ("%s%c%s", get_data_dir(), G_DIR_SEPARATOR, server_get_name (group->server));
+		if (!pan_file_ensure_path_exists (pch))
 			log_add_va (LOG_ERROR, _("Can't create directory \"%s\""), pch);
 		g_free (pch);
 	}
@@ -545,7 +399,7 @@ file_headers_save_group (Group * group, StatusItem * status)
 
 	/* open index file */
 	idx_path = g_strdup_printf ("%s%c%s%c%s.idx.tmp",
-		data_dir, G_DIR_SEPARATOR,
+		get_data_dir(), G_DIR_SEPARATOR,
 		server_get_name (group->server), G_DIR_SEPARATOR,
 		group->name);
 	idx_fp = fopen (idx_path, "w+");
@@ -557,7 +411,7 @@ file_headers_save_group (Group * group, StatusItem * status)
 
 	/* open data file */
 	dat_path = g_strdup_printf ("%s%c%s%c%s.dat.tmp",
-		data_dir, G_DIR_SEPARATOR,
+		get_data_dir(), G_DIR_SEPARATOR,
 		server_get_name (group->server), G_DIR_SEPARATOR,
 		group->name);
 	dat_fp = fopen (dat_path, "w+");
@@ -571,28 +425,23 @@ file_headers_save_group (Group * group, StatusItem * status)
 	}
 
 	/* Write DATBASE_VERSION */
-	fprintf (idx_fp, "5\n%ld\n", (long)articles->len);
+	fprintf (idx_fp, "8\n%ld\n", (long)articles->len);
 
 	/* Write the article information... */
 	pos = 0;
-	hash_name = g_hash_table_new (g_str_hash, g_str_equal);
-	hash_mail = g_hash_table_new (g_str_hash, g_str_equal);
-	hash_refs = g_hash_table_new (g_str_hash, g_str_equal);
-	hash_misc = g_hash_table_new (g_str_hash, g_str_equal);
+	hash = g_hash_table_new (g_str_hash, g_str_equal);
 	long_chunk = memchunk_new (sizeof(glong), 4096, FALSE);
 	for (i=0; i!=articles->len; ++i)
 	{
 		Article * a = ARTICLE(g_ptr_array_index (articles, i));
-		GPtrArray * extra_headers = article_get_all_headers (a);
-		const gchar * subject = article_get_subject (a);
-		const gchar * message_id = article_get_message_id (a);
-		guint j;
-		const long id_idx          = get_string_offset (NULL,      NULL,       message_id,     dat_fp, &pos);
-		const long author_addr_idx = get_string_offset (hash_mail, long_chunk, a->author_addr, dat_fp, &pos);
-		const long author_real_idx = get_string_offset (hash_name, long_chunk, a->author_real, dat_fp, &pos);
-		const long subj_idx        = get_string_offset (hash_misc, long_chunk, subject,        dat_fp, &pos);
-		const long refs_idx        = get_string_offset (hash_refs, long_chunk, a->references,  dat_fp, &pos);
-		const long xref_idx        = get_string_offset (hash_refs, long_chunk, a->xref      ,  dat_fp, &pos);
+		const char * subject = article_get_subject (a);
+		const char * message_id = article_get_message_id (a);
+		const long id_idx          = get_string_offset (hash, long_chunk, message_id,     dat_fp, &pos);
+		const long author_addr_idx = get_string_offset (hash, long_chunk, a->author_addr, dat_fp, &pos);
+		const long author_real_idx = get_string_offset (hash, long_chunk, a->author_real, dat_fp, &pos);
+		const long subj_idx        = get_string_offset (hash, long_chunk, subject,        dat_fp, &pos);
+		const long refs_idx        = get_string_offset (hash, long_chunk, a->references,  dat_fp, &pos);
+		const long xref_idx        = get_string_offset (hash, long_chunk, a->xref,        dat_fp, &pos);
 
 		pan_warn_if_fail (a->number != 0);
 
@@ -601,70 +450,43 @@ file_headers_save_group (Group * group, StatusItem * status)
 			"%ld\n" "%ld\n" "%ld\n" "%ld\n" "%ld\n" "%ld\n"
 			"%d\n" "%d\n"
 			"%u\n"
-			"%d\n"
-			"%d\n"
+			"%u\n"
 			"%lu\n"
 			"%lu\n"
 			"%d\n",
 			id_idx, author_addr_idx, author_real_idx, subj_idx, refs_idx, xref_idx,
 			(int)a->part, (int)a->parts,
 			(unsigned int)a->linecount,
-			(int)a->crosspost_qty,
 			(int)a->state,
 			(unsigned long)a->date,
 			(unsigned long)a->number,
 			(int)(a->is_new != 0));
-
-		/* make sure the extra headers are in pairs */
-		if ((extra_headers->len % 2) != 0) {
-			pan_warn_if_reached ();
-			g_ptr_array_set_size (extra_headers, 0);
-		}
-
-		/* remove extra headers that we don't want to save */
-		for (j=0; j<extra_headers->len; ) {
-			const gchar * key = (const gchar*) g_ptr_array_index (extra_headers, j);
-			const gchar * val = (const gchar*) g_ptr_array_index (extra_headers, j+1);
-			if (is_nonempty_string(key) && is_nonempty_string(val))
-				j += 2;
-			else {
-				g_ptr_array_remove_index (extra_headers, j);
-				g_ptr_array_remove_index (extra_headers, j);
-			}
-		}
-
-		/* write the extra headers */
-		fprintf (idx_fp, "%d\n", (int)extra_headers->len/2); /* extra header qty */
-		for (j=0; j<extra_headers->len; j+=2) {
-			const gchar * key = (const gchar*) g_ptr_array_index (extra_headers, j);
-			const gchar * val = (const gchar*) g_ptr_array_index (extra_headers, j+1);
-			const long key_index = get_string_offset (hash_misc, long_chunk, key, dat_fp, &pos);
-			const long val_index = get_string_offset (hash_misc, long_chunk, val, dat_fp, &pos);
-			fprintf (idx_fp, "%ld\n%ld\n", key_index, val_index);
-		}
-
-		pan_warn_if_fail (article_is_valid(a)); /* FIXME: just testing, remove later */
-		g_ptr_array_free (extra_headers, TRUE);
 	}
-	g_hash_table_destroy (hash_name); hash_name = NULL;
-	g_hash_table_destroy (hash_mail); hash_mail = NULL;
-	g_hash_table_destroy (hash_refs); hash_refs = NULL;
-	g_hash_table_destroy (hash_misc); hash_misc = NULL;
+	g_hash_table_destroy (hash);
+	hash = NULL;
 	memchunk_destroy (long_chunk);
 
+	/* did the files write out okay? */
+	ok = !ferror(idx_fp) && !ferror(dat_fp);
+
 	/* close the temp files */
 	fclose (idx_fp);
 	fclose (dat_fp);
 
-	ok = TRUE;
+	if (!ok)
+	{
+		log_add_va (LOG_ERROR, _("Unable to save headers for group \"%s\" - is the disk full?"), group->name);
+		unlink (idx_path);
+		unlink (dat_path);
+	}
 
 	if (ok)
 	{
 		pch = g_strdup_printf ("%s%c%s%c%s.idx",
-			data_dir, G_DIR_SEPARATOR,
+			get_data_dir(), G_DIR_SEPARATOR,
 			server_get_name (group->server), G_DIR_SEPARATOR,
 			group->name);
-		ok = pan_rename (idx_path, pch);
+		ok = pan_file_rename (idx_path, pch);
 		g_free (idx_path);
 		g_free (pch);
 	}
@@ -672,10 +494,10 @@ file_headers_save_group (Group * group, StatusItem * status)
 	if (ok)
 	{
 		pch = g_strdup_printf ("%s%c%s%c%s.dat",
-			data_dir, G_DIR_SEPARATOR,
+			get_data_dir(), G_DIR_SEPARATOR,
 			server_get_name (group->server), G_DIR_SEPARATOR,
 			group->name);
-		ok = pan_rename (dat_path, pch);
+		ok = pan_file_rename (dat_path, pch);
 		g_free (dat_path);
 		g_free (pch);
 	}
@@ -706,31 +528,26 @@ file_headers_save_noref (Group * group, StatusItem * status)
 		{
 			file_headers_destroy (group);
 		}
-		else /* save the group */
+		else if (!group_is_folder (group))/* save the group */
 		{
-			struct timeval start;
-			struct timeval finish;
+			GTimeVal start;
+			GTimeVal finish;
 			double diff;
-			gboolean ok = TRUE;
 
 			/* start the stopwatch */
-			gettimeofday (&start, NULL);
+			g_get_current_time (&start);
 
 			/* save the group */
-			ok = group_is_folder (group)
-				? file_headers_save_folder (group, status)
-				: file_headers_save_group (group, status);
-
-			if (!ok)
+			if (!file_headers_save_group (group, status))
 				group->articles_dirty = TRUE;
 			else {
 				/* end the stopwatch */
-				gettimeofday (&finish, NULL);
+				g_get_current_time (&finish);
 				diff = finish.tv_sec - start.tv_sec;
-				diff += (finish.tv_usec - start.tv_usec)/1000000.0;
+				diff += (finish.tv_usec - start.tv_usec)/(double)G_USEC_PER_SEC;
 				log_add_va (LOG_INFO, _("Saved %d articles in \"%s\" in %.1f seconds (%.0f art/sec)"),
 					articles->len,
-					group_get_readable_name (group),
+					group_get_name (group),
 					diff,
 					articles->len/(fabs(diff)<0.001?0.001:diff));
 			}
@@ -748,7 +565,7 @@ void
 file_headers_save (Group * group, StatusItem * status)
 {
 	debug_enter ("file_headers_save");
-	g_return_if_fail (group_is_valid(group));
+	g_return_if_fail (group!=NULL);
 
 	group_ref_articles (group, status);
 	file_headers_save_noref (group, status);
@@ -762,24 +579,23 @@ file_headers_destroy (const Group * group)
 {
 	debug_enter ("file_headers_destroy");
 
-	g_return_if_fail (group_is_valid(group));
-	g_return_if_fail (is_nonempty_string(data_dir));
+	g_return_if_fail (group_is_valid (group));
 
-	if (group_is_folder(group))
+	if (!group_is_folder(group))
 	{
-		gchar * mbox = get_mbox_filename (group);
-		remove (mbox);
-		g_free (mbox);
-	}
-	else
-	{
-		gchar * path;
+		char * path;
 
-		path = g_strdup_printf ("%s%c%s%c%s.idx", data_dir, G_DIR_SEPARATOR, server_get_name (group->server), G_DIR_SEPARATOR, group->name);
+		path = g_strdup_printf ("%s%c%s%c%s.idx",
+		                        get_data_dir(), G_DIR_SEPARATOR,
+		                        server_get_name (group->server), 
+					G_DIR_SEPARATOR, group->name);
 		remove (path);
 		g_free (path);
 
-		path = g_strdup_printf ("%s%c%s%c%s.dat", data_dir, G_DIR_SEPARATOR, server_get_name (group->server), G_DIR_SEPARATOR, group->name);
+		path = g_strdup_printf ("%s%c%s%c%s.dat",
+		                        get_data_dir(), G_DIR_SEPARATOR,
+		                        server_get_name (group->server), 
+					G_DIR_SEPARATOR, group->name);
 		remove (path);
 		g_free (path);
 	}
