@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -25,20 +23,17 @@
 
 #include <config.h>
 
-#include <limits.h>
-#include <string.h>
-#include <time.h>
-#include <unistd.h>
-#include <sys/time.h>
-
 #include <ctype.h>
+#include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <string.h>
+#include <limits.h>
 #include <time.h>
 
 #include <glib.h>
 
+#include <unistd.h>
+
 #include <pan/base/debug.h>
 #include <pan/base/gnksa.h>
 #include <pan/base/pan-i18n.h>
@@ -48,17 +43,59 @@
 **********************  BEGINNING OF SOURCE
 *********************/
 
-/*****
-******
-*****/
-
-
 #define is_code_char(A) (_code_chars[(guchar)(A)])
 #define is_tag_char(A) (_tag_chars[(guchar)(A)])
 #define is_unquoted_char(A) (_unquoted_chars[(guchar)(A)])
 #define is_quoted_char(A) (_quoted_chars[(guchar)(A)])
 #define is_paren_char(A) (_paren_chars[(guchar)(A)])
 
+#if 0
+#define PRINT_TABLE(A) \
+	printf ("static char " #A "[UCHAR_MAX] = {"); \
+	for (i=0; i<UCHAR_MAX; ++i) { \
+		if (!(i%40)) \
+			printf ("\n\t"); \
+		printf ("%d,", A[i]); \
+	} \
+	printf ("};\n\n");
+
+static char _unquoted_chars[UCHAR_MAX];
+static char _quoted_chars[UCHAR_MAX];
+static char _tag_chars[UCHAR_MAX];
+static char _code_chars[UCHAR_MAX];
+static char _paren_chars[UCHAR_MAX];
+
+void
+gnksa_init (void)
+{
+	int i;
+	unsigned char ch;
+
+	for (ch=0; ch<UCHAR_MAX; ++ch) {
+		_unquoted_chars[ch] = isgraph(ch) && ch!='!' && ch!='(' && ch!=')' && ch!='<'
+		                                  && ch!='>' && ch!='@' && ch!=',' && ch!=';'
+		                                  && ch!=':' && ch!='\\' && ch!='"' && ch!='.'
+		                                  && ch!='[' && ch!=']';
+		_quoted_chars[ch] = isgraph(ch) && ch!='"' && ch!='(' && ch!=')'
+		                                && ch!='<' && ch!='>' && ch!='\\';
+		_tag_chars[ch] = isgraph(ch) && ch!='!' && ch!='(' && ch!=')' && ch!='<'
+		                             && ch!='>' && ch!='@' && ch!=',' && ch!=';'
+		                             && ch!=':' && ch!='\\' && ch!='"' && ch!='['
+		                             && ch!=']' && ch!='/' && ch!='?' && ch!='=';
+		_paren_chars[ch] = isgraph(ch) && ch!='(' && ch!=')'
+		                               && ch!='<' && ch!='>'
+		                               && ch!='\\';
+		_code_chars[ch] = isgraph(ch) && ch!='?';
+	}
+
+	PRINT_TABLE(_unquoted_chars)
+	PRINT_TABLE(_quoted_chars)
+	PRINT_TABLE(_tag_chars)
+	PRINT_TABLE(_code_chars)
+	PRINT_TABLE(_paren_chars)
+}
+#else
+
 static char _unquoted_chars[UCHAR_MAX] = {
 	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
 	0,0,1,1,0,1,0,1,1,1,1,1,1,1,1,1,1,1,0,0,0,1,0,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
@@ -104,6 +141,8 @@ static char _paren_chars[UCHAR_MAX] = {
 	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
 	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,};
 
+#endif
+
 
 /**
 ***
@@ -200,9 +239,9 @@ static gboolean
 read_plain_word (const char * pch, const char ** end)
 {
 	/* unquoted-word / quoted-word / encoded-word */
-	if (read_unquoted_word(pch,end)) return TRUE;
 	if (read_quoted_word(pch,end)) return TRUE;
 	if (read_encoded_word(pch,end)) return TRUE;
+	if (read_unquoted_word(pch,end)) return TRUE;
 	return FALSE;
 }
 
@@ -349,7 +388,7 @@ gnksa_check_domain (const char * domain, guint domain_len)
 	/* last label -- toplevel domain */
 	top = get_next_token_run (pch, '.', &pch, &label, &label_len);
 	debug1 (DEBUG_GNKSA, "top level domain: [%s]", top);
-	switch (label_len)
+	switch ((int)strlen(top))
 	{
 		case 1:
 			if (isdigit((guchar)top[0]))
@@ -414,7 +453,7 @@ static gint
 gnksa_check_address (const gchar * address, gint address_len)
 {
 	gint retval = GNKSA_OK;
-	gchar * tmp;
+	gchar * work;
 	gchar * begin;
 
 	/* get rid of vacuous case */
@@ -422,8 +461,8 @@ gnksa_check_address (const gchar * address, gint address_len)
 		return GNKSA_LOCALPART_MISSING;
 
 	/* check the address */
-	pan_strndup_alloca (tmp, address, address_len);
-	begin = strrchr (address, '@');
+	pan_strndup_alloca (work, address, address_len);
+	begin = strrchr (work, '@');
 	if (begin==NULL)
 	{
 		if (retval == GNKSA_OK)
@@ -437,14 +476,11 @@ gnksa_check_address (const gchar * address, gint address_len)
 
 		/* check the domain */
 		if (retval == GNKSA_OK)
-			retval = gnksa_check_domain (begin, address_len - (begin-address));
+			retval = gnksa_check_domain (begin, address_len - (begin-work));
 
 		/* check the localpart */
 		if (retval == GNKSA_OK)
-			retval = gnksa_check_localpart (address);
-
-		/* restore separator */
-		*--begin = '@';
+			retval = gnksa_check_localpart (work);
 	}
 
 	return retval;
@@ -492,7 +528,7 @@ gnksa_split_from (const gchar  * from,
 			return GNKSA_LANGLE_MISSING;
 
 		/* copy route address from inside the <> brackets */
-		strncpy (addr_buf, begin+1, addr_max);
+		g_strlcpy (addr_buf, begin+1, addr_max);
 		*strchr(addr_buf, '>') = '\0';
 
 		/* From: [plain-phrase space] "<" address ">" */
@@ -505,8 +541,7 @@ gnksa_split_from (const gchar  * from,
 
 		/* get realname part */
 		g_strstrip (work);
-		strncpy (name_buf, work, name_max);
-		name_buf[name_max-1] = '\0';
+		g_strlcpy (name_buf, work, name_max);
 	}
 	else if ((lparen = strchr (work,'(')) != NULL) /* charles@rebelbase.com (Charles Kerr) */
 	{
@@ -518,8 +553,7 @@ gnksa_split_from (const gchar  * from,
 
 		/* address part */
 		len = MIN(addr_max, lparen-work);
-		strncpy (addr_buf, work, len);
-		addr_buf[len] = '\0';
+		g_strlcpy (addr_buf, work, len);
 		g_strstrip (addr_buf);
 
 		if (strict) {
@@ -540,19 +574,16 @@ gnksa_split_from (const gchar  * from,
 		if (strict && (!read_paren_phrase(lparen+1,&end) || end==NULL))
 			return GNKSA_ILLEGAL_PAREN_PHRASE;
 
-		strncpy (name_buf, real, name_max);
-		name_buf[name_max-1] = '\0';
+		g_strlcpy (name_buf, real, name_max);
 	}
 	else if (strchr(work,'@') != NULL) /* charles@rebelbase.com */
 	{
-		strncpy (addr_buf, work, addr_max);
-		addr_buf[addr_max-1] = '\0';
+		g_strlcpy (addr_buf, work, addr_max);
 		return strict ? gnksa_check_address(addr_buf, strlen(addr_buf)) : GNKSA_OK;
 	}
 	else /* who knows what this thing is... */
 	{
-		strncpy (name_buf, work, name_max);
-		name_buf[name_max-1] = '\0';
+		g_strlcpy (name_buf, work, name_max);
 		return GNKSA_LPAREN_MISSING;
 	}
 
@@ -572,48 +603,48 @@ gnksa_split_from (const gchar  * from,
 ******
 *****/
 
-GPtrArray*
-gnksa_split_addresses (const char * from)
-{
-	GPtrArray * a = g_ptr_array_new ();
-	gboolean quoted = FALSE;
-	const gchar * pch = from;
-	const gchar * start = from;
-
-	for (pch=start=from; is_nonempty_string(pch); )
-	{
-		while (*pch && !strchr("\",;:",*pch))
-			++pch;
-
-		if (*pch=='"') /* toggle quoted */
-			quoted = !quoted;
-		else if (!quoted) /* delimiter */
-		{
-			gchar * s = g_strndup (start, pch-start);
-			g_strstrip (s);
-			if (is_nonempty_string(s))
-				g_ptr_array_add (a, s);
-			else /* too small; throw it back */
-				g_free (s);
-			if (*pch)
-				start = pch + 1;
-		}
-
-		if (*pch)
-			++pch;
-	}
-
-	return a;
-}
-
-/*****
-******
-*****/
 
 
-/*****
-******
-*****/
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
 
 int
 gnksa_do_check_from (const char   * from,
@@ -794,7 +825,7 @@ remove_broken_message_ids_from_references (const char * references, GString * se
 
 		/* check the message-id for validity */
 		if (gnksa_check_message_id (begin, end-begin) == GNKSA_OK) {
-			pan_g_string_append_len (setme, begin, end-begin);
+			g_string_append_len (setme, begin, end-begin);
 			g_string_append_c (setme, ' ');
 		}
 
@@ -863,7 +894,7 @@ gnksa_trim_references_to_len (const gchar* refs, int cutoff)
 	return retval;
 }
 
-static const gchar* default_domain = "nospam.com";
+static const char* default_domain = "nospam.com";
 
 /**
  * thus spake son-of-1036: "the most popular method of generating local parts
@@ -871,10 +902,9 @@ static const gchar* default_domain = "nospam.com";
  * simultaneous postings on the same host (e.g., a process number), and encode
  * them in a suitably-reduced alphabet.
  */
-gchar*
-gnksa_generate_message_id (const gchar * domain_name)
+char*
+gnksa_generate_message_id (const char * domain_name)
 {
-	char * tmp;
 	GString * msgid = g_string_sized_new (256);
 
 	/* start with '<' */
@@ -882,19 +912,17 @@ gnksa_generate_message_id (const gchar * domain_name)
 
 	/* add unique local part to message-id */
 	{
-		gulong pid;
-		struct timeval now;
+		GTimeVal now;
 		struct tm local_now;
 		char buf[64];
 
-		pid = (gulong) getpid ();
-		gettimeofday (&now, NULL);
-		gmtime_r (&now.tv_sec, &local_now);
+		g_get_current_time (&now);
+		pan_gmtime_r (&now.tv_sec, &local_now);
 		strftime (buf, sizeof(buf), "%Y.%m.%d.%H.%M.%S", &local_now);
 
 		g_string_append (msgid, "pan.");
 		g_string_append (msgid, buf);
-		g_string_sprintfa (msgid, ".%ld.%lu", now.tv_usec, pid);
+		g_string_append_printf (msgid, ".%ld", now.tv_usec);
 	}
 
 	/* delimit */
@@ -909,9 +937,7 @@ gnksa_generate_message_id (const gchar * domain_name)
 	g_string_append_c (msgid, '>');
 
 	/* return */
-	tmp = msgid->str;
-	g_string_free (msgid, FALSE);
-	return tmp;
+	return g_string_free (msgid, FALSE);
 }
 
 gchar*
@@ -986,150 +1012,34 @@ gnksa_generate_references (const gchar * references,
 ****
 ***/
 
-/* converts struct tm to time_t, but does not take the local
-   timezone into account unless ``local'' is nonzero.
-   This is lifted from mutt 1.2.4, which is GPL and
-   copyrighted by Michael R. Elkins <me@cs.hmc.edu> */
-static time_t
-mutt_mktime (struct tm *t)
+time_t
+tzoffset_sec (time_t * now)
 {
-	time_t g;
+	struct tm gmt, lt;
+	int off;
 
-	static int AccumDaysPerMonth[12] = {
-		0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334
-	};
+	pan_gmtime_r (now, &gmt);
+	pan_localtime_r (now, &lt);
 
-	/* Compute the number of days since January 1 in the same year */
-	g = AccumDaysPerMonth [t->tm_mon % 12];
+	off = (lt.tm_hour - gmt.tm_hour) * 60 + lt.tm_min - gmt.tm_min;
 
-	/* The leap years are 1972 and every 4. year until 2096,
-	 * but this algoritm will fail after year 2099 */
-	g += t->tm_mday;
-	if ((t->tm_year % 4) || t->tm_mon < 2)
-		g--;
-	t->tm_yday = g;
+	if (lt.tm_year < gmt.tm_year)
+		off -= 24 * 60;
+	else if (lt.tm_year > gmt.tm_year)
+		off += 24 * 60;
+	else if (lt.tm_yday < gmt.tm_yday)
+		off -= 24 * 60;
+	else if (lt.tm_yday > gmt.tm_yday)
+		off += 24 * 60;
 
-	/* Compute the number of days since January 1, 1970 */
-	g += (t->tm_year - 70) * 365;
-	g += (t->tm_year - 69) / 4;
+	if (off >= 24 * 60)             /* should be impossible */
+		off = 23 * 60 + 59;     /* if not, insert silly value */
+	if (off <= -24 * 60)
+		off = -(23 * 60 + 59);
+	if (off > 13 * 60) 
+		off -= 24 * 60;
+	if (off < -12 * 60)
+		off += 24 * 60;
 
-	/* Compute the number of hours */
-	g *= 24;
-	g += t->tm_hour;
-
-	/* Compute the number of minutes */
-	g *= 60;
-	g += t->tm_min;
-
-	/* Compute the number of seconds */
-	g *= 60;
-	g += t->tm_sec;
-
-	return g;
+	return off * 60;
 }
-
-gchar*
-rfc822_date_generate (time_t secs)
-{
-	static const gchar *const months[] = {
-		"Jan","Feb","Mar","Apr","May","Jun",
-		"Jul","Aug","Sep","Oct","Nov","Dec"
-	};
-	static const gchar *const days[] = {
-		"Sun","Mon","Tue","Wed","Thu","Fri","Sat"
-	};
-
-	int diff;
-	int tz_diff_hours;
-	int tz_diff_minutes;
-	gboolean neg;
-	gchar fmt[64] = { '\0' };
-	gchar buf[64] = { '\0' };
-	struct tm ut;
-	struct tm local;
-
-	/* if 0 passed in, use current time */
-	if (secs == 0)
-		secs = time (NULL);
-
-	/* figure out the difference between this timezone and UT */
-	localtime_r (&secs, &local);
-	gmtime_r (&secs, &ut);
-	diff = (int) difftime (mutt_mktime(&local), mutt_mktime(&ut));
-	neg = diff<0;
-	diff = abs(diff);
-	tz_diff_hours = diff / 3600;
-	diff %= 3600;
-	tz_diff_minutes = diff / 60;
-
-	/* decide on the time string format.  see son of 1036 sec 5.1 for
-	 * the rationale of the offset-from-ut notation.
-         *
-	 * Update: INN 1.7.2-4 (and others) don't accept the <TZ> string at
-	 * the end of son-of-1036's date string.  Thanks to Valentini Jove
-	 * for testing to see what worked & what didn't.
-	 *
-	 * Update 2: Big Thanks to Tov Jacobsen for investigating what
-	 * format other programs use, for suggesting code, and for noting
-	 * that we need to roll our own day-of-week and month abbreviations
-	 * because son-of-1036 requires English text but strftime() localizes
-	 * (Okt instead of Oct or Man instead of Mon).
-	 *
-	 * Update 3: Gaelyne Gasson spotted a bug that the minute tz offset
-	 * should have been divided by 60 to give minutes instead of seconds.
-	 * This was causing problems in that tz, which is +0930.  I'm feeling
-	 * pretty dumb here... :)
-	 */
-	g_snprintf (fmt, sizeof(fmt), "%s, %%d %s %%Y %%H:%%M:%%S %c%02d%02d",
-		days[local.tm_wday],
-		months[local.tm_mon],
-		(neg?'-':'+'),
-		tz_diff_hours,
-		tz_diff_minutes);
-
-	/* generate the time */
-	strftime (buf, sizeof(buf), fmt, &local);
-	return g_strdup (buf);
-}
-
-/***
-****
-***/
-
-#ifndef HAVE_LOCALTIME_R
-struct tm*
-localtime_r(const time_t *const timep, struct tm *p_tm)
-{
-	struct tm * retval;
-	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
-
-	g_static_mutex_lock (&mutex);
-	retval = localtime (timep);
-	if (retval != NULL) {
-		memcpy (p_tm, retval, sizeof(struct tm));
-		retval = p_tm;
-	}
-	g_static_mutex_unlock (&mutex);
-
-	return retval;
-}
-#endif
-
-#ifndef HAVE_GMTIME_R
-struct tm*
-gmtime_r (const time_t * const timep, struct tm * p_tm)
-{
-	struct tm * retval;
-	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
-
-	g_static_mutex_lock (&mutex);
-	retval = gmtime (timep);
-	if (retval != NULL) {
-		memcpy (p_tm, retval, sizeof(struct tm));
-		retval = p_tm;
-	}
-	g_static_mutex_unlock (&mutex);
-
-	return retval;
-}
-#endif
