@@ -1,11 +1,11 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #include <config.h>
@@ -23,11 +22,15 @@
 #include <limits.h>
 #include <string.h>
 #include <stdlib.h>
+#include <stdio.h>
 
 #include <glib.h>
 
+#include <gmime/gmime-stream-file.h>
+
 #include <pan/base/acache.h>
 #include <pan/base/article-thread.h>
+#include <pan/base/base-prefs.h>
 #include <pan/base/debug.h>
 #include <pan/base/file-headers.h>
 #include <pan/base/group.h>
@@ -83,8 +86,8 @@ group_constructor (Group                * g,
 	g->filter_name = NULL;
 	g->identity_name = NULL;
 	g->description = NULL;
-	g->readable_name = NULL;
 	g->download_dir = NULL;
+	g->default_charset = NULL;
 	g->chunk = NULL;
 
 	g->_articles_refcount = (gint)0;
@@ -114,8 +117,8 @@ group_destructor (Group * o)
 	/* strings */
 	replace_gstr (&g->name, NULL);
 	replace_gstr (&g->description, NULL);
-	replace_gstr (&g->readable_name, NULL);
 	replace_gstr (&g->download_dir, NULL);
+	replace_gstr (&g->default_charset, NULL);
 	replace_gstr (&g->filter_name, NULL);
 	replace_gstr (&g->identity_name, NULL);
 
@@ -234,6 +237,36 @@ group_set_download_dir (Group * group, const gchar* download_dir)
 	replace_gstr (&group->download_dir, g_strdup(download_dir));
 }
 
+void
+group_set_default_charset (Group * group,
+                           const char * default_charset)
+{
+	g_return_if_fail (group_is_valid(group));
+
+	if (!is_nonempty_string (default_charset))
+		default_charset = NULL;
+
+	replace_gstr (&group->default_charset, g_strdup(default_charset));
+
+	group_set_dirty (group);
+	fire_groups_changed (&group, 1);
+}
+
+const char*
+group_get_default_charset (const Group * group)
+{
+	const char * retval = NULL;
+
+	g_return_val_if_fail (group_is_valid(group), retval);
+
+	retval = group->default_charset;
+
+	if (!is_nonempty_string (retval))
+		retval = get_charset_from_locale ();
+
+	return retval;
+}
+
 void
 group_set_flags (Group * group, guint flags)
 {
@@ -481,9 +514,83 @@ group_empty (Group * group, gboolean clear_counts)
 		group_set_article_range (group, 0, 0);
 }
 
-/**
-***  ARTICLE LIST
-**/
+/***
+****
+****   FOLDERS
+****
+***/
+
+void
+folder_add_message (Group            * folder,
+	            GMimeMessage     * message)
+{
+	gboolean is_new;
+	char * text;
+	const char * message_id;
+
+	g_return_if_fail (group_is_valid(folder));
+	g_return_if_fail (group_is_folder(folder));
+	g_return_if_fail (GMIME_IS_MESSAGE(message));
+
+	/* do we already have this message? */
+	message_id = g_mime_message_get_message_id (message);
+	is_new = acache_has_message (message_id);
+
+	/* write the message */
+	text = g_mime_message_to_string (message);
+	acache_set_message (folder->name, message_id, text, strlen(text));
+	g_free (text);
+
+	/* update articles */
+	if (is_new)
+	{
+		if (group_ref_articles_if_loaded (folder))
+		{
+			Article * a = article_new (folder);
+			article_set_from_g_mime_message (a, message);
+
+			if (article_is_valid (a))
+				group_add_article (folder, a);
+
+			group_unref_articles (folder, NULL);
+		}
+		else
+		{
+			group_set_article_qty (folder, folder->article_qty+1);
+		}
+	}
+}
+
+void
+folder_remove_message  (Group         * folder,
+                        GMimeMessage  * message)
+{
+	const char * message_id;
+
+	/* sanity clause */
+	g_return_if_fail (group_is_valid (folder));
+	g_return_if_fail (group_is_folder (folder));
+	g_return_if_fail (GMIME_IS_MESSAGE(message));
+
+	/* delete the message from our cache */
+	message_id = g_mime_message_get_message_id (message);
+	acache_expire_messages (&message_id, 1);
+
+	/* delete the message from the group's tables */
+	if (!group_ref_articles_if_loaded (folder))
+		group_set_article_qty (folder, folder->article_qty-1);
+	else {
+		Article * a = group_get_article_by_message_id (folder, message_id);
+		if (a != NULL)
+			group_remove_article (folder, a);
+		group_unref_articles (folder, NULL);
+	}
+}
+
+
+/***
+****
+***/
 
 static void
 group_add_articles_ghfunc (gpointer key, gpointer val, gpointer data)
@@ -594,10 +701,9 @@ group_add_articles_impl (Group             * group,
 	**/
 
 	new_read_qty = 0;
-	tmp_used = g_ptr_array_new ();
+	tmp_used = g_ptr_array_sized_new (articles->len);
 	newsrc = group_get_newsrc (group);
 	purged = group_get_purged (group);
-	pan_g_ptr_array_reserve (tmp_used, articles->len);
 
 	if (lock_group) {
 		g_mutex_lock (group->article_mutex);
@@ -741,7 +847,7 @@ group_set_articles_dirty (Group * group)
 }
 
 gpointer
-group_get_article_by_message_id (Group * group, const gchar * message_id)
+group_get_article_by_message_id (Group * group, const char * message_id)
 {
 	g_return_val_if_fail (group_is_valid(group), NULL);
 	g_return_val_if_fail (is_nonempty_string(message_id), NULL);
@@ -889,8 +995,7 @@ group_expire_articles_not_in_range (Group * g, gulong low, gulong high)
 
 	/* find old articles to remove */
 	tmp = group_get_article_array (g);
-	removeme = g_ptr_array_new ();
-	pan_g_ptr_array_reserve (removeme, tmp->len);
+	removeme = g_ptr_array_sized_new (tmp->len);
 	for (i=0; i!=tmp->len; ++i) {
 		Article * a = ARTICLE(g_ptr_array_index(tmp,i));
 		if ((a->number<low) || (a->number>high))
@@ -902,7 +1007,7 @@ group_expire_articles_not_in_range (Group * g, gulong low, gulong high)
 	{
 		log_add_va (LOG_INFO, _("Expired %u articles from `%s'"),
 			removeme->len,
-			group_get_readable_name(g));
+			group_get_name(g));
 
 		group_remove_articles (g, removeme);
 	}
@@ -920,6 +1025,28 @@ group_expire_articles_not_in_range (Group * g, gulong low, gulong high)
 
 static GStaticMutex article_refcount_mutex = G_STATIC_MUTEX_INIT;
 
+gboolean
+group_ref_articles_if_loaded (Group * group)
+{
+	gboolean retval = FALSE;
+	debug_enter ("group_ref_articles_if_loaded");
+
+	/* sanity clause */
+	g_return_val_if_fail (group_is_valid(group), FALSE);
+	g_return_val_if_fail (group->_articles_refcount>=0, FALSE);
+
+	/* ref if reffed */
+	g_static_mutex_lock (&article_refcount_mutex);
+	if (group->_articles_refcount >= 1) {
+		++group->_articles_refcount;
+		retval = TRUE;
+	}
+	g_static_mutex_unlock (&article_refcount_mutex);
+
+	debug_exit ("group_ref_articles_if_loaded");
+	return retval;
+}
+
 void
 group_ref_articles (Group * group, StatusItem * status)
 {
@@ -994,29 +1121,42 @@ group_unref_articles (Group * group, StatusItem * status)
 **/
 
 
-void
-group_set_readable_name (Group         * group,
-	                 const gchar   * readable_name)
-{
-	g_return_if_fail (group_is_valid(group));
-
-	replace_gstr (&group->readable_name, g_strdup(readable_name));
-
-	group_set_dirty (group);
-	fire_groups_changed (&group, 1);
-}
-
-
-const gchar*
-group_get_readable_name (const Group * group)
+const char*
+group_get_name (const Group * group)
 {
 	g_return_val_if_fail (group_is_valid(group), _("(No Group)"));
 
-	return is_nonempty_string (group->readable_name)
-		? group->readable_name
-		: group->name;
+	return group->name;
 }
 
+void
+group_get_collapsed_name (const Group   * group,
+                          char          * buf,
+                          size_t          bufsize)
+{
+	int pos = 0;
+	int len = 0;
+	const char * pch = NULL;
+	const char * name = group_get_name (group);
+	const char * march = name;
+	const char * last = "";
+
+	while (pos+2<bufsize && get_next_token_run (march, '.', &march, &pch, &len)) {
+		last = pch;
+		if (len > 0) {
+			buf[pos++] = *pch;
+			buf[pos++] = '.';
+		}
+	}
+	if (pos != 0)
+		pos -= 2;
+	buf[pos] = '\0';
+
+	if (is_nonempty_string(last))
+		g_strlcpy (buf+pos, last, bufsize-pos);
+}
+
+
 /**
 ***
 **/
@@ -1101,29 +1241,37 @@ compare_ppA_to_ppA_by_number (const void * va, const void * vb)
 {
 	const Article * a = *(const Article **)va;
 	const Article * b = *(const Article **)vb;
-	if (a->number < b->number) return -1;
-	if (a->number > b->number) return 1;
-	return 0;
+	int retval;
+
+	if (a->number < b->number)
+		retval = -1;
+	else if (a->number > b->number)
+		retval = 1;
+	else
+		retval = 0;
+
+	return retval;
 }
 
 void
 group_remove_crossposts (Group     * g,
 		         gulong    * numbers,
-                         gint        number_qty)
+                         int         number_qty)
 {
+	/* sanity clause */
 	g_return_if_fail (group_is_valid(g));
 	g_return_if_fail (numbers!=NULL);
 	g_return_if_fail (number_qty>0);
 
-	/* remove articles if they're already present */
-	if (g->_articles_refcount>0)
+	/* if that group's articles are loaded, remove the corresponding Articles */
+	if (group_ref_articles_if_loaded (g))
 	{
-		gint i;
+		int i;
 		GPtrArray * articles;
 		GPtrArray * remove;
 
 		/* get a list of articles sorted by number for fast lookup */
-		remove = g_ptr_array_new ();
+		remove = g_ptr_array_sized_new (number_qty);
 		articles = group_get_article_array (g);
 		qsort (articles->pdata, articles->len, sizeof(Article*), compare_ppA_to_ppA_by_number);
 
@@ -1153,6 +1301,7 @@ group_remove_crossposts (Group     * g,
 
 		g_ptr_array_free (remove, TRUE);
 		g_ptr_array_free (articles, TRUE);
+		group_unref_articles (g, NULL);
 	}
 
 	/* flag articles for deletion in case they get downloaded later */
@@ -1175,7 +1324,7 @@ group_alloc_new_article (Group * group)
 	g_return_val_if_fail (group_is_valid(group), NULL);
 
 	if (group->_article_chunk == NULL)
-		group->_article_chunk = memchunk_new (sizeof(Article), 200, FALSE);
+		group->_article_chunk = memchunk_new (sizeof(Article), 500, FALSE);
 
 	a = (Article*) memchunk_alloc (group->_article_chunk);
 	a->group = group;
