@@ -1,11 +1,11 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team (pan@rebelbase.com)
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #ifndef __GROUPS_H__
@@ -24,6 +23,7 @@
 #include <glib.h>
 
 #include <gmime/memchunk.h>
+#include <gmime/gmime-message.h>
 
 #include <pan/base/newsrc.h>
 #include <pan/base/server.h>
@@ -49,7 +49,7 @@ enum
 enum
 {
         STATE_FILTER_UNREAD                     = (1<<0),
-        STATE_FILTER_READ                       = (1<<1),
+        STATE_FILTER_DEPRECATED_DONT_USE        = (1<<1),
         STATE_FILTER_WATCHED                    = (1<<2),
         STATE_FILTER_IGNORED                    = (1<<3),
         STATE_FILTER_NORMAL_RANK                = (1<<4),
@@ -62,7 +62,7 @@ enum
 	STATE_FILTER_NEW			= (1<<11),
 	STATE_FILTER_CACHED			= (1<<12),
 	STATE_FILTER_NOT_CACHED			= (1<<13),
-	STATE_FILTER_OLD                        = (1<<14),
+	STATE_FILTER_READ                       = (1<<14),
 	STATE_FILTER_MINE                       = (1<<15),
 	STATE_FILTER_NOT_MINE                   = (1<<16)
 };
@@ -83,7 +83,7 @@ typedef struct _Group
 
         guint          loaded_since_last_fetch  : 1;
         guint          articles_dirty           : 1;     /* do the articles need saving? */
-	gchar          permission;                       /* (read only) 'y'=posting, 'n'=nopost, 'm'=moderated */
+	char           permission;                       /* (read only) 'y'=posting, 'n'=nopost, 'm'=moderated */
 	gint8          flags;                            /* (read only) new, subscribed, etc */
 	gint8          old_sort_style;                   /* (read only) hint for sorting in UI */
 	gint8          new_sort_style;                   /* (read only) hint for sorting in UI */
@@ -96,22 +96,22 @@ typedef struct _Group
 	gulong         article_high_old;                 /* (read only) previous high article */
 
 	Server       * server;
-	gchar        * name;                             /* alt.what.is.my.name */
-	gchar        * filter_name;                      /* article toolbar, articlelist */
-	gchar        * identity_name;                    /* last identity used for this group */
-	gchar        * description;                      /* maybe null */
-	gchar        * readable_name;                    /* maybe null, user alias for group */
-	gchar        * download_dir;                     /* where to save files, or NULL */
+	char         * name;                             /* alt.what.is.my.name */
+	char         * filter_name;                      /* article toolbar, articlelist */
+	char         * identity_name;                    /* last identity used for this group */
+	char         * description;                      /* maybe null */
+	char         * download_dir;                     /* where to save files, or NULL */
+	char         * default_charset;                  /* default charset for articles that don't specify, or NULL */
 	GStringChunk * chunk;                            /* string chunk used by articles */
 
 	/* PRIVATE */
-	gint           _articles_refcount;
+	int            _articles_refcount;
 	MemChunk     * _article_chunk;                   /* Article objects */
 	Newsrc       * _newsrc;                          /* tracks read/unread */
 	Newsrc       * _purged;                          /* tracks deleted articles */
 	GHashTable   * _articles;                        /* articles */
 	GPtrArray    * _deleted_articles;                /* deleted articles */
-	gchar        * _one_big_chunk;                   /* used by loading local headers */
+	char         * _one_big_chunk;                   /* used by loading local headers */
 }
 Group;
 
@@ -124,13 +124,13 @@ Group*    group_new                      (Server        * server,
                                           const char    * name);
 
 void      group_inc_article_read_qty     (Group         * group,
-                                          gint            article_read_qty);
+                                          int             article_read_qty);
 
 void      group_set_article_qty          (Group         * group,
-                                          gint            article_qty);
+                                          int             article_qty);
 
 void      group_set_article_read_qty     (Group         * group,
-                                          gint            article_read_qty);
+                                          int             article_read_qty);
 
 void      group_set_article_range        (Group         * group,
                                           gulong          article_low,
@@ -147,15 +147,15 @@ void      group_mark_all_read            (Group         * group,
                                           gboolean        read);
 
 void      group_set_sort_style           (Group         * group,
-                                          gint            sort_style);
+                                          int             sort_style);
 
 void      group_set_filter               (Group         * group,
                                           guint           filter_bits,
                                           gulong          filter_show,
-                                          const gchar   * filter_name);
+                                          const char    * filter_name);
 
 void      group_set_identity             (Group         * group,
-                                          const gchar   * identity_name);
+                                          const char    * identity_name);
 
 void      group_set_flags                (Group         * group,
                                           guint           flags);
@@ -176,14 +176,15 @@ gboolean  group_is_read_only             (const Group   * group);
 void      group_set_is_folder            (Group         * group,
                                           gboolean        is_folder);
 
-void      group_set_readable_name        (Group         * group,
-                                          const gchar   * readable_name);
+void      group_set_default_charset      (Group         * group,
+                                          const char    * default_charset);
 
-const gchar*    group_get_readable_name        (const Group   * group);
+const char*    group_get_name            (const Group   * group);
 
+const char*    group_get_default_charset (const Group   * group);
 
 void      groups_set_subscribed          (Group        ** group,
-                                          gint            qty,
+                                          int             qty,
                                           gboolean        subscribed);
 
 gboolean  group_is_subscribed            (const Group   * group);
@@ -198,6 +199,22 @@ void      group_set_dirty                (Group         * group);
 void      group_empty                    (Group         * group,
                                           gboolean        clear_counts);
 
+void      group_get_collapsed_name       (const Group   * group,
+                                          char          * buf,
+                                          size_t          bufsize);
+
+
+/**
+ * FOLDERS
+ *
+ */
+
+void          folder_add_message                  (Group            * folder,
+                                                   GMimeMessage     * message);
+
+void          folder_remove_message               (Group            * folder,
+                                                   GMimeMessage     * message);
+
 /**
  * ARTICLES
  *
@@ -235,7 +252,9 @@ void          group_remove_articles               (Group            * group,
 
 void          group_remove_crossposts             (Group            * g,
 		                                   gulong           * numbers,
-					           gint               number_qty);
+					           int                number_qty);
+
+gboolean      group_ref_articles_if_loaded        (Group            * group);
 
 void          group_ref_articles                  (Group            * group,
                                                    StatusItem       * item);
@@ -248,8 +267,8 @@ void          group_expire_articles_not_in_range  (Group            * group,
                                                    gulong             low,
                                                    gulong             high);
 
-const gchar*  group_chunk_string                  (Group            * group,
-                                                   const gchar      * chunkme,
+const char*   group_chunk_string                  (Group            * group,
+                                                   const char       * chunkme,
                                                    gboolean           share);
 
 void          group_set_articles_dirty            (Group            * group);
@@ -265,7 +284,7 @@ GHashTable*   group_get_articles                  (Group            * group);
 GPtrArray*    group_get_article_array             (Group            * group);
 
 gpointer      group_get_article_by_message_id     (Group            * group,
-                                                   const gchar      * message_id);
+                                                   const char       * message_id);
 
 struct _Article;
 
@@ -277,13 +296,13 @@ struct _Article*  group_alloc_new_article        (Group         * group);
 **/
 
 void group_constructor    (Group                        * group,
-			   const gchar                  * name);
+			   const char                   * name);
 
 void group_destructor     (Group                        * group);
 
 
-extern const gchar * PAN_SENDLATER;
-extern const gchar * PAN_SENT;
+extern const char * PAN_SENDLATER;
+extern const char * PAN_SENT;
 
 
 /**
