@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 /*********************
@@ -165,8 +162,8 @@ init_read (Newsrc * n)
 static int
 compare_pulong_to_pRange (const void * a, const void * b)
 {
-	gulong number = *(gulong *)a;
-	const Range * range = (const Range*) b;
+	const register gulong number = *(gulong *)a;
+	const register Range * range = (const Range*) b;
 	int retval;
 
 	if (number < range->low)
@@ -197,7 +194,7 @@ range_contains_point (const Range * r1, gulong point)
 ***/
 
 static gboolean
-maybe_merge_ranges (GArray * a, gint low)
+maybe_merge_ranges (GArray * a, int low)
 {
 	gboolean merged = FALSE;
 
@@ -223,9 +220,9 @@ static gulong
 mark_range_read (Newsrc * n, Range * rr)
 {
 	GArray * a = n->read;
-	gint i;
-	gint low_index = 0;
-	gint high_index = 0;
+	int i;
+	int low_index = 0;
+	int high_index = 0;
 	gulong retval = 0;
 	gboolean range_found = FALSE;
 
@@ -243,51 +240,54 @@ mark_range_read (Newsrc * n, Range * rr)
 	                                      compare_pulong_to_pRange,
 	                                      NULL);
 
-	for (i=low_index; i<=high_index && i<a->len; )
+	retval = rr->high + 1 - rr->low;
+
+	for (i=low_index; i<=high_index && i<a->len; ++i)
 	{
 		Range * r = &g_array_index (a, Range, i);
 
 		if (range_contains (rr, r)) /* read range is engulfed; remove */
 		{
-			retval += r->high+1 - r->low;
+			retval -= r->high+1 - r->low; /* remove r from retval */
 			g_array_remove_index (a, i);
 			--high_index;
+			--i;
 		}
 		else if (range_contains (r, rr)) /* no-op */
 		{
+			retval = 0;
 			range_found = TRUE;
-			++i;
 		}
 		else if (range_contains_point (r, rr->high)) /* change low */
 		{
+			Range * prev = !i ? NULL : &g_array_index (a, Range, i-1);
 			range_found = TRUE;
-			retval += r->low - rr->low;
-			r->low = rr->low;
-			if (maybe_merge_ranges (a, i-1))
-				--high_index;
-			else
-				++i;
+			retval -= rr->high+1 - r->low; /* remove intersection from retval */
+			r->low = prev ? MAX(rr->low, prev->high+1) : rr->low;
 		}
 		else if (range_contains_point (r, rr->low)) /* change high */
 		{
+			Range * next = i==a->len-1 ? NULL : &g_array_index (a, Range, i+1);
 			range_found = TRUE;
-			retval += rr->high - r->high;
-			r->high = rr->high;
-			if (maybe_merge_ranges (a, i))
-				--high_index;
-			else
-				++i;
+			retval -= r->high+1 - rr->low; /* remove intersection from retval */
+			r->high = next ? MIN(rr->high, next->low-1) : rr->high;
 		}
-		else ++i;
 	}
 
 	if (!range_found)
 	{
 		Range r = *rr;
-		retval = (r.high+1-r.low) - retval;
 		g_array_insert_val (a, low_index, r);
-		maybe_merge_ranges (a, low_index);
-		maybe_merge_ranges (a, low_index-1);
+		--low_index;
+		++high_index;
+	}
+
+	for (i=low_index; i<=high_index && i<a->len; )
+	{
+		if (maybe_merge_ranges (a, i))
+			--high_index;
+		else
+			++i;
 	}
 
 	return retval;
@@ -296,9 +296,9 @@ mark_range_read (Newsrc * n, Range * rr)
 static gulong
 mark_range_unread (Newsrc * n, const Range * ur)
 {
-	gint i;
-	gint low_index = -1;
-	gint high_index = -1;
+	int i;
+	int low_index = -1;
+	int high_index = -1;
 	gboolean contains_low = FALSE;
 	gboolean contains_high = FALSE;
 	gulong retval = 0;
@@ -431,6 +431,10 @@ newsrc_destructor (PanObject * obj)
 void
 newsrc_init (Newsrc * n, const char * read_str, gulong low, gulong high)
 {
+	/* sanity clause */
+	g_return_if_fail (n!=NULL);
+
+	/* update low/high/str */
 	n->group_low = low;
 	n->group_high = high;
 	replace_gstr (&n->read_str, g_strdup (read_str?read_str:""));
@@ -459,10 +463,11 @@ void
 newsrc_set_group_range (Newsrc * n, gulong low, gulong high)
 {
 	gboolean exact_match;
-	gint low_index;
-	gint high_index;
+	int low_index;
+	int high_index;
 
-	g_return_if_fail (n != NULL);
+	/* sanity clause */
+	g_return_if_fail (n!=NULL);
 
 	n->group_low = low;
 	n->group_high = high;
@@ -516,6 +521,9 @@ newsrc_set_group_range (Newsrc * n, gulong low, gulong high)
 void
 newsrc_mark_all (Newsrc * n, gboolean read)
 {
+	/* sanity clause */
+	g_return_if_fail (n!=NULL);
+
 	if (n->read == NULL)
 		init_read (n);
 
@@ -537,6 +545,7 @@ newsrc_is_article_read (const Newsrc * n, gulong article_number)
 {
 	gboolean exact_match = FALSE;
 
+	/* sanity clause */
 	g_return_val_if_fail (n!=NULL, FALSE);
 	g_return_val_if_fail (article_number!=0, FALSE);
 
@@ -566,6 +575,7 @@ newsrc_import_line (Newsrc      * n,
 	gulong low, high;
 	GArray * a;
 
+	/* sanity clause */
 	g_return_if_fail (n != NULL);
 	g_return_if_fail (setme_subscribed != NULL);
 	g_return_if_fail (import != NULL);
@@ -601,16 +611,16 @@ newsrc_import_line (Newsrc      * n,
 ******
 *****/
 
-gchar*
+char*
 newsrc_get_read_str (const Newsrc * n)
 {
-	gchar * retval = NULL;
+	char * retval = NULL;
 
 	if (n!=NULL && n->read!=NULL) /* someone's been changing the read/unread */
 	{
-		gint i;
-		const gint max_chars_per_range = 22; /* "%lu-%lu," w/10 digits for %lu */
-		gchar * pch = retval = g_new0 (gchar, n->read->len*max_chars_per_range + 1);
+		int i;
+		const int max_chars_per_range = 22; /* "%lu-%lu," w/10 digits for %lu */
+		GString * str = g_string_sized_new (max_chars_per_range * n->read->len);
 
 		for (i=0; i<n->read->len; ++i)
 		{
@@ -620,14 +630,16 @@ newsrc_get_read_str (const Newsrc * n)
 			if (r->high < n->group_low) continue;
 			if (r->low > n->group_high) continue;
 
-			pch += sprintf (pch, "%lu", r->low);
+			g_string_append_printf (str, "%lu", r->low);
 
 			if (r->low!=r->high)
-				pch += sprintf (pch, "-%lu", r->high);
+				g_string_append_printf (str, "-%lu", r->high);
 
 			if (i != n->read->len-1)
-				*pch++ = ',';
+				g_string_append_c (str, ',');
 		}
+
+		retval = g_string_free (str, FALSE);
 	}
 	else if (n!=NULL && is_nonempty_string(n->read_str)) /* just use the fields passed in */
 	{
@@ -647,7 +659,7 @@ newsrc_export_line (const Newsrc  * n,
                     gboolean        subscribed,
 	            GString       * appendme)
 {
-	gchar * buf;
+	char * buf;
 
 	g_return_if_fail (is_nonempty_string(group_name));
 
