@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -62,7 +60,7 @@ pan_callback_free (PanCallback* pcl)
 void
 pan_callback_call (PanCallback* pcl, gpointer call_object, gpointer call_arg)
 {
-	gint i;
+	int i;
 	Impl impl = *(Impl*) pcl->impl;
 
 	for (i=0; i<impl.qty; ++i)
@@ -79,7 +77,7 @@ pan_callback_add (PanCallback* pcl, PanCallbackFunc cb, gpointer user)
 	if (impl->qty >= MAX_LISTENERS)
 		g_warning ("Too many listeners to pan_callback %p", pcl);
 	else {
-		const gint qty = impl->qty++;
+		const int qty = impl->qty++;
 		impl->listeners[qty].callback = cb;
 		impl->listeners[qty].user_data = user;
 	}
@@ -88,7 +86,7 @@ pan_callback_add (PanCallback* pcl, PanCallbackFunc cb, gpointer user)
 void
 pan_callback_remove (PanCallback* pcl, PanCallbackFunc cb, gpointer user)
 {
-	gint i;
+	int i;
 	Impl * impl = (Impl*) pcl->impl;
 
 	for (i=0; i<impl->qty; ++i)
