@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2001  Pan Development Team <pan@rebelbase.com>
+/*
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #include <config.h>
@@ -26,9 +23,18 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <locale.h>
 
 #include <glib.h>
 
+#include <gmime/gmime-charset.h>
+#include <gmime/gmime-utils.h>
+
+#include <iconv.h>
+#include <gmime/gmime-iconv.h>
+#include <gmime/gmime-iconv-utils.h>
+
+#include <pan/base/log.h>
 #include <pan/base/pan-glib-extensions.h>
 
 
@@ -50,7 +56,7 @@ pan_strstr (const char * s1,
 }
 
 /* null-safe strcmp.  NULLs < strings. */
-gint 
+int 
 pan_strcmp (const char * a,
             const char * b)
 {
@@ -68,51 +74,6 @@ pan_strcmp (const char * a,
 	return retval;
 }
 
-char*
-pan_stristr (const char * string, const char * pattern)
-{
-	size_t slen;
-	size_t plen;
-	char * start = (char*) string;
-
-	g_return_val_if_fail (string!=NULL, NULL);
-	g_return_val_if_fail (pattern!=NULL, NULL);
-
- 	slen = strlen (string);
-	plen = strlen (pattern);
-
-	for (; slen>=plen; ++start, --slen)
-	{
-		char * sptr;
-		char * pptr;
-
-		/* find start of pattern in string */
-		while (toupper(*start) != toupper(*pattern))
-		{
-			++start;
-			--slen;
-
-			/* if pattern longer than string */
-			if (slen < plen)
-				return NULL;
-		}
-
-		sptr = start;
-		pptr = (char *) pattern;
-
-		while (toupper(*sptr) == toupper(*pptr))
-		{
-			++sptr;
-			++pptr;
-
-			if (!*pptr)
-				return start;
-		}
-	}
-
-	return NULL;
-}
-
 gboolean
 string_ends_with (const char * str,
                   const char * end)
@@ -134,16 +95,16 @@ string_ends_with (const char * str,
 
 
 void
-replace_gstr (char   ** target_gfree_old,
-              char    * assign_from_me)
+replace_gstr (char   ** oldaddr,
+              char    * newval)
 {
-	char * gfree_old;
+	char * oldval;
 
- 	g_return_if_fail (target_gfree_old != NULL);
+ 	g_return_if_fail (oldaddr != NULL);
 
-	gfree_old = *target_gfree_old;
-	*target_gfree_old = assign_from_me;
-	g_free (gfree_old);
+	oldval = *oldaddr;
+	*oldaddr = newval;
+	g_free (oldval);
 }
 
 /**
@@ -360,28 +321,6 @@ pan_g_string_strstrip (GString * string)
 	}
 }
 
-void
-pan_g_string_append_len  (GString      * str,
-                          const char   * appendme,
-                          guint          len)
-{
-	g_return_if_fail (str!=NULL);
-	g_return_if_fail (appendme!=NULL);
-
-	if (len < 1024)
-	{
-		char buf[1024];
-		memcpy (buf, appendme, len);
-		buf[len] = '\0';
-		g_string_append (str, buf);
-	}
-	else
-	{
-		char * pch = g_strndup (appendme, len);
-		g_string_append (str, pch);
-		g_free (pch);
-	}
-}
 
 /***
 ****
@@ -395,14 +334,26 @@ skip_next_token (const char   * pch,
                  const char     delimiter,
                  const char  ** setme_next_token)
 {
+	register char ch;
+
 	if (is_nonempty_string(pch))
 	{
-		while (*pch && *pch!=delimiter)
-			++pch;
-		if (*pch==delimiter)
-			++pch; /* skip past delimiter */
-		if (setme_next_token)
-			*setme_next_token = pch;
+		for (;;) {
+			ch = *pch;
+			if (ch == delimiter) {
+				++pch; /* skip past delimiter */
+				if (setme_next_token)
+					*setme_next_token = pch;
+				return;
+			}
+			if (ch)
+			       ++pch;
+			else {
+				if (setme_next_token)
+					*setme_next_token = pch;
+				return;
+			}
+		}
 	}
 }
  
@@ -561,7 +512,7 @@ get_next_token_g_str (const char   * pch,
 	if (end == pch)
 		retval = end==NULL ? FALSE : *end==delimiter;
 	else {
-		pan_g_string_append_len (setme, pch, end-pch);
+		g_string_append_len (setme, pch, end-pch);
 		retval = TRUE;
 	}
 
@@ -571,14 +522,12 @@ get_next_token_g_str (const char   * pch,
 	return retval;
 }
 
-
 /***
 ****
 ****  UNSORTED
 ****
 ***/
 
-
 int
 lower_bound (const void         * key,
              const void         * base,
@@ -611,32 +560,6 @@ lower_bound (const void         * key,
 	return low;
 }
 
-
-void
-commatize_ulong (gulong    num,
-		 char    * setme)
-{
-	char buf[32], *src=buf;
-	int len = g_snprintf (buf, sizeof(buf), "%ld", num);
-	int i = len % 3;
-
-	for (;;)
-	{
-		while (*src && i--)
-			*setme++ = *src++;
-		if (!*src) {
-			*setme = '\0';
-			return;
-		}
-		else if (src!=buf)
-		   *setme++ = ',';
-		i = 3;
-	}
-
-	pan_warn_if_reached ();
-}
-
-
 /***
 ****
 ****  XML
@@ -721,120 +644,227 @@ pan_str_unescape (const char * escaped)
 }
 
 /***
-****  This is taken from Sylpheed 0.7.0.
+****
 ***/
 
-static char*
-strstr_with_skip_quote (const char *haystack, const char *needle)
+struct tm*
+pan_localtime_r(const time_t *const timep, struct tm *p_tm)
 {
-	register guint haystack_len, needle_len;
-	gboolean in_squote = FALSE, in_dquote = FALSE;
+	struct tm * retval;
+#ifdef HAVE_LOCALTIME_R
+	retval = localtime_r (timep, p_tm);
+#else
+	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
 
-	haystack_len = strlen(haystack);
-	needle_len   = strlen(needle);
+	g_static_mutex_lock (&mutex);
+	retval = localtime (timep);
+	if (retval != NULL) {
+		memcpy (p_tm, retval, sizeof(struct tm));
+		retval = p_tm;
+	}
+	g_static_mutex_unlock (&mutex);
+#endif
+	return retval;
+}
 
-	if (haystack_len < needle_len || needle_len == 0)
+struct tm*
+pan_gmtime_r (const time_t * const timep, struct tm * p_tm)
+{
+	struct tm * retval;
+#ifdef HAVE_GMTIME_R
+	retval = gmtime_r (timep, p_tm);
+#else
+	static GStaticMutex mutex = G_STATIC_MUTEX_INIT;
+
+	g_static_mutex_lock (&mutex);
+	retval = gmtime (timep);
+	if (retval != NULL) {
+		memcpy (p_tm, retval, sizeof(struct tm));
+		retval = p_tm;
+	}
+	g_static_mutex_unlock (&mutex);
+#endif
+	return retval;
+}
+
+/* The language isn't used yet, but will be useful for get the default
+ * language when we integrate a spellchecker.
+ */
+
+struct LocaleStruct {
+    const char *locale, *charset;
+} locales[] = {
+    {"en_US",        "ISO-8859-1"}, 
+    {"pt_BR",        "ISO-8859-1"},
+    {"ca_ES",        "ISO-8859-15"},
+    {"zh_CN.GB2312", "gb2312"},
+    {"zh_TW.Big5",   "big5"},
+    {"cs_CZ",        "ISO-8859-2"},
+    {"da_DK",        "ISO-8859-1"},
+    {"de_DE",        "ISO-8859-15"},
+    {"nl_NL",        "ISO-8859-15"},
+    {"et_EE",        "ISO-8859-15"},
+    {"fi_FI",        "ISO-8859-15"},
+    {"fr_FR",        "ISO-8859-15"},
+    {"el_GR",        "ISO-8859-7"},
+    {"hu_HU",        "ISO-8859-2"},
+    {"it_IT",        "ISO-8859-15"},
+    {"ja_JP",        "ISO-2022-jp"},
+    {"ko_KR",        "euc-kr"},
+    {"lv_LV",        "ISO-8859-13"},
+    {"lt_LT",        "ISO-8859-13"},
+    {"no_NO",        "ISO-8859-1"},
+    {"pl_PL",        "ISO-8859-2"},
+    {"pt_PT",        "ISO-8859-15"},
+    {"ro_RO",        "ISO-8859-2"},
+    {"ru_RU",        "KOI8-R"},
+    {"ru_SU",        "ISO-8859-5"},
+    {"sk_SK",        "ISO-8859-2"},
+    {"es_ES",        "ISO-8859-15"},
+    {"sv_SE",        "ISO-8859-1"},
+    {"tr_TR",        "ISO-8859-9"},
+    {"uk_UK",        "KOI8-U"}
+};
+
+/* find_locale_index_by_locale:
+ * finds the longest fit so the one who has en_GB will get en_US if en_GB
+ * is not defined.
+ * This function is lifted from Balsa.
+ */
+static gint
+get_closest_locale (void)
+{
+	const char * locale = setlocale (LC_CTYPE, NULL);
+	guint i, j, maxfit = 0, maxpos = 0;
+
+	g_return_val_if_fail (locale != NULL, -1);
+
+	if (!locale || strcmp(locale, "C") == 0)
+		return 0;
+
+	for (i = 0; i < G_N_ELEMENTS(locales); i++) {
+		for (j=0; locale[j] && locales[i].locale[j] == locale[j]; j++);
+		if (j > maxfit) {
+			maxfit = j;
+			maxpos = i;
+		}
+	}
+	return maxpos;
+}
+
+#define PAN_DEFAULT_CHARSET  "ISO-8859-1"
+
+const char *
+get_charset_from_locale (void)
+{
+	gint loc_idx;
+
+	loc_idx = get_closest_locale ();
+
+	return loc_idx != -1 ? locales[loc_idx].charset : PAN_DEFAULT_CHARSET;
+}
+
+const char*
+pan_utf8ize (const char    * str,
+             gssize          len,
+             char         ** g_freeme)
+{
+	const char * retval;
+
+	g_return_val_if_fail (g_freeme!=NULL, NULL);
+
+	if (!is_nonempty_string (str))
+		retval = *g_freeme = g_strdup ("");
+	else if  (!g_utf8_validate (str, len, NULL))
+		retval = *g_freeme = g_locale_to_utf8 (str, len, NULL, NULL, NULL);
+	else {
+		*g_freeme = NULL;
+		retval = str;
+	}
+
+	return retval;
+}
+
+char*
+pan_header_to_utf8 (const char   * header,
+                    int            header_len,
+                    const char   * charset)
+{
+	char * retval = NULL;
+
+	if (header == NULL)
 		return NULL;
 
-	while (haystack_len >= needle_len) {
-		if (!in_squote && !in_dquote &&
-		    !strncmp(haystack, needle, needle_len))
-			return (char *)haystack;
+	if (header_len < 0)
+		header_len = strlen (header);
 
-		/* 'foo"bar"' -> foo"bar"
-		   "foo'bar'" -> foo'bar' */
-		if (*haystack == '\'') {
-			if (in_squote)
-				in_squote = FALSE;
-			else if (!in_dquote)
-				in_squote = TRUE;
-		} else if (*haystack == '\"') {
-			if (in_dquote)
-				in_dquote = FALSE;
-			else if (!in_squote)
-				in_dquote = TRUE;
+	/* nice girls encode their 8bit strings */
+	if (g_strstr_len (header, header_len, "=?") != NULL) {
+		guchar * pch;
+	       	pch = (guchar*) g_strndup (header, header_len);
+		retval = g_mime_utils_8bit_header_decode (pch);
+		g_free (pch);
+	}
+
+	/* naughty boys don't properly encode their 8bit string */
+	else if (g_mime_utils_text_is_8bit (header, header_len)) {
+		if (is_nonempty_string(charset)) {
+			iconv_t cd = g_mime_iconv_open ("UTF-8", charset);
+			retval = g_mime_iconv_strndup (cd, header, header_len);
+			g_mime_iconv_close (cd);
 		}
-
-		haystack++;
-		haystack_len--;
-	}
-
-	return NULL;
-}
-
-/* this fuction was taken from gstrfuncs.c in glib. */
-static char **
-strsplit_with_quote (const char * str,
-                     const char * delim,
-                     int max_tokens)
-{
-	GSList *string_list = NULL, *slist;
-	char **str_array, *s;
-	guint i, n = 1;
-
-	g_return_val_if_fail(str != NULL, NULL);
-	g_return_val_if_fail(delim != NULL, NULL);
-
-	if (max_tokens < 1)
-		max_tokens = G_MAXINT;
-
-	s = strstr_with_skip_quote(str, delim);
-	if (s) {
-		guint delimiter_len = strlen(delim);
-
-		do {
-			guint len;
-			char *new_str;
-
-			len = s - str;
-			new_str = g_new(char, len + 1);
-			strncpy(new_str, str, len);
-			new_str[len] = 0;
-			string_list = g_slist_prepend(string_list, new_str);
-			n++;
-			str = s + delimiter_len;
-			s = strstr_with_skip_quote(str, delim);
-		} while (--max_tokens && s);
-	}
-
-	if (*str) {
-		n++;
-		string_list = g_slist_prepend(string_list, g_strdup(str));
-	}
-
-	str_array = g_new(char*, n);
-
-	i = n - 1;
-
-	str_array[i--] = NULL;
-	for (slist = string_list; slist; slist = slist->next)
-		str_array[i--] = slist->data;
-
-	g_slist_free(string_list);
-
-	return str_array;
-}
-
-char**
-strsplit_command_line_arguments (const char * command_line)
-{
-	int i;
-	char ** argv;
-
-	/* split the command line into arguments */
-	argv = strsplit_with_quote (command_line, " ", 0);
-
-	/* remove the quotes from quoted arguments */
-	for (i=0; argv[i]!=NULL; i++) {
-		char * str = argv[i];
-		if (str[0] == '\'' || str[0] == '\"') {
-			int len = strlen(str);
-			if (str[len - 1] == str[0]) {
-				str[len - 1] = '\0';
-				memmove(str, str + 1, len - 1);
-			}
+		if (retval == NULL) {
+			retval = g_mime_iconv_locale_to_utf8_length (header, header_len);
 		}
 	}
 
-	return argv;
+	/* fallback */
+	if (retval == NULL)
+		retval = g_strndup (header, header_len);
+
+	return retval;
+}
+
+char*
+pan_body_to_utf8 (GMimePart     * part,
+                  const char    * default_charset)
+{
+	char * retval = NULL;
+	const char * content;
+	size_t content_len = 0ul;
+
+	/* sanity clause */
+	g_return_val_if_fail (GMIME_IS_PART(part), NULL);
+
+	/* check for an empty part */
+	content = g_mime_part_get_content (part, &content_len);
+	if (!is_nonempty_string(content))
+		return NULL;
+
+	/* use the part's content-type charset, if it's got one */
+	if (retval == NULL) {
+		const char * charset = g_mime_object_get_content_type_parameter (GMIME_OBJECT (part), "charset");
+		if (is_nonempty_string (charset)) {
+			iconv_t cd = g_mime_iconv_open ("UTF-8", charset);
+			retval = g_mime_iconv_strndup (cd, content, content_len);
+			g_mime_iconv_close (cd);
+		}
+	}
+
+	/* try the group's default charset */
+	if (retval==NULL && is_nonempty_string (default_charset)) {
+		iconv_t cd = g_mime_iconv_open ("UTF-8", default_charset);
+		retval = g_mime_iconv_strndup (cd, content, content_len);
+		g_mime_iconv_close (cd);
+	}
+
+	/* otherwise just use the text and hope for the best */
+	if (retval == NULL) {
+		const char * end = NULL;
+		if (!g_utf8_validate (content, content_len, &end))
+			log_add_va (LOG_ERROR, "Some parts of the article are corrupt and can't be displayed.");
+		retval = g_strndup (content, end-content);
+	}
+
+	return retval;
 }
