@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #include <config.h>
@@ -38,13 +35,13 @@
 
 typedef struct
 {
-	gchar * leader;
-	gchar * content;
+	char * leader;
+	char * content;
 }
 Paragraph;
 
 static Paragraph*
-paragraph_new (const gchar * leader, const gchar * content)
+paragraph_new (const char * leader, const char * content)
 {
 	Paragraph * p = g_new0 (Paragraph, 1);
 	p->leader = g_strdup (leader);
@@ -61,7 +58,7 @@ paragraph_free (Paragraph * p)
 }
 
 static GPtrArray*
-get_paragraphs (const gchar * body, gint wrap_column)
+get_paragraphs (const char * body, int wrap_column)
 {
 	GPtrArray * lines = g_ptr_array_new ();
 	GPtrArray * paragraphs = g_ptr_array_new ();
@@ -69,14 +66,14 @@ get_paragraphs (const gchar * body, gint wrap_column)
 	/* build an array of lines that are split between leader & content */
 	if (1)
 	{
-		gint line_len = 0;
-		const gchar * line_start = NULL;
+		int line_len = 0;
+		const char * line_start = NULL;
 
 		while (get_next_token_run (body, '\n', &body, &line_start, &line_len))
 		{
 			Paragraph * p;
-			const gchar * pch = line_start;
-			const gchar * end = line_start + line_len;
+			const char * pch = line_start;
+			const char * end = line_start + line_len;
 
 			while (pch!=end && (is_quote_char((guchar)*pch) || isspace((guchar)*pch)))
 				++pch;
@@ -94,8 +91,8 @@ get_paragraphs (const gchar * body, gint wrap_column)
 	/* merge the lines into paragraphs */
 	if (1)
 	{
-		gint i;
-		gint prev_content_len = 0;
+		int i;
+		int prev_content_len = 0;
 		GString * cur_leader = g_string_new (NULL);
 		GString * cur_content = g_string_new (NULL);
 
@@ -153,13 +150,13 @@ get_paragraphs (const gchar * body, gint wrap_column)
 	return paragraphs;
 }
 
-static gchar*
-wrap_long_lines (gchar   * string,
-                 gint      maxlen)
+static char*
+wrap_long_lines (char    * string,
+                 int       maxlen)
 {
-	gchar * linefeed_here;
-	gchar * pch;
-	gchar * line_start;
+	char * linefeed_here;
+	char * pch;
+	char * line_start;
 
 
 	/* walk through the entire string */
@@ -186,13 +183,13 @@ wrap_long_lines (gchar   * string,
 static void
 fill_paragraph (const Paragraph   * p,
                 GPtrArray         * setme,
-                gint                wrap_column)
+                int                 wrap_column)
 {
-	gchar * pch = g_strdup (p->content);
-	const gchar * march;
-	const gint wrap_len = wrap_column - strlen(p->leader);
-	const gchar * line_begin;
-	int line_len;
+	char * pch = g_strdup (p->content);
+	const char * march;
+	const int wrap_len = wrap_column - strlen(p->leader);
+	const char * line_begin = NULL;
+	int line_len = 0;
 
 	wrap_long_lines (pch, wrap_len);
 	march = pch;
@@ -205,15 +202,15 @@ fill_paragraph (const Paragraph   * p,
 	g_free (pch);
 }
 
-gchar*
-fill_body (const gchar * body, gint wrap_column)
+char*
+pan_text_fill (const char * body, int wrap_column)
 {
 	GPtrArray * array = g_ptr_array_new ();
 	GPtrArray * paragraphs = NULL;
-	gchar * new_body = NULL;
-	gchar * tmp_body = NULL;
-	gchar * sig = NULL;
-	gint i = 0;
+	char * new_body = NULL;
+	char * tmp_body = NULL;
+	char * sig = NULL;
+	int i = 0;
 
 	/* sanity checks */
 	g_return_val_if_fail (is_nonempty_string(body), NULL);
@@ -233,7 +230,7 @@ fill_body (const gchar * body, gint wrap_column)
 	/* make a single string of all filled lines */
 	g_ptr_array_add (array, NULL);
 	if (1) {
-		gint i;
+		int i;
 		GString * s = g_string_new (NULL);
 		for (i=0; g_ptr_array_index(array,i)!=NULL; ++i) {
 			g_string_append (s, (const char*)g_ptr_array_index(array,i));
@@ -241,8 +238,7 @@ fill_body (const gchar * body, gint wrap_column)
 		}
 		if (s->len > 0)
 			g_string_truncate (s, s->len-1);
-		new_body = s->str;
-		g_string_free (s, FALSE);
+		new_body = g_string_free (s, FALSE);
 	}
 
 	/* if we had a sig, put it back in */
@@ -265,14 +261,13 @@ fill_body (const gchar * body, gint wrap_column)
 ****
 ***/
 
-gchar*
-mute_quoted_text_in_body (const gchar * text)
+char*
+pan_text_mute_quoted (const char * text)
 {
-	gint line_len = 0;
-	const gchar * line_begin = NULL;
-	const gchar * march;
+	int line_len = 0;
+	const char * line_begin = NULL;
+	const char * march;
 	gboolean last_line_was_quote;
-	gchar * retval;
 	GString * buf;
 
 	g_return_val_if_fail (text!=NULL, NULL);
@@ -286,7 +281,7 @@ mute_quoted_text_in_body (const gchar * text)
 
 		if (!is_quote)
 		{
-			pan_g_string_append_len (buf, line_begin, line_len);
+			g_string_append_len (buf, line_begin, line_len);
 			g_string_append_c (buf, '\n');
 		}
 		else if (!last_line_was_quote)
@@ -300,25 +295,24 @@ mute_quoted_text_in_body (const gchar * text)
 	if (buf->len > 0)
 		g_string_truncate (buf, buf->len-1); /* trim last \n */
 
-	retval = buf->str;
-	g_string_free (buf, FALSE);
-	return retval;
+	return g_string_free (buf, FALSE);
 }
 
 
-gchar*
-rot13_inplace (gchar * text)
+#warning is this utf8-friendly...
+char*
+pan_text_rot13_inplace (char * text)
 {
 	const char * plain = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
 	const char * roted = "nopqrstuvwxyzabcdefghijklmNOPQRSTUVWXYZABCDEFGHIJKLM";
 
 	if (text != NULL)
 	{
-		gchar * march;
+		char * march;
 
 		for (march=text; *march!='\0'; ++march)
 		{
-			const gchar * offset = strchr (plain, *march);
+			const char * offset = strchr (plain, *march);
 			if (offset != NULL)
 			{
 				const int index = offset - plain;
