@@ -1,14 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Pan - A Newsreader for X
- * Copyright (C) 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -62,43 +59,43 @@ filter_phrase_to_string (const Filter * filter)
 	if (state==FILTER_PHRASE_SUBJECT) {
 		if (is_regex) {
 			if (neg)
-				pch = g_strdup_printf (_("Subject doesn't match the regex `%s'"), phrase);
+				pch = g_strdup_printf (_("Subject doesn't match the regular expression \"%s\""), phrase);
 			else
-				pch = g_strdup_printf (_("Subject matches the regex `%s'"), phrase);
+				pch = g_strdup_printf (_("Subject matches the regular expression \"%s\""), phrase);
 		}
 		else {
 			if (neg)
-				pch = g_strdup_printf (_("Subject doesn't match the phrase `%s'"), phrase);
+				pch = g_strdup_printf (_("Subject doesn't match the phrase \"%s\""), phrase);
 			else
-				pch = g_strdup_printf (_("Subject matches the phrase `%s'"), phrase);
+				pch = g_strdup_printf (_("Subject matches the phrase \"%s\""), phrase);
 		}
 	}
 	else if (state==FILTER_PHRASE_AUTHOR) {
 		if (is_regex) {
 			if (neg)
-				pch = g_strdup_printf (_("Author doesn't match regular expression `%s'"), phrase);
+				pch = g_strdup_printf (_("Author doesn't match regular expression \"%s\""), phrase);
 			else
-				pch = g_strdup_printf (_("Author matches regular expression `%s'"), phrase);
+				pch = g_strdup_printf (_("Author matches regular expression \"%s\""), phrase);
 		}
 		else {
 			if (neg)
-				pch = g_strdup_printf (_("Author doesn't match phrase `%s'"), phrase);
+				pch = g_strdup_printf (_("Author doesn't match phrase \"%s\""), phrase);
 			else
-				pch = g_strdup_printf (_("Author matches phrase `%s'"), phrase);
+				pch = g_strdup_printf (_("Author matches phrase \"%s\""), phrase);
 		}
 	}
 	else if (state==FILTER_PHRASE_MESSAGE_ID) {
 		if (is_regex) {
 			if (neg)
-				pch = g_strdup_printf (_("Message-ID doesn't match regular expression `%s'"), phrase);
+				pch = g_strdup_printf (_("Message-ID doesn't match regular expression \"%s\""), phrase);
 			else
-				pch = g_strdup_printf (_("Message-ID matches regular expression `%s'"), phrase);
+				pch = g_strdup_printf (_("Message-ID matches regular expression \"%s\""), phrase);
 		}
 		else {
 			if (neg)
-				pch = g_strdup_printf (_("Message-ID doesn't match phrase `%s'"), phrase);
+				pch = g_strdup_printf (_("Message-ID doesn't match phrase \"%s\""), phrase);
 			else
-				pch = g_strdup_printf (_("Message-ID matches phrase `%s'"), phrase);
+				pch = g_strdup_printf (_("Message-ID matches phrase \"%s\""), phrase);
 		}
 	}
 	else pch = g_strdup_printf (_("Error"));
@@ -117,8 +114,8 @@ filter_phrase_test_articles (Filter          * filter,
 
 	for (i=0; i<article_qty; ++i)
 	{
+		gchar author[512];
 		const gchar * checkme = NULL;
-		gchar * freeme = NULL;
 
 		switch (f->state) {
 			case FILTER_PHRASE_SUBJECT:
@@ -128,15 +125,13 @@ filter_phrase_test_articles (Filter          * filter,
 				checkme = article_get_message_id (articles[i]);
 				break;
 			case FILTER_PHRASE_AUTHOR:
-				checkme = freeme = article_get_author_str (articles[i]);
+				checkme = article_get_author_str (articles[i], author, sizeof(author));
 				break;
 		}
 
 		passfail[i] = f->is_regex
 			? !regexec (&f->phrase_regex,  checkme, 0, NULL, 0)
 			: !fnmatch (f->phrase, checkme, PAN_CASEFOLD);
-
-		g_free (freeme);
 	}
 }
 
@@ -205,7 +200,7 @@ filter_phrase_set (FilterPhrase        * filter,
 		if (filter->is_regex)
 		{
 			filter->phrase = g_strdup (phrase);
-			regcomp (&filter->phrase_regex, filter->phrase, REG_ICASE);
+			regcomp (&filter->phrase_regex, filter->phrase, REG_EXTENDED|REG_ICASE);
 		}
 		else
 		{
