@@ -1,11 +1,11 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,14 +15,14 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #include <config.h>
 
 #include <ctype.h>
 
-#include <gnome.h>
+#include <glib.h>
+#include <gtk/gtk.h>
 
 #include <pan/base/debug.h>
 #include <pan/base/pan-config.h>
@@ -85,31 +85,7 @@ gui_paned_construct (void)
 
 	pan_lock ();
 
-	/* clear out the workarea */
-	gtk_widget_unrealize (GTK_WIDGET(contents_vbox));
-	gtk_widget_hide_all (GTK_WIDGET(contents_vbox));
-	if (groups_vbox->parent != NULL)
-		gtk_container_remove (GTK_CONTAINER(groups_vbox->parent), groups_vbox);
-	if (articlelist_ctree->parent != NULL)
-		gtk_container_remove (GTK_CONTAINER(articlelist_ctree->parent), articlelist_ctree);
-	if (text_box->parent != NULL)
-		gtk_container_remove (GTK_CONTAINER(text_box->parent), text_box);
-	gtk_widget_unrealize (groups_vbox);
-	gtk_widget_unrealize (articlelist_ctree);
-	gtk_widget_unrealize (text_box);
-
-	/* make a new UI */
-	if (1) {
-		GList * children = gtk_container_children (GTK_CONTAINER(contents_vbox));
-		GList * l;
-		for (l=children; l!=NULL; l=l->next)
-			gtk_container_remove (GTK_CONTAINER(contents_vbox), GTK_WIDGET(l->data));
-		g_list_free (l);
-	}
-
-	gtk_widget_show (groups_vbox);
-	gtk_widget_show (articlelist_ctree);
-	gtk_widget_show (text_box);
+	gui_layout_clear_workarea ();
 
 	/**
 	***  The layout_str is a four-character string where the first
@@ -127,7 +103,7 @@ gui_paned_construct (void)
 		else if (tolower(*pch)=='t')
 			w[i] = show_header_pane ? articlelist_ctree : NULL;
 		else
-			w[i] = show_article_pane ? text_box : NULL;
+			w[i] = show_body_pane ? text_box : NULL;
 	}
 
 	/* layout part 2: build the array */
@@ -167,12 +143,12 @@ gui_paned_construct (void)
 	}
 
 	if (work != NULL)
-		gtk_box_pack_start (GTK_BOX(contents_vbox), work, TRUE, TRUE, 0);
+		gtk_box_pack_start (GTK_BOX(Pan.workarea), work, TRUE, TRUE, 0);
 
 	Pan.viewmode = GUI_PANED;
-	gui_restore_column_widths (Pan.group_clist, "group");
+	gui_restore_column_widths (Pan.group_tree, "group");
 	gui_restore_column_widths (Pan.article_ctree, "thread_pane");
-	gtk_widget_show_all (GTK_WIDGET(contents_vbox));
+	gtk_widget_show_all (GTK_WIDGET(Pan.workarea));
 
 	pan_unlock ();
 	debug_exit ("gui_paned_construct");
@@ -187,12 +163,12 @@ gui_paned_construct (void)
 int
 gui_paned_get_current_pane_nolock (void)
 {
-	gint retval = GROUPS_PAGE; /* default to group pane */
+	int retval = GROUPS_PANE; /* default to group pane */
 
 	if (GTK_WIDGET_HAS_FOCUS (Pan.article_ctree))
-		retval = ARTICLELIST_PAGE;
+		retval = HEADERS_PANE;
 	else if (GTK_WIDGET_HAS_FOCUS (Pan.text))
-		retval = MESSAGE_PAGE;
+		retval = BODY_PANE;
 	
 	return retval;
 }
