@@ -1,11 +1,11 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,26 +15,21 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #ifndef __GUI_H__
 #define __GUI_H__
 
 #include <gtk/gtkwidget.h>
-#include <libgnomeui/gnome-app.h>
-#include <libgnomeui/gnome-app-helper.h>
 
 #include <pan/base/article.h>
 #include <pan/base/group.h>
 #include <pan/base/status-item.h>
+#include <pan/filters/filter.h>
 
 extern GtkWidget * groups_vbox;
 extern GtkWidget * articlelist_ctree;
 extern GtkWidget * text_box;
-extern GtkWidget * contents_vbox;
-
-extern GnomeUIInfo help_menu[];
 
 enum {
 	GUI_PANED = 1,
@@ -42,24 +37,29 @@ enum {
 };
 
 enum {
-	GROUPS_PAGE = 0,
-	ARTICLELIST_PAGE,
-	MESSAGE_PAGE
+	GROUPS_PANE = 0,
+	HEADERS_PANE,
+	BODY_PANE
 };
 
 void pan_widget_set_font (GtkWidget* w, const char* font_name);
 
-void gui_set_layout(int);
+void gui_layout_clear_workarea (void);
+void gui_layout_set_mode (int type);
+void gui_layout_refresh (void);
+
 
 void gui_construct (const gchar *geometry);
 
 void gui_page_set (int page);
 
 void gui_restore_column_widths    (GtkWidget * clist, const gchar * key);
+void gui_restore_column_widths_tree_view    (GtkWidget * tree_view, const gchar * key);
 
 void gui_save_column_widths       (GtkWidget * clist, const gchar * key);
+void gui_save_column_widths_tree_view       (GtkWidget * tree_view, const gchar * key);
 
-void gui_restore_window_size      (GtkWidget * window, const gchar * key);
+gboolean gui_restore_window_size      (GtkWidget * window, const gchar * key);
 
 void gui_save_window_size         (GtkWidget * window, const gchar * key);
 
@@ -67,12 +67,12 @@ void gui_set_connecting_flag (gboolean is_connecting);
 void gui_inc_connection_size (int inc);
 void gui_set_queue_size (guint running, guint total);
 
+GtkWidget* gui_add_network_button_to_toolbar  (GtkWidget * toolbar);
+
 void gui_queue_add (GSourceFunc func, gpointer user_data);
 
+GdkColormap* cmap;
 
-extern GnomeUIInfo group_menu [];
-extern GnomeUIInfo folder_menu [];
-
-extern GdkColormap * cmap;
+void gui_edit_current_filter (void);
 
 #endif /* __GUI_H__ */
