@@ -1,11 +1,11 @@
-    /*
- * Pan - A Newsreader for X
- * Copyright (C) 2001  Pan Development Team <pan@rebelbase.com>
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/*
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,17 +15,15 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
  */
 
 #include <glib.h>
 
-#include <gnome.h>
-
 #include <pan/prefs.h>
 
 #include <pan/base/base-prefs.h>
 #include <pan/base/debug.h>
+#include <pan/base/pan-config.h>
 #include <pan/base/pan-glib-extensions.h>
 
 #include <pan/base/pan-callback.h>
@@ -50,7 +48,7 @@ static gchar     * _default_mail_id = NULL;
 static gchar *
 get_identity_filename (void)
 {
-	return g_strdup_printf ("%s%cprofiles.xml", data_dir, G_DIR_SEPARATOR);
+	return g_strdup_printf ("%s%cprofiles.xml", get_data_dir(), G_DIR_SEPARATOR);
 }
 
 /*
@@ -59,7 +57,7 @@ get_identity_filename (void)
  * set in ~/.gnome/Pan and add it to the _identities database.
  *
  * Since all settings will be removed from prefs.c, we'll directly
- * access the preferences through gnome_config_get_string()
+ * access the preferences through pan_config_get_string()
  */
 
 static Identity * 
@@ -71,8 +69,8 @@ construct_identity_from_prefs (void)
 
 	debug_enter ("construct_identity_from_prefs");
 
-	real = gnome_config_get_string("/Pan/User/Full_Name");
-	addr = gnome_config_get_string("/Pan/User/Email");
+	real = pan_config_get_string("/Pan/User/Full_Name");
+	addr = pan_config_get_string("/Pan/User/Email");
 
 	if (is_nonempty_string(real) && is_nonempty_string(addr))
 	{
@@ -87,22 +85,22 @@ construct_identity_from_prefs (void)
 		identity_set_author_real (id, real);
 		identity_set_author_addr (id, addr);
 	
-		pch = gnome_config_get_string("/Pan/User/Reply_To");
+		pch = pan_config_get_string("/Pan/User/Reply_To");
 		if (is_nonempty_string (pch))
 			identity_set_reply_to (id, pch);
 		g_free (pch);
 
-		pch = gnome_config_get_string("/Pan/User/Organization");
+		pch = pan_config_get_string("/Pan/User/Organization");
 		if (is_nonempty_string (pch))
 			identity_set_organization (id, pch);
 		g_free (pch);
 
-		pch = gnome_config_get_string("/Pan/User/Signature_File");
+		pch = pan_config_get_string("/Pan/User/Signature_File");
 		if (is_nonempty_string (pch))
 			identity_set_signature (id, pch);
 		g_free (pch);
 
-		pch = gnome_config_get_string ("/Pan/User/attribution_line");
+		pch = pan_config_get_string ("/Pan/User/attribution_line");
 		if (is_nonempty_string (pch))
 			identity_set_attribution (id, pch);
 		g_free (pch);
@@ -114,6 +112,22 @@ construct_identity_from_prefs (void)
 	return id;
 }
 
+static void
+identity_manager_save (void)
+{
+	if (_identities)
+	{
+		gchar * fname= get_identity_filename ();
+
+		/* TODO: support for dirty */
+
+		identity_xml_write (fname, (const Identity **) _identities->pdata,
+			_identities->len, _default_news_id, _default_mail_id);
+
+		g_free (fname);
+	}
+}
+
 static void
 identity_manager_module_init (void)
 {
@@ -128,20 +142,16 @@ identity_manager_module_init (void)
 
 	if (_identities->len == 0)
 	{
-                gchar    * fname = get_identity_filename ();
 		Identity * id = construct_identity_from_prefs();
 
 		if (id)
 		{
 			g_ptr_array_add (_identities, id);
+			replace_gstr (&_default_news_id, g_strdup(id->name));
+			replace_gstr (&_default_mail_id, g_strdup(id->name));
 
-			replace_gstr(&_default_news_id, g_strdup(id->name));
-			replace_gstr(&_default_mail_id, g_strdup(id->name));
-
-        	        identity_xml_write (fname, (Identity **) _identities->pdata,
-                	        _identities->len, _default_news_id, _default_mail_id);
+			identity_manager_save ();
 		}
-                g_free (fname);
 	}
 
 	debug_exit ("identity_manager_module_init");
@@ -266,17 +276,10 @@ identity_manager_set_default_no_save (const gchar * id_name, int type)
 void
 identity_manager_set_default (const gchar * id_name, int type)
 {
-	gchar     * fname;
-
 	debug_enter ("identity_manager_set_default");
 
 	identity_manager_set_default_no_save (id_name, type);
-
-	/* save them to disk */
-	fname = get_identity_filename ();
-	identity_xml_write (fname, (Identity  **)_identities->pdata, 
-		_identities->len, _default_news_id, _default_mail_id);
-	g_free (fname);
+	identity_manager_save ();
 
 	debug_exit ("identity_manager_set_default");
 }
@@ -304,7 +307,6 @@ identity_manager_set_identities (GPtrArray   * ids,
                                  const gchar * default_mail)
 {
 	guint i;
-	gchar * fname;
 	GPtrArray * tmp;
 	GPtrArray * old;
 
@@ -328,11 +330,7 @@ identity_manager_set_identities (GPtrArray   * ids,
 	identity_manager_set_default_no_save(default_news, ID_NEWS_DEFAULT);
 	identity_manager_set_default_no_save(default_mail, ID_MAIL_DEFAULT);
 
-	/* save them to disk */
-	fname = get_identity_filename ();
-	identity_xml_write (fname, (Identity  **)_identities->pdata, 
-		_identities->len, _default_news_id, _default_mail_id);
-	g_free (fname);
+	identity_manager_save ();
 
 	/* fire a notice to listeners */
 	pan_callback_call (identity_manager_get_identities_changed_callback(),
@@ -356,6 +354,7 @@ identity_manager_add_identity (const Identity * id)
 		identity_manager_module_init ();
 
 	g_ptr_array_add (_identities, identity_dup (id));
+	identity_manager_save ();
 
 	debug_exit ("identity_manager_add_identity");
 	
@@ -366,17 +365,7 @@ identity_manager_shutdown_module (void)
 {
 	debug_enter ("identity_manager_shutdown_module");
 
-	if (_identities)
-	{
-		gchar * fname= get_identity_filename ();
-
-		/* TODO: support for dirty */
-
-		identity_xml_write (fname, (Identity **) _identities->pdata,
-			_identities->len, _default_news_id, _default_mail_id);
-
-		g_free (fname);
-	}
+	identity_manager_save ();
 
 	pan_g_ptr_array_foreach (_identities, (GFunc)pan_object_unref, NULL);
         g_ptr_array_free (_identities, TRUE);
