@@ -1,11 +1,11 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #include <config.h>
@@ -41,8 +40,6 @@
 #include <pan/task.h>
 #include <pan/auth-spa.h>
 
-gboolean online_prompt = TRUE;
-
 /** deprecated, use nntp_command */
 extern const char * sockwrite_err_msg;
 extern const char * sockread_err_msg;
@@ -104,7 +101,6 @@ nntp_command (StatusItem           * status,
 {
 	int val;
 	size_t len;
-	debug_enter ("nntp_command");
 
 	g_return_val_if_fail (sock!=NULL, -1);
 	g_return_val_if_fail (is_nonempty_string(command), -1);
@@ -465,7 +461,7 @@ nntp_can_post (StatusItem   * status,
 ***
 **/
 
-gint
+int
 nntp_set_group (StatusItem   * status,
                 PanSocket    * sock,
                 const char   * group_name)
@@ -486,7 +482,7 @@ nntp_set_group (StatusItem   * status,
 	/* change to that group */
 	val = nntp_command_va (status, sock, &response, "GROUP %s", group_name);
 	if (val != OK_GROUP) {
-		status_item_emit_error_va (status, _("Unable to set group `%s': %s"), group_name, response);
+		status_item_emit_error_va (status, _("Unable to set group \"%s\": %s"), group_name, response);
 		return val==-1 ? TASK_FAIL : TASK_FAIL_HOPELESS;
 	}
 
@@ -499,7 +495,7 @@ nntp_set_group (StatusItem   * status,
 ***
 **/
 
-gint
+int
 nntp_post (StatusItem    * status,
            PanSocket     * sock,
            const gchar   * msg)
@@ -525,16 +521,16 @@ nntp_post (StatusItem    * status,
 		/* tell the server we want to post */
 		val = nntp_command (status, sock, &response, "POST");
 		if (val != 340) {
-			status_item_emit_error_va (status, _("Posting failed: %s"), response);
+			status_item_emit_error_va (status, _("Posting failed.  Server said: %s"), response);
 			return val==-1 ? TASK_FAIL : TASK_FAIL_HOPELESS;
 		}
 
 		/* post the article */
 		val = nntp_command_va (status, sock, &response, "%s\r\n.\r\n", msg);
 		if (val == 240) {
-			log_add_va (LOG_INFO, _("Posting complete: %s"), response);
+			log_add_va (LOG_INFO, _("Posting complete.  Server said: %s"), response);
 		} else {
-			status_item_emit_error_va (status, "Posting failed: %s", response);
+			status_item_emit_error_va (status, "Posting failed.  Server said: %s", response);
 			return val==-1 ? TASK_FAIL : TASK_FAIL_HOPELESS;
 		} 
 	}
@@ -558,7 +554,7 @@ nntp_noop (StatusItem    * status,
 	g_return_val_if_fail (socket!=NULL, TASK_FAIL);
 
 	/* do work */
-	status_item_emit_status_va (status, _("\"stay connected\" sent to %s"), socket->host); 
+	status_item_emit_status_va (status, _("\"stay connected\" sent to %s"), socket->host_name); 
 	return nntp_can_post (status, socket, &can_post);
 }
 
@@ -566,82 +562,50 @@ nntp_noop (StatusItem    * status,
 ***
 **/
 
-static gboolean
-init_header_if_match (Article       * a,
-                      const gchar   * token,
-                      const gchar   * token_key,
-                      guint           token_key_len,
-                      const gchar   * header,
-                      HeaderAction    action)
+int
+nntp_article_download (StatusItem         * status,
+                       PanSocket          * sock,
+                       MessageIdentifier  * mid,
+                       const gboolean     * abort,
+                       int                  verbose)
 {
-	gboolean match = FALSE;
-
-	/* entry assertions */
-	g_return_val_if_fail (a!=NULL, FALSE);
-	g_return_val_if_fail (is_nonempty_string(token_key), FALSE);
-	g_return_val_if_fail (token_key_len>0, FALSE);
-	g_return_val_if_fail (is_nonempty_string(header), FALSE);
-	g_return_val_if_fail (token!=NULL, FALSE);
-
-	/* if this is the token we're looking for, keep it. */
-	if (!strncmp (token, token_key, token_key_len))
-	{
-		const gchar * val = token + token_key_len;
-		if (!pan_header_is_8bit_encoded (val))
-			article_init_header (a, header, val, action);
-		else {
-			gchar * b = g_mime_utils_8bit_header_decode ((const guchar*)val);
-			article_init_header (a, header, b, action);
-			g_free (b);
-		}
-		match = TRUE;
-	}
-
-	return match;
-}
-
-gint
-nntp_article_download_to_gstr (StatusItem      * status,
-                               PanSocket       * sock,
-                               Group           * group,
-                               Article         * article,
-                               const gboolean  * abort,
-                               gint              verbose,
-                               gboolean          checkout,
-                               GString         * setme)
-{
-	const gchar * response;
+	const char * response;
 	gboolean getting_headers;
-	gint retval;
-	gint val;
+	MessageSource * source;
+	int retval;
+	int result;
+	int val;
 	GString * line;
+	GString * setme;
 	debug_enter ("nntp_article_download");
 
 	/* sanity clause */
 	g_return_val_if_fail (status!=NULL, TASK_FAIL);
 	g_return_val_if_fail (sock!=NULL, TASK_FAIL);
-	g_return_val_if_fail (group_is_valid(group), TASK_FAIL);
-	g_return_val_if_fail (article_is_valid(article), TASK_FAIL);
+	g_return_val_if_fail (message_identifier_is_valid(mid), TASK_FAIL);
 	g_return_val_if_fail (abort!=NULL, TASK_FAIL);
 
 	/* let the user know what we're doing */
 	if (verbose & NNTP_VERBOSE_TEXT)
-		status_item_emit_status (status, article_get_subject(article));
+		status_item_emit_status (status, mid->message_id);
+
+	/* change to the group we want */
+	source = message_identifier_get_source_for_server (mid, sock->host_name);
+	if (*abort || !source)
+		return TASK_FAIL;
+	result = nntp_set_group (status, sock, source->group_name);
+	if (result != TASK_SUCCESS)
+		return result;
 
 	/* request the article from the server */
-	val = nntp_command_va (status, sock, &response, "ARTICLE %lu", article->number);
-	if (val != 220)
-	{
-		gchar * pch = g_strdup_printf (_("Getting article `%s' (Message-ID: %s) body failed: %s"),
-		                               article_get_subject(article),
-		                               article_get_message_id(article),
-		                               response);
+	val = nntp_command_va (status, sock, &response, "ARTICLE %lu", source->number);
+	if (val != 220) {
+		char * pch = g_strdup_printf (_("Getting article \"%s\" body failed: %s"), mid->message_id, response);
 		status_item_emit_error (status, pch);
 		g_free (pch);
-
 		if (val==-1)
 			return TASK_FAIL;
-		articles_add_flag (&article, 1, STATE_ERROR);
+		/*FIXME articles_add_flag (&article, 1, STATE_ERROR);*/
 		return TASK_FAIL_HOPELESS;
 	}
 
@@ -649,10 +613,10 @@ nntp_article_download_to_gstr (StatusItem      * status,
 	getting_headers = TRUE;
 	line = g_string_sized_new (80);
 
-	g_string_truncate (setme, 0);
+	setme = g_string_sized_new (80 * 1000);
 	retval = TASK_SUCCESS;
 	if (verbose & NNTP_VERBOSE_INIT_STEPS)
-		status_item_emit_init_steps (status, article->linecount);
+		status_item_emit_init_steps (status, mid->line_qty);
 
 	for (;;)
 	{
@@ -686,76 +650,24 @@ nntp_article_download_to_gstr (StatusItem      * status,
 		if (verbose & NNTP_VERBOSE_NEXT_STEP)
 			status_item_emit_next_step (status);
 
+
 		/* save the headers that we want to save */
 		if (getting_headers)
 		{
 			if (line->len==1 && !strcmp(line->str, "\n")) /* header/body separator */
 				getting_headers = FALSE;
-			else 
-			{
-				gint i;
-				Article * a = article;
-				gboolean matched = FALSE;
-
-				struct table {
-					const gchar * token;
-					guint token_len;
-					const gchar * header;
-					gint header_action;
-				};
-				static struct table tokens[] = {
-					{ "Newsgroups: ", 12, HEADER_NEWSGROUPS, DO_CHUNK_SHARE },
-					{ "Reply-To: ", 10, HEADER_REPLY_TO, DO_CHUNK_SHARE },
-					{ "Followup-To: ", 13, HEADER_FOLLOWUP_TO, DO_CHUNK_SHARE },
-					{ "Xref: ", 6, HEADER_XREF, DO_CHUNK },
-				};
-				static gint TOKEN_QTY = sizeof(tokens) / sizeof(tokens[0]);
-
-				/* trim linefeed */
-				g_strstrip (line->str);
-				line->len = strlen(line->str);
-
-				/* check the header */
-				for (i=0; !matched && i<TOKEN_QTY; ++i)
-					matched = init_header_if_match (a,
-					                                line->str,
-					                                tokens[i].token,
-					                                tokens[i].token_len,
-					                                tokens[i].header,
-					                                tokens[i].header_action);
-			}
 		}
 	}
 
+	if (verbose & NNTP_VERBOSE_NEXT_STEP)
+		status_item_emit_next_step (status);
+
 	/* save the message */
-	if (retval == TASK_SUCCESS) {
-        	acache_set_message (article_get_message_id(article), setme->str, setme->len+1, checkout);
-		articles_remove_flag (&article, 1, STATE_DOWNLOAD_FLAGGED);
-	}
+	if (retval == TASK_SUCCESS)
+        	acache_set_message (ACACHE_DEFAULT_KEY, mid->message_id, setme->str, setme->len);
 
 	g_string_free (line, TRUE);
-	debug_exit ("nntp_article_download");
-	return retval;
-}
-
-gint
-nntp_article_download (StatusItem      * status,
-                       PanSocket       * sock,
-                       Group           * group,
-                       Article         * article,
-                       const gboolean  * abort,
-                       gint              verbose,
-                       gboolean          checkout)
-{
-	GString * tmp;
-	gint retval;
-	debug_enter ("nntp_article_download");
-
-	/* download the message */
- 	tmp = g_string_sized_new (80 * 400); /* preallocate space for ~400 lines */
-	retval = nntp_article_download_to_gstr (status, sock, group, article, abort, verbose, checkout, tmp);
-	g_string_free (tmp, TRUE);
-
+	g_string_free (setme, TRUE);
 	debug_exit ("nntp_article_download");
 	return retval;
 }
@@ -790,7 +702,7 @@ nntp_get_group_info (StatusItem       * status,
 	/* change to that group */
 	val = nntp_command_va (status, sock, &response, "GROUP %s", group_name);
 	if (val != OK_GROUP) {
-		status_item_emit_error_va (status, _("Unable to set group `%s': %s"), group_name, response);
+		status_item_emit_error_va (status, _("Unable to set group \"%s\": %s"), group_name, response);
 		return val==-1 ? TASK_FAIL : TASK_FAIL_HOPELESS;
 	}
 
@@ -854,7 +766,7 @@ nntp_download_headers (StatusItem       * status,
 	**/
 
 	if (total == 0) {
-		const gchar * n = group_get_readable_name (group);
+		const gchar * n = group_get_name (group);
 		status_item_emit_status_va (status, _("No articles found for group \"%s\""), n);
 		return TASK_SUCCESS;
 	}
@@ -877,14 +789,16 @@ nntp_download_headers (StatusItem       * status,
 	pan_g_ptr_array_reserve (setme, setme->len + (high-low));
 	while (!*abort && !((read_status = pan_socket_getline (sock, &buffer))))
 	{
-		gint part = 0;
-	        gint parts = 0;
-		const gchar * pch;
-		const gchar * s;
+		int part = 0;
+	        int parts = 0;
+		const char * pch;
+		const char * s;
 		Article * a;
-		const gchar * march = NULL;
-		const gchar * tok = NULL;
+		const char * march = NULL;
+		const char * tok = NULL;
 		int tok_len = 0;
+		const char * charset = group_get_default_charset (group);
+		char * cvt;
 
 		/* handle end-of-list */
 		if (!strncmp(buffer, ".\r\n", 3))
@@ -903,26 +817,21 @@ nntp_download_headers (StatusItem       * status,
 
 		/* get subject */
 		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
-			((gchar*)tok)[tok_len] = '\0';
-			if (!pan_header_is_8bit_encoded (tok))
-				article_init_header (a, HEADER_SUBJECT, tok, DO_CHUNK_SHARE);
-			else {
-				gchar * dec = g_mime_utils_8bit_header_decode ((const guchar*)tok);
-				article_init_header (a, HEADER_SUBJECT, dec, DO_CHUNK_SHARE);
-				g_free (dec);
-			}
+			((char*)tok)[tok_len] = '\0';
+			cvt = pan_header_to_utf8 (tok, tok_len, charset);
+			article_init_header (a, HEADER_SUBJECT, cvt, DO_CHUNK_SHARE);
+			g_free (cvt);
 		}
 
 		/* get author */
 		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
-			((gchar*)tok)[tok_len] = '\0';
-			if (is_nonempty_string(tok))
-				article_init_author_from_header (a, tok);
+			((char*)tok)[tok_len] = '\0';
+			article_init_author_from_header (a, tok);
 		}
 
 		/* get date */
 		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
-			((gchar*)tok)[tok_len] = '\0';
+			((char*)tok)[tok_len] = '\0';
 			a->date = is_nonempty_string(tok)
 				? g_mime_utils_header_decode_date (tok, NULL)
 				: (time_t)0;
@@ -930,7 +839,7 @@ nntp_download_headers (StatusItem       * status,
 
 		/* get message id */
 		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
-			((gchar*)tok)[tok_len] = '\0';
+			((char*)tok)[tok_len] = '\0';
 			if (is_nonempty_string(tok) && *tok=='<')
 				article_init_header (a, HEADER_MESSAGE_ID, tok, DO_CHUNK);
 		}
@@ -952,17 +861,18 @@ nntp_download_headers (StatusItem       * status,
 
 		/* get crossref */
 		if (get_next_token_run (march, '\t', &march, &tok, &tok_len)) {
-			((gchar*)tok)[tok_len] = '\0';
-			if (tok_len==0 || strncmp(tok,"Xref: ", 6))
-				a->crosspost_qty = 1;
-			else {
-				const gchar * foo;
-				g_strstrip ((gchar*)tok); /* remove trailing linefeed */
-				a->crosspost_qty = 0;
-				for (foo=tok+6; *foo; ++foo)
-					if (*foo==':')
-						++a->crosspost_qty;
-				article_init_header (a, HEADER_XREF, tok+6, DO_CHUNK);
+			((char*)tok)[tok_len] = '\0';
+			if (tok_len>6 && !strncmp(tok,"Xref: ", 6)) {
+				const char * header = tok + 6;
+				const char * pch;
+				int crosspost_qty = 0;
+				for (pch=header; *pch; ++pch)
+					if (*pch==':')
+						++crosspost_qty;
+				if (crosspost_qty > 1) {
+					g_strstrip ((char*)header); /* remove trailing linefeed */
+					article_init_header (a, HEADER_XREF, header, DO_CHUNK);
+				}
 			}
 		}
 
@@ -1045,7 +955,7 @@ nntp_download_headers (StatusItem       * status,
 
 		/* update the count & progress ui */
 		status_item_emit_next_step (status);
-		if (!(setme->len % 50))
+		if (!(setme->len % 100))
 			status_item_emit_status_va (status, progress_fmt, setme->len, (high-low+1));
 	}
 	g_string_free (buf, TRUE);
@@ -1063,51 +973,39 @@ nntp_download_headers (StatusItem       * status,
 ***
 **/
 
-gint
-nntp_download_bodies (StatusItem       * status,
-                      Group            * group,
-                      PanSocket        * sock,
-                      const gboolean   * abort,
-                      const GPtrArray  * const articles,
-                      gint             * index,
-                      gboolean           checkout,
-                      gboolean           abort_if_one_fails)
+int
+nntp_download_bodies (StatusItem                * status,
+                      PanSocket                 * sock,
+                      const gboolean            * abort,
+                      MessageIdentifier        ** mids,
+                      int                         mid_qty,
+                      int                       * index,
+                      gboolean                    abort_if_one_fails)
 {
-	gint result;
-	gint i;
-	gint lines_so_far;
-	gint lines_total;
+	int result;
+	int i;
 	debug_enter ("nntp_download_bodies");
 
 	/* sanity checks pt 1 */
-	g_return_val_if_fail (articles!=NULL, TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (mids!=NULL, TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (mid_qty>0, TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (index!=NULL, TASK_FAIL_HOPELESS);
 
 	/**
 	***  If we have the bodies, then just unflag 'em and maybe check them out
 	**/
 
-	for (i=*index; i<articles->len; ++i)
-		if (!article_has_body (ARTICLE(g_ptr_array_index(articles,i))))
+	/* find the first message we don't have cached */
+	for (i=*index; i<mid_qty; ++i)
+		if (!acache_has_message (mids[i]->message_id))
 			break;
 
-	if (i==articles->len) {
-		const gint qty = articles->len - *index;
-		if (qty > 0) {
-			articles_remove_flag ((Article**)articles->pdata+*index, qty, STATE_DOWNLOAD_FLAGGED);
-			if (checkout) {
-				const gchar ** message_ids = alloca (sizeof(gchar*) * qty);
-				for (i=*index; i<articles->len; ++i)
-					message_ids[i] = article_get_message_id (ARTICLE(g_ptr_array_index(articles,i)));
-				acache_file_checkout (message_ids, qty);
-			}
-		}
+	/* are we done? */
+	if (i==mid_qty)
 		return TASK_SUCCESS;
-	}
 
 	/* sanity checks pt 2 */
 	g_return_val_if_fail (status!=NULL, TASK_FAIL_HOPELESS);
-	g_return_val_if_fail (group_is_valid(group), TASK_FAIL_HOPELESS);
 	g_return_val_if_fail (sock!=NULL, TASK_FAIL);
 	g_return_val_if_fail (abort!=NULL, TASK_FAIL_HOPELESS);
 
@@ -1115,56 +1013,44 @@ nntp_download_bodies (StatusItem       * status,
 	***  Phooey, we have to download some of the bodies
 	**/
 
-	/* change to the group we want */
-	if (*abort)
-		return TASK_FAIL;
-	result = nntp_set_group (status, sock, group->name);
-	if (result != TASK_SUCCESS)
-		return result;
-
 	/* status item */
-	lines_so_far = lines_total = 0;
-	for (i=0; i<articles->len; ++i) {
-		Article * article = ARTICLE(g_ptr_array_index(articles,i));
-		lines_total += article->linecount;
-		if (i<*index)
-			lines_so_far += article->linecount;
+	if (1) {
+		gulong line_qty = 0ul;
+		gulong line_qty_so_far = 0ul;
+		for (i=0; i<mid_qty; ++i) {
+			line_qty += mids[i]->line_qty;
+			if (i<*index)
+				line_qty_so_far += mids[i]->line_qty;
+		}
+		status_item_emit_init_steps (status, line_qty);
+		status_item_emit_set_step (status, line_qty_so_far);
 	}
-	status_item_emit_init_steps (status, lines_total);
-	status_item_emit_set_step (status, lines_so_far);
 
 	/* download the articles */
-	for (; *index<articles->len && !*abort; ++(*index))
+	for (; *index<mid_qty && !*abort; ++(*index))
 	{
-		const gchar * pch;
-		Article * article = ARTICLE(g_ptr_array_index(articles,*index));
+		MessageIdentifier * mid = mids[*index];
 
-		status_item_emit_status_va (status, _("Getting %d of %d: \"%s\""),
-		                                    1+*index, articles->len,
-		                                    article_get_subject(article));
+		if (mid_qty == 1)
+			status_item_emit_status_va (status, _("Getting \"%s\""),
+							      mid->message_id);
+		else
+			status_item_emit_status_va (status, _("Getting %d of %d"),
+							      1+*index, mid_qty);
 
 		/* download if we have to. */
-		pch = article_get_message_id (article);
-		if (article_has_body (article))
+		if (acache_has_message (mid->message_id))
 		{
 			result = TASK_SUCCESS;
-			status_item_emit_inc_step (status, article->linecount);
-
-			/* since it's cached locally, we can unflag */
-			articles_remove_flag (&article, 1, STATE_DOWNLOAD_FLAGGED);
-
-			if (checkout)
-				acache_file_checkout (&article->message_id, 1);
+			status_item_emit_inc_step (status, mid->line_qty);
 		}
 		else
 		{
 			result = nntp_article_download (status,
 			                                sock,
-			                                group,
-			                                article,
+			                                mid,
 			                                abort,
-			                                NNTP_VERBOSE_NEXT_STEP,
-			                                checkout);
+			                                NNTP_VERBOSE_NEXT_STEP);
 
 			if (result!=TASK_SUCCESS && abort_if_one_fails)
 				return result;
@@ -1175,48 +1061,56 @@ nntp_download_bodies (StatusItem       * status,
 }
 
 
-gint
+int
 nntp_cancel (StatusItem     * status,
-             const Article  * article,
+             const char     * message_id,
              PanSocket      * sock)
 {
-	GString * msg;
-	const gchar * newsgroups;
-	gchar author[512];
-	gchar * author_encoded;
-	gint retval;
-	const gchar * msg_id = article_get_message_id (article);
+	const char * newsgroups;
+	const char * author;
+	int retval;
+	MessageIdentifier * mid;
+	GMimeMessage * message;
 	debug_enter ("nntp_cancel");
 
 	/* sanity checks */
 	g_return_val_if_fail (sock != NULL, TASK_FAIL);
 	g_return_val_if_fail (status != NULL, TASK_FAIL_HOPELESS);
-	g_return_val_if_fail (article_is_valid(article), TASK_FAIL_HOPELESS);
-	g_return_val_if_fail (is_nonempty_string(msg_id), TASK_FAIL_HOPELESS);
-	newsgroups = article_get_extra_header (article, HEADER_NEWSGROUPS);
-	g_return_val_if_fail (is_nonempty_string(newsgroups), TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (is_nonempty_string(message_id), TASK_FAIL_HOPELESS);
+	g_return_val_if_fail (acache_has_message(message_id), TASK_FAIL_HOPELESS);
 
-	/* let the user know what we're doing */
-	status_item_emit_status_va (status, _("Cancelling message \"%s\""), msg_id);
+	/* get info */
+	retval = TASK_FAIL;
+	mid = message_identifier_new (message_id);
+	message = acache_get_message (&mid, 1);
+	newsgroups = g_mime_message_get_header (message, HEADER_NEWSGROUPS);
+	author = g_mime_message_get_sender (message);
+	if (is_nonempty_string(newsgroups) && is_nonempty_string(author))
+	{
+		GString * msg;
 
-	/* build the message to post */
-	msg = g_string_new (NULL);
-	article_get_author_str (article, author, sizeof(author));
-	author_encoded = g_mime_utils_8bit_header_encode ((const guchar*)author);
-	g_string_sprintfa (msg, "From: %s\r\n", author_encoded);
-	g_string_sprintfa (msg, "Newsgroups: %s\r\n", newsgroups);
-	g_string_sprintfa (msg, "Subject: cancel %s\r\n", msg_id);
-	g_string_sprintfa (msg, "Control: cancel %s\r\n", msg_id);
-	g_string_sprintfa (msg, "\r\n");
-	g_string_sprintfa (msg, "Ignore\r\nArticle canceled by Pan %s\r\n", VERSION);
+		/* let the user know what we're doing */
+		status_item_emit_status_va (status, _("Canceling article"));
 
-	/* post the cancel message */
-	retval = nntp_post (status, sock, msg->str);
+		/* build the message to post */
+		msg = g_string_sized_new (1024);
+		g_string_append_printf (msg, "From: %s\r\n", author);
+		g_string_append_printf (msg, "Newsgroups: %s\r\n", newsgroups);
+		g_string_append_printf (msg, "Subject: cancel %s\r\n", message_id);
+		g_string_append_printf (msg, "Control: cancel %s\r\n", message_id);
+		g_string_append (msg, "\r\n");
+		g_string_append_printf (msg, "Ignore\r\nArticle canceled by Pan %s\r\n", VERSION);
+
+		/* post the cancel message */
+		retval = nntp_post (status, sock, msg->str);
+
+		/* cleanup */
+		g_string_free (msg, TRUE);
+	}
 
 	/* cleanup */
-	g_free (author_encoded);
-	g_string_free (msg, TRUE);
-
+	g_object_unref (message);
+	g_object_unref (mid);
 	debug_exit ("nntp_cancel");
 	return retval;
 }
