@@ -1,13 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Author: Charles Kerr <charles@rebelbase.com>
- *
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -17,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #ifndef __QUEUE_H__
@@ -28,16 +25,21 @@
 #include <pan/task.h>
 
 
-
 extern void queue_init (int max_sockets, int max_tries);
 
 extern void queue_add (Task*);
 
-extern void queue_shutdown (gboolean save_tasks);
+extern void queue_shutdown (void);
 
-extern void queue_set_paused (gboolean paused);
+extern void queue_set_online (gboolean online);
 
-extern gboolean queue_is_paused (void);
+extern gboolean queue_is_online (void);
+
+enum {
+	QUEUE_MESSAGE_ID_DOWNLOAD	= (1<<0),
+	QUEUE_MESSAGE_ID_SAVE		= (1<<1)
+};
+extern int queue_get_message_id_status (const char * message_id);
 
 
 /**
@@ -45,7 +47,6 @@ extern gboolean queue_is_paused (void);
 **/
 
 
-
 /**
 ***  CALLBACKS
 ***
@@ -53,22 +54,34 @@ extern gboolean queue_is_paused (void);
 **/
 
 /**
- * @param call_object the Task* added.
- * @param call_arg the GINT_TO_POINTER() of the position in the queue
+ * @call_obj: const char ** message_ids
+ * @call_arg: GINT_TO_POINTER(message_id_qty)
  */
-extern PanCallback *queue_task_added;
+PanCallback*  queue_get_message_id_status_changed (void);
 
 /**
- * @param call_object the Task* being removed.
- * @param call_arg the GINT_TO_POINTER() of the position in the queue
+ * @call_obj: Group**
+ * @call_arg: GINT_TO_POINTER(qty)
  */
-extern PanCallback *queue_task_removed;
+PanCallback*  queue_get_online_status_changed_callback (void);
 
 /**
- * @param call_object the Task* being repositioned.
- * @param call_arg int* to two ints: the the old and new position in the queue
+ * @param call_object GSList* of Task*s which were added
+ * @param call_arg the GINT_TO_POINTER() of the position in the queue where the first was added
  */
-extern PanCallback *queue_task_moved;
+extern PanCallback *queue_tasks_added;
+
+/**
+ * @param call_object GSList* of Task*s which were removed
+ * @param call_arg NULL
+ */
+extern PanCallback *queue_tasks_removed;
+
+/**
+ * @param call_object GSList of Task*s that were moved.
+ * @param call_arg the index of the first Task in the list; the remaining tasks are at index+1, index+2, and so on.
+ */
+extern PanCallback *queue_tasks_moved;
 
 extern PanCallback *queue_task_status_changed;
 
@@ -86,36 +99,41 @@ extern PanCallback *queue_max_tries_changed;
  */
 extern GPtrArray* queue_get_tasks (void);
 
-/**
- * Add an item to the queue.  This increases the task's refcount by 1.
- * @param task the task to insert.
- * @param index into the queue, or -1 to add at the end.
- */
-extern void queue_task_insert (Task*, int index);
+extern gboolean queue_is_empty (void);
 
 /**
- * Remove an item from the queue.  This decreases the task's refcount by 1.
- * @param task the task to remove
+ * Add an item to the queue.  This increases the task's refcount by 1.
+ * @param tasks the tasks to insert.  queue assumes ownership of the list.
+ * @param index into the queue, or -1 to add at the end.
  */
-extern void queue_task_remove (Task*);
+extern void queue_insert_tasks (GSList * tasks, int index);
+
+/**
+ * Aborts a running task, has no effect on non-running tasks.
+ * @param list of tasks to abort.  queue assumes ownership of the list.
+ */
+extern void queue_abort_tasks (GSList * tasks);
+
+/**
+ * Remove an item from the queue.  This decreases each task's refcount by 1.
+ * @param tasks the tasks to remove.  queue assumes ownership of the list.
+ */
+extern void queue_remove_tasks (GSList * tasks);
 
 /**
  * Reposition an item in the queue.
- * @param task the task to move
+ * @param tasks the tasks to move
  * @param index the new position in the queue, or -1 for the end.
  */
-extern void queue_task_move (Task*, int index);
+extern void queue_move_tasks (GSList * tasks, int index);
 
 /**
  * Requeue a failed task.
- * @param task
- * @index the new position in the queue, or -1 for the end.
+ * @param tasks the tasks to requeue
  */
-extern void queue_task_requeue_failed (Task*, int index);
+extern void queue_requeue_failed_tasks (GSList * tasks);
 
 
-extern void queue_task_abort (Task*);
-
 /**
 ***  CONNECTION
 **/
@@ -145,7 +163,8 @@ typedef enum
 	QUEUE_TASK_STATUS_RUNNING	= 2,
 	QUEUE_TASK_STATUS_FAILED	= 4,
 	QUEUE_TASK_STATUS_ABORTING	= 5,
-	QUEUE_TASK_STATUS_CONNECTING	= 6
+	QUEUE_TASK_STATUS_REMOVING	= 6,
+	QUEUE_TASK_STATUS_CONNECTING	= 7
 }
 QueueTaskStatus;
 
@@ -157,5 +176,7 @@ extern QueueTaskStatus queue_get_task_status (const Task*);
 
 guint queue_get_running_task_count (void);
 
+char* queue_get_tasks_filename (void);
+
 
 #endif /* __QUEUE_H__ */
