@@ -1,11 +1,11 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,14 +15,13 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 #ifndef __TASK_BODIES_H_
 #define __TASK_BODIES_H_
 
-#include "task.h"
-#include "group.h"
+#include <pan/base/group.h>
+#include <pan/task.h>
 
 #define TASK_BODIES(obj) ((TaskBodies*)(obj))
 
@@ -31,9 +30,7 @@ typedef struct
 	/* parent class */
 	Task           task;
 
-	/* private, passed in ctor */
-	Group        * group;
-	GPtrArray    * articles;
+	/* private */
 	int            working_index;
 }
 TaskBodies;
@@ -42,8 +39,12 @@ TaskBodies;
 ***  PUBLIC
 **/
 
-PanObject*   task_bodies_new   (Group             * group,
-                                const GPtrArray   * articles);
+PanObject*   task_bodies_new                (Server              * server,
+                                             MessageIdentifier  ** mids,
+                                             int                   mid_qty);
 
 
+PanObject*   task_bodies_new_from_articles  (const Article      ** articles,
+                                             int                   article_qty);
+
 #endif /* __TASK_BODY__H__ */
