@@ -1,11 +1,11 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 /*********************
@@ -91,7 +90,7 @@ task_group_count_destructor (PanObject * o )
 
 PanObject*
 task_group_count_new   (Group      ** groups,
-	                  guint         qty)
+                        guint         qty)
 {
 	TaskGroupCount * t = NULL;
 
@@ -105,10 +104,10 @@ task_group_count_new   (Group      ** groups,
 
 	/* initialize the parent class */
 	task_constructor (TASK(t),
-		task_group_count_destructor,
-		task_group_count_describe,
-		task_group_count_run,
-		groups[0]->server, TRUE, TRUE);
+	                  task_group_count_destructor,
+	                  task_group_count_describe,
+	                  task_group_count_run,
+	                  groups[0]->server, TRUE, TRUE);
 
 	/* initialize the task-bodies */
 	t->groups = g_ptr_array_new ();
@@ -127,7 +126,7 @@ task_group_count_describe (const StatusItem * status)
 	const TaskGroupCount * t = TASK_GROUP_COUNT(status);
 
 	return g_strdup_printf (
-		_("Updating article counts for %u groups from server `%s'"),
+		_("Updating article counts for %u groups from server \"%s\""),
 	       t->groups->len,
 	       TASK(t)->server->name);
 }
@@ -174,8 +173,8 @@ task_group_count_run (Task * task)
 		else
 		{
 			status_item_emit_error_va (STATUS_ITEM(task),
-			                           _("Unable to update counts for group `%s'"),
-			                           group_get_readable_name(g));
+			                           _("Unable to update counts for group \"%s\""),
+			                           group_get_name(g));
 		}
 	}
 
