@@ -1,11 +1,11 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 /*********************
@@ -130,23 +129,26 @@ task_grouplist_destructor (PanObject * o)
 	task_destructor (o);
 }
 
-static gchar*
+static char*
 task_grouplist_describe (const StatusItem* item)
 {
-	const gchar* action = NULL;
+	int type;
+	char * retval;
+	const char * name;
 
 	/* sanity checks */
 	g_return_val_if_fail (item!=NULL, NULL);
+	type = TASK_GROUPLIST(item)->download_type;
+	name = server_get_name (TASK_GROUPLIST(item)->server);
 
-	switch (TASK_GROUPLIST(item)->download_type) {
-		case GROUPLIST_ALL: action=_("all groups"); break;
-		case GROUPLIST_NEW: action=_("new groups"); break;
-		default: pan_warn_if_reached(); action=_("BUG IN CODE"); break;
-	}
+	if (type == GROUPLIST_ALL)
+		retval = g_strdup_printf (_("Getting all groups from server \"%s\""), name);
+	else if (type == GROUPLIST_NEW)
+		retval = g_strdup_printf (_("Getting new groups from server \"%s\""), name);
+	else
+		retval = g_strdup ("BUG!");
 
-	return g_strdup_printf ( _("Getting %s from %s"),
-		action,
-		server_get_name (TASK_GROUPLIST(item)->server) );
+	return retval;
 }
 
 static gint
@@ -156,10 +158,14 @@ task_grouplist_run (Task *item)
 	Server *server = item->server;
 	gint result;
 
-	status_item_emit_status_va (STATUS_ITEM(item),
-		_("Got %u groups for \"%s\""),
-		item_g->groups->len,
-		server_get_name (server));
+	if (1) {
+		char buf[32];
+		g_snprintf (buf, sizeof(buf), "%lu", (gulong)item_g->groups->len);
+		status_item_emit_status_va (STATUS_ITEM(item),
+			_("Got %s groups from server \"%s\""),
+			buf,
+			server_get_name (server));
+	}
 
 	/**
 	***  Try to get the groups from the server
@@ -187,7 +193,7 @@ task_grouplist_run (Task *item)
 	**/
 
 	status_item_emit_status_va (STATUS_ITEM(item),
-		_("Adding groups to \"%s\""), server_get_name (server));
+		_("Adding groups to server \"%s\""), server_get_name (server));
 
 	/* for the next time we call GROUPLIST_NEW... */
 	server->last_newgroup_list_time = time(0);
@@ -215,7 +221,7 @@ compare_name_ppgroup (const void * a, const void * b)
 }
 
 static int
-compare_ppgroup_ppgroup (const void * a, const void * b)
+compare_ppgroup_ppgroup (gconstpointer a, gconstpointer b, gpointer unused)
 {
 	return strcmp ((*(const Group**)a)->name,
 	               (*(const Group**)b)->name);
@@ -236,8 +242,8 @@ nntp_grouplist_download_all (TaskGroupList * item)
 	StatusItem * status = STATUS_ITEM(item);
 
 	status_item_emit_status_va (status,
-		_("Got %u groups for \"%s\""),
-		groups->len,
+		_("Got %d groups from server \"%s\""),
+		(int)groups->len,
 		server_get_name (server));
 
 	/**
@@ -282,9 +288,9 @@ nntp_grouplist_download_all (TaskGroupList * item)
 			if (groups->len % 256 == 0)
 			{
 				char buf[32];
-				commatize_ulong ((gulong)groups->len, buf);
+				g_snprintf (buf, sizeof(buf), "%lu", (gulong)groups->len);
 				status_item_emit_status_va (status,
-					_("Got %s groups for \"%s\""),
+					_("Got %s groups from server \"%s\""),
 					buf, server_get_name (server));
 			}
 
@@ -305,10 +311,7 @@ nntp_grouplist_download_all (TaskGroupList * item)
 	}
 
 	/* sort so that we can bsearch them... */
-	qsort (groups->pdata,
-	       groups->len,
-	       sizeof(Group*),
-	       compare_ppgroup_ppgroup);
+	g_ptr_array_sort_with_data (groups, compare_ppgroup_ppgroup, NULL);
 
 
 	/**
@@ -358,10 +361,10 @@ nntp_grouplist_download_all (TaskGroupList * item)
 		if (description_count % 43 == 0)
 		{
 			char buf[32];
-			commatize_ulong ((gulong)description_count, buf);
+			g_snprintf (buf, sizeof(buf), "%d", description_count);
 			status_item_emit_status_va (
 				STATUS_ITEM(item),
-				_("Got %s descriptions for \"%s\""),
+				_("Got %s descriptions from server \"%s\""),
 				buf, server_get_name (server));
 		}
 
@@ -391,7 +394,7 @@ nntp_grouplist_download_all (TaskGroupList * item)
 	if (read_err)
 		return TASK_FAIL;
 
-	log_add_va (LOG_INFO, _("Got %d groups for server `%s'"),
+	log_add_va (LOG_INFO, _("Got %d groups from server \"%s\""),
 		groups->len,
 		server_get_name (server));
 
@@ -420,10 +423,11 @@ nntp_grouplist_download_new (TaskGroupList * item)
 	Group * group = NULL;
 
 	status_item_emit_status_va (STATUS_ITEM(item),
-		_("Retrieving 0 new groups for \"%s\""), server_get_name (server));
+		_("Getting %d new groups from server \"%s\""),
+		0, server_get_name (server));
 
 	/* get the date of the last grouplist update */
-	gmtime_r (&server->last_newgroup_list_time, &gmt);
+	pan_gmtime_r (&server->last_newgroup_list_time, &gmt);
 	strftime (datestr, sizeof(datestr), "%Y%m%d %H%M%S GMT", &gmt);
 
 	/* ask the server which groups have been added since then */
@@ -467,7 +471,7 @@ nntp_grouplist_download_new (TaskGroupList * item)
 			status_item_emit_progress (STATUS_ITEM(item), count);
 
 			status_item_emit_status_va (STATUS_ITEM(item),
-				_("Retrieving %d new groups for \"%s\""), 
+				_("Getting %d new groups from server \"%s\""), 
 				count, server_get_name (server));
 		}
 
@@ -476,10 +480,12 @@ nntp_grouplist_download_new (TaskGroupList * item)
 		}
 	}
 
-	log_add_va (LOG_INFO, _("Got %d new groups"), count);
-
-	status_item_emit_status_va (STATUS_ITEM(item),
-		_("Got %d news groups for \"%s\""), count, server_get_name (server));
+	if (1) {
+		char buf[512];
+		g_snprintf (buf, sizeof(buf), _("Got %d new groups from server \"%s\""), count, server_get_name(server));
+		log_add (LOG_INFO, buf);
+		status_item_emit_status (STATUS_ITEM(item), buf);
+	}
 
 	return TASK_SUCCESS;
 }
