@@ -1,11 +1,11 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Pan - A Newsreader for X
- * Copyright (C) 1999, 2000, 2001  Pan Development Team <pan@rebelbase.com>
+ * Pan - A Newsreader for Gtk+
+ * Copyright (C) 2002  Charles Kerr <charles@rebelbase.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
+ * the Free Software Foundation; version 2 of the License.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -15,7 +15,6 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- * 
  */
 
 /*********************
@@ -24,19 +23,23 @@
 
 #include <config.h>
 
+#include <stdio.h>
+#include <errno.h>
+
 #include <glib.h>
 
 #include <pan/base/base-prefs.h>
 #include <pan/base/pan-glib-extensions.h>
-#include <pan/base/util-file.h>
+#include <pan/base/pan-i18n.h>
 
-#include <pan/prefs.h>
 #include <pan/thread-watch.h>
 
 /*********************
 **********************  Defines / Enumerated types
 *********************/
 
+#define WATCHED_THREAD_SPAN_DAYS 31
+
 /*********************
 **********************  Macros
 *********************/
@@ -83,10 +86,10 @@ static guint hash_size = 0;
 ******
 *****/
 
-static gchar*
+static char*
 get_filename (void)
 {
-	return g_strdup_printf ("%s%cthreads.txt", data_dir, G_DIR_SEPARATOR);
+	return g_strdup_printf ("%s%cthreads.txt", get_data_dir(), G_DIR_SEPARATOR);
 }
 
 /*****
@@ -107,10 +110,10 @@ thread_init_module (void)
 	{
 		const time_t current_time = time(0);
 		RecordStruct * data;
-		gchar * filename;
-		GArray * file;
-		gchar ** v;
-		gint i;
+		char * filename;
+		char * file;
+		char ** v;
+		int i;
 
 		/* create the hashtable */
 		hash = g_hash_table_new (g_str_hash, g_str_equal);
@@ -119,15 +122,15 @@ thread_init_module (void)
 
 		/* load the file */
 		filename = get_filename ();
-		file = read_file (filename);
-		v = file==NULL ? NULL : g_strsplit (file->data, "\n", -1);
+		g_file_get_contents (filename, &file, NULL, NULL);
+		v = file==NULL ? NULL : g_strsplit (file, "\n", -1);
 		g_hash_table_freeze (hash);
 		for (i=0; v!=NULL && v[i]!=NULL; ++i)
 		{
 			int state;
 			time_t start_time;
-			gchar * id;
-			const gchar * march = v[i];
+			char * id;
+			const char * march = v[i];
 			time_t age_secs;
 
 			/* skip comments */
@@ -141,7 +144,7 @@ thread_init_module (void)
 
 			/* age out threads > one month old */
 			age_secs = current_time - start_time;
-			if ((age_secs/86400) > watched_thread_span_days) {
+			if ((age_secs/86400) > WATCHED_THREAD_SPAN_DAYS) {
 				dirty = TRUE;
 				g_free (id);
 				continue;
@@ -158,8 +161,7 @@ thread_init_module (void)
 
 		/* cleanup */
 		g_free (filename);
-		if (file != NULL)
-			g_array_free (file, TRUE);
+		g_free (file);
 		g_strfreev (v);
 	}
 }
@@ -172,10 +174,10 @@ thread_init_module (void)
 *************  PUBLIC
 ************/
 
-gint
+int
 pan_thread_get_state (const Article * a)
 {
-	gint retval = THREAD_NOTHING;
+	int retval = THREAD_NOTHING;
 
 	/* sanity clause */
 	g_return_val_if_fail(a, retval);
@@ -189,7 +191,7 @@ pan_thread_get_state (const Article * a)
 	{
 		if (a != NULL)
 		{
-			gchar * id = article_get_thread_message_id (a);
+			char * id = article_get_thread_message_id (a);
 
 			if (id != NULL)
 			{
@@ -212,7 +214,7 @@ pan_thread_get_state (const Article * a)
 void
 pan_thread_set_state (const Article * a, int state)
 {
-	gchar * id;
+	char * id;
 
 	/* sanity checks */
 	g_return_if_fail (article_is_valid(a));
@@ -282,7 +284,7 @@ thread_write_line (gpointer key, gpointer value, gpointer user_data)
 	const RecordStruct * data = (const RecordStruct*) value;
 	if (data->state != THREAD_NOTHING)
 	{
-		const gchar * id = (const gchar*) key;
+		const char * id = (const char*) key;
 		FILE * fp = (FILE*)user_data;
 
 		fprintf (fp, "%d,%lu,%s\n",
@@ -298,15 +300,24 @@ thread_shutdown_module (void)
 	if (dirty)
 	{
 		/* write the thread states to disk */
-		gchar * filename = get_filename ();
-		FILE * fp = fopen (filename, "w+");
-		fprintf (fp, "# one record per line, comma-delimited fields\n");
-		fprintf (fp, "# each record has three fields:\n");
-		fprintf (fp, "#    state (0=nothing,1=watched,2=ignored)\n");
-		fprintf (fp, "#    when this state was set, in secs since Jan 1, 1970.  These states expire after 1 month.\n");
-		fprintf (fp, "#    message-id of parent thread\n");
-		g_hash_table_foreach (hash, thread_write_line, fp);
-		fclose (fp);
+		char * filename = get_filename ();
+		FILE * fp = fopen (filename, "w");
+
+		if (fp != NULL)
+		{
+			fprintf (fp, "# one record per line, comma-delimited fields\n");
+			fprintf (fp, "# each record has three fields:\n");
+			fprintf (fp, "#    state (0=nothing,1=watched,2=ignored)\n");
+			fprintf (fp, "#    when this state was set, in secs since Jan 1, 1970.  These states expire after 1 month.\n");
+			fprintf (fp, "#    message-id of parent thread\n");
+			g_hash_table_foreach (hash, thread_write_line, fp);
+			fclose (fp);
+		}
+		else
+		{
+			g_warning (_("Could not open \"%s\" for writing: %s\n"),
+				filename, g_strerror(errno));
+		}
 		g_free (filename);
 		dirty = FALSE;
 	}
