@@ -5,6 +5,7 @@
 
 #include <glib.h>
 
+#include <gmime/gmime.h>
 #include <gmime/md5-utils.h>
 
 #include <pan/base/acache.h>
@@ -30,7 +31,7 @@ check_md5 (const gchar * filename, const guchar * expected_sum)
 
 	/* get the md5sum */
 	full_filename = g_strdup_printf ("%s/%s", download_dir, filename);
-	pan_normalize_filename_inplace (full_filename);
+	pan_file_normalize_inplace (full_filename);
 	g_message ("md5 check for `%s'", full_filename);
 	md5_get_digest_from_file (full_filename, sum);
 	for (i=0; i<16; ++i)
@@ -53,13 +54,15 @@ check_md5 (const gchar * filename, const guchar * expected_sum)
 static void
 init_pan (void)
 {
-	gchar * base;
-	gchar * data_dir;
+	char * base;
+	char * data_dir;
 
 	base = getenv ("srcdir");
-	base = g_strdup_printf ("%s/", base!=NULL ? base : getenv("PWD"));
-	data_dir = g_strdup_printf ("%s/data/", base);
-	base_prefs_init (data_dir, g_get_tmp_dir(), ">", 100, FALSE, FALSE);
+	base = g_strdup_printf ("%s", base!=NULL ? base : getenv("PWD"));
+	data_dir = g_strdup_printf ("PAN_HOME=%s/data/", base);
+	putenv (data_dir);
+	base_prefs_init (g_get_tmp_dir(), ">", 100, FALSE, FALSE);
+	g_message ("data_dir = `%s'", get_data_dir());
 
 	g_free (data_dir);
 }
@@ -72,29 +75,24 @@ static void
 decode_from_message_ids (const char ** message_ids, int parts)
 {
 	decode_data dd;
-	GSList * sl = NULL;
+	MessageIdentifier ** mids;
 	guint i;
 
+       	mids = g_alloca (sizeof(MessageIdentifier*) * parts);
 	for (i=0; i<parts; ++i)
-	{
-		Article * article = article_new (group);
-		article_set_header (article, HEADER_SUBJECT, message_ids[i], DO_CHUNK);
-		article_set_header (article, HEADER_MESSAGE_ID, message_ids[i], DO_CHUNK);
-		article->part = i+1;
-		article->parts = parts;
-		article->number = 1;
-		sl = g_slist_append (sl, article);
-	}
+		mids[i] = message_identifier_new (message_ids[i]);
 	
 	dd.server = server;
-	dd.group = group;
-	dd.articles = sl;
+	dd.subject = message_ids[0];
+	dd.mids = mids;
+	dd.mid_qty = parts;
 	dd.path = download_dir;
 	dd.filename = NULL;
 	dd.item = status;
 	decode_article (&dd);
-	g_slist_free (sl);
-	sl = NULL;
+
+	for (i=0; i<parts; ++i)
+		g_object_unref (mids[i]);
 }
 
 extern int server_saving_enabled;
@@ -104,8 +102,12 @@ main (void)
 {
 	const char * message_ids [64];
 	int parts = 0;
+	GTimeVal start;
+	GTimeVal finish;
+	double diff;
 
 	g_thread_init (NULL);
+	g_mime_init (0);
 	init_pan ();
 	server_saving_enabled = FALSE;
 
@@ -117,26 +119,20 @@ main (void)
 	server_add_groups (server, &group, 1, NULL, NULL);
 	status = status_item_new_with_description ("decoding regression tests");
 
+	g_usleep (G_USEC_PER_SEC);
+	g_get_current_time (&start);
+
 	/**
-	***  Yenc Attachment
+	*** "Cut Here"
+	*** I hate those posts.
 	**/
 
 	parts = 0;
-	message_ids[parts++] = "<yenc>";
+	message_ids[parts++] = "<noisy_1_2>";
+	message_ids[parts++] = "<noisy_2_2>";
 	decode_from_message_ids (message_ids, parts);
-	check_md5 ("Jungle ha itsumo Hale nochi Guu - OST.txt", "e3e458125877636a58fca66f47cd9bac");
-	g_message ("PASS: yenc stuff bug");
-
-	/**
-	***  Multipart yenc
-	**/
-
-	parts = 0;
-	message_ids[parts++] = "<yenc_1_2>";
-	message_ids[parts++] = "<yenc_2_2>";
-	decode_from_message_ids (message_ids, parts);
-	check_md5 ("joystick.jpg", "3331360a7976e76a9fb57eb4be23e4a3");
-	g_message ("PASS: multi part yenc");
+	check_md5 ("chessboard.gif", "9fa22e8183029f23bff2a85da3c2a034");
+	g_message ("PASS: noisy message with `cut here' and end-of-message sig noise");
 
 	/**
 	***  Two UU Attachments
@@ -149,6 +145,17 @@ main (void)
 	check_md5 ("foo_02.txt", "9c22e248d30256891e2f0d61b67bc287");
 	g_message ("PASS: two uu attachments in message");
 
+	/**
+	***  Multipart yenc
+	**/
+
+	parts = 0;
+	message_ids[parts++] = "<yenc_1_2>";
+	message_ids[parts++] = "<yenc_2_2>";
+	decode_from_message_ids (message_ids, parts);
+	check_md5 ("joystick.jpg", "3331360a7976e76a9fb57eb4be23e4a3");
+	g_message ("PASS: multi part yenc");
+
 	/**
 	***  Mime Attachment
 	**/
@@ -160,6 +167,16 @@ main (void)
 	check_md5 ("The-Only-Answer-1.JPG", "857f7f9e1fce68414cdf6a110a4ad2d2");
 	g_message ("PASS: two mime attachments in message");
 
+	/**
+	***  UUEncoded,  in Filename
+	**/
+
+	parts = 0;
+	message_ids[parts++] = "<micro>";
+	decode_from_message_ids (message_ids, parts);
+	check_md5 ("micro.txt", "39c14a46362de0f4764faef78e1a77b7");
+	g_message ("PASS: filename with ");
+
 	/**
 	***  Mime Attachment with x-uuencode encoding
 	**/
@@ -170,6 +187,16 @@ main (void)
 	check_md5 ("firstspacetrip1024.jpg", "d3ac2f5a4b4b344565547af23f592fda");
 	g_message ("PASS: mime message with Content-Transfer-Encoding: x-uuencode");
 
+	/**
+	***  Yenc Attachment
+	**/
+
+	parts = 0;
+	message_ids[parts++] = "<yenc>";
+	decode_from_message_ids (message_ids, parts);
+	check_md5 ("Jungle ha itsumo Hale nochi Guu - OST.txt", "e3e458125877636a58fca66f47cd9bac");
+	g_message ("PASS: yenc stuff bug");
+
 	/**
 	***  Filename collision - should rename the second one
 	**/
@@ -182,16 +209,6 @@ main (void)
 	check_md5 ("collision_3", "25246baff3f46d51e18d6bbb5398560e");
 	g_message ("PASS: filenamename collision avoided");
 
-	/**
-	***  UUEncoded,  in Filename
-	**/
-
-	parts = 0;
-	message_ids[parts++] = "<micro>";
-	decode_from_message_ids (message_ids, parts);
-	check_md5 ("micro.txt", "39c14a46362de0f4764faef78e1a77b7");
-	g_message ("PASS: filename with ");
-
 	/**
 	***  UUEncoded, in Apostrophe Filename
 	**/
@@ -202,18 +219,6 @@ main (void)
 	check_md5 ("ap'str'ph'.txt", "ddcfb039ae385314aa5d8155196bb550");
 	g_message ("PASS: filenames with apostrophes");
 
-	/**
-	*** "Cut Here"
-	*** I hate those posts.
-	**/
-
-	parts = 0;
-	message_ids[parts++] = "<noisy_1_2>";
-	message_ids[parts++] = "<noisy_2_2>";
-	decode_from_message_ids (message_ids, parts);
-	check_md5 ("chessboard.gif", "9fa22e8183029f23bff2a85da3c2a034");
-	g_message ("PASS: noisy message with `cut here' and end-of-message sig noise");
-
 	/**
 	***  This one's so noisy there's even crap INSIDE the encoded text
 	**/
@@ -229,6 +234,9 @@ main (void)
 	***  Done
 	**/
 
-	g_message ("All decode tests passed");
+	g_get_current_time (&finish);
+	diff = finish.tv_sec - start.tv_sec;
+	diff += (finish.tv_usec - start.tv_usec)/(double)G_USEC_PER_SEC;
+	g_message ("All decode tests passed in %.3f seconds", diff);
 	return 0;
 }
