@@ -2,7 +2,8 @@
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
-#include <pan/pan-glib-extensions.h>
+
+#include <pan/base/pan-glib-extensions.h>
 
 static gint
 compare_pint_pint_by_value (const void * va, const void *vb)
@@ -41,19 +42,6 @@ int main (void)
 		g_message ("PASS pan_strcmp");
 	}
 
-	/* pan-stristr */
-	++num;
-	if (1)
-	{
-		gchar * a = "abcdefg";
-		gchar * b = "BCD";
-		if (pan_stristr(a,b) != a+1) return num;
-		if (pan_stristr(a, NULL) != NULL) return num;
-		if (pan_stristr(NULL, b) != NULL) return num;
-		if (pan_stristr(NULL, NULL) != NULL) return num;
-		g_message ("PASS pan_stristr");
-	}
-
 	/* string-ends-with */
 	++num;
 	if (1)
@@ -256,6 +244,51 @@ int main (void)
 		g_message ("PASS pan_hash_to_g_ptr_array");
 	}
 
+
+	/* get_next_token_g_str */
+	++num;
+	if (1)
+	{
+		GString * str = g_string_new (NULL);
+		const gchar * string;
+		const gchar * march;
+		gint status;
+
+		/* test empty string */
+		string = NULL;
+		march = NULL;
+		status = get_next_token_g_str (NULL, '\n',  &march, str);
+		if (status != FALSE) return num;
+		g_message ("PASS get_next_token_g_str 1");
+
+		/* test empty string 2 */
+		string = "";
+		march = NULL;
+		status = get_next_token_g_str (NULL, '\n',  &march, str);
+		if (status != FALSE) return num;
+		g_message ("PASS get_next_token_g_str 2");
+
+		/* test tokens */
+		march = string = "foo\n bar\nmum ";
+		status = get_next_token_g_str (march, '\n', &march, str);
+		if (status == FALSE) return num;
+		if (pan_strcmp (str->str, "foo")) return num;
+		status = get_next_token_g_str (march, '\n', &march, str);
+		if (status == FALSE) return num;
+		if (pan_strcmp (str->str, " bar")) return num;
+		status = get_next_token_g_str (march, '\n', &march, str);
+		if (status == FALSE) return num;
+		if (pan_strcmp (str->str, "mum ")) return num;
+		status = get_next_token_g_str (march, '\n', &march, str);
+		if (status != FALSE) return num;
+		g_message ("PASS get_next_token_g_str 3");
+
+		/* cleanup */
+		g_string_free (str, TRUE);
+		g_message ("PASS get_next_token_g_str");
+	}
+
+
 	/* skip next token */
 	++num;
 	if (1)
@@ -384,6 +417,7 @@ int main (void)
 		if (pan_strcmp(get_next_token_str(march, ' ', &march),"1")) return num;
 		if (pan_strcmp(get_next_token_str(march, ' ', &march),"\t")) return num;
 		if (pan_strcmp(get_next_token_str(march, ' ', &march),"2")) return num;
+		if (pan_strcmp(get_next_token_str(march, ' ', &march),NULL)) return num;
 		g_message ("PASS get_next_token_str");
 
 		string = "Foo\t\t\t10\t2112";
@@ -393,6 +427,7 @@ int main (void)
 		if (pan_strcmp(get_next_token_str(march, '\t', &march),"")) return num;
 		if (pan_strcmp(get_next_token_str(march, '\t', &march),"10")) return num;
 		if (pan_strcmp(get_next_token_str(march, '\t', &march),"2112")) return num;
+		if (pan_strcmp(get_next_token_str(march, '\t', &march),NULL)) return num;
 		g_message ("PASS get_next_token_str 2");
 	}
 
@@ -581,37 +616,6 @@ int main (void)
 		g_message ("PASS pan_g_string_strstrip");
 	}
 
-	/* pan_g_string_append_len */
-	++num;
-	if (1)
-	{
-		GString * s = g_string_new (NULL);
-
-		/* add */
-		pan_g_string_append_len (s, "abc", 3);
-		if (pan_strcmp(s->str,"abc")) return num;
-
-		/* append */
-		pan_g_string_append_len (s, "def", 3);
-		if (pan_strcmp(s->str,"abcdef")) return num;
-
-		/* append short */
-		pan_g_string_append_len (s, "ghijklm", 3);
-		if (pan_strcmp(s->str,"abcdefghi")) return num;
-
-		/* the number's too big */
-		pan_g_string_append_len (s, "j", 3);
-		if (pan_strcmp(s->str,"abcdefghij")) return num;
-
-		/* feed it trash & don't crash */
-		pan_g_string_append_len (NULL, "abc", 2);
-		pan_g_string_append_len (NULL, NULL, 2);
-		pan_g_string_append_len (s, NULL, 2);
-
-		g_string_free (s, TRUE);
-		g_message ("PASS pan_g_string_append_len");
-	}
-
 	/* lower bound */
 	++num;
 	if (1)
