@@ -1,25 +1,23 @@
-#include <pan/pan-callback.h>
+#include <pan/base/pan-callback.h>
 
 static int cb_calls = 0;
 static PanCallback * cb = NULL;
 
-static gint
+static void
 callback_1 (gpointer a, gpointer b, gpointer c)
 {
 	g_assert (a == (gpointer)0xF00F);
 	g_assert (c == GINT_TO_POINTER(1));
 	++cb_calls;
-	return 0;
 }
 
-static gint
+static void
 callback_3 (gpointer a, gpointer b, gpointer c)
 {
 	++cb_calls;
-	return 0;
 }
 
-static gint
+static void
 callback_2 (gpointer a, gpointer b, gpointer c)
 {
 	g_assert (a == (gpointer)0xF00F);
@@ -27,7 +25,6 @@ callback_2 (gpointer a, gpointer b, gpointer c)
 	++cb_calls;
 	pan_callback_remove (cb, callback_3, GINT_TO_POINTER(3));
 	pan_callback_remove (cb, callback_3, GINT_TO_POINTER(4));
-	return 0;
 }
 
 int
